#!/bin/sh
version=6.0.1
libversion=0
bindir=/usr/bin
mandir=/usr/man
osname=`uname -s`
if test -z "$1" ; then
   installdir=/usr/local
else
   installdir="/$1"
fi
logfile=$installdir/share/TFORMer/install.log

umask 022
cd `dirname $0`/usr/local

create_link()
{
	echo "$2 # link to $1" >> $logfile
	ln -sf $1 $2 2> /dev/null
}

copy_dir()
{
	cp -R $1 $installdir
	find $1 -type f | awk -F "#" '{ print "$installdir/" $1 }' >> $logfile
}

CheckFiles()
{
  if test ! -d include
  then
    echo "Error: Could not find directory 'include'!"
    return 1;
  fi
   
  if test ! -d lib
  then
    echo "Error: Could not find directory 'lib'!"
    return 1;
  fi
  
  if test ! -d share
  then
    echo "Error: Could not find directory 'share'!"
    return 1;
  fi
  
  return 0;
}

InstallFiles()
{
  if test ! -d $installdir
  then
		mkdir -p $installdir
  fi

	mkdir -p `dirname $logfile`
	echo "# Installation log (" `date` ")" >> $logfile

  echo "Copying TFORMer files..."
	copy_dir share

  echo "Copying include files..."
	copy_dir include

  echo "Copying libraries..."
  copy_dir lib
}

PostInstallationProcedure()
{
	if test -f $installdir/lib/libTFORMer6.so.$libversion
	then
		echo "Registering TFORMer Library..."
		create_link $installdir/lib/libTFORMer6.so.$libversion $installdir/lib/libTFORMer6.so
	fi

	if test -f $installdir/lib/libTFORMer60JNI.so.$libversion
	then
		echo "Registering TFORMer JNI Library..."
		create_link $installdir/lib/libTFORMer60JNI.so.$libversion $installdir/lib/libTFORMer60JNI.so
	fi
  
  echo "Creating link for TFORMer executable..."
  create_link $installdir/share/TFORMer/tfprint $bindir/tfprint
  if test $? -ne 0
  then
    echo "Warning: Could not create link to TFORMer in $bindir !"
  fi

  echo "Creating link for TFORMer manual..."
  create_link $installdir/share/man/man1/tfprint.1 $mandir/man1
  if test $? -ne 0
  then
    echo "Warning: Could not create link to TFORMer manual in $mandir !"
  fi
  
  echo "Setting file permissions..."
  chmod a+r $installdir/share/TFORMer/*
  chmod a+x $installdir/share/TFORMer/*.sh
  chmod a+x $installdir/share/TFORMer/tfprint
  if test $? -ne 0
  then
    echo "Error: chmod failed!"
    return 1;
  fi
  return 0;
}

LICENSE_FILE=tecit_license_agreement.txt

echo " TFORMer for $osname - Installation"
echo " ---------------------------------"
echo ""

# Display license agreeement
if test -e $LICENSE_FILE
  then
  cat $LICENSE_FILE | more
  agreed=""
  while [ -z "$agreed" ]; do
    echo ""
    echo "Do you agree to the above license terms? [yes or no]"
    read reply leftover
    case $reply in
      [yY] | [yY][eE][sS])
         agreed=1
         ;;
      [nN] | [nN][oO])
         echo "If you don't agree to the license you can't install this software"
         exit 1
         ;;
     esac
   done
else
  echo "License agreement $LICENSE_FILE not found!"
  exit 1;
fi

if CheckFiles && InstallFiles && PostInstallationProcedure
then    
  echo "Installation finished."
  echo "Do you want to print a test case using the newly installed TFORMer Runtime? [y or n]"
  read reply
  case $reply in
    [yY] | [yY][eE][sS])
      sh /usr/local/share/TFORMer/Demos/DemoBarcodeLabels.sh
      ;;
  esac
  exit 0
else
  echo "Installation aborted."
  exit 1
fi

