/*===========================================================================*/
/*                         Copyright (C) 2006-2009 by                        */
/*                                                                           */
/*                    -- TEC-IT Datenverarbeitung GmbH --                    */
/*   -- team for engineering and consulting in information technologies --   */
/*                                                                           */
/*                           All rights reserved.                            */
/*                                                                           */
/*              This is a part of the TEC-IT Standard Software.              */
/*                                                                           */
/*    This source code is only intended as a supplement to the References    */
/*      and related electronic documentation provided with the product.      */
/*                                                                           */
/*     See these sources for detailed information regarding this product     */
/*===========================================================================*/


///////////////////////////////////////////////////////////////////////////////
/// \file TFormer6.h
/// Header File for TFormer6.0\n
/// Include this header file in your modules to access the <em>TFORMer Runtime</em> API.
///
/// \attention
/// Do not forget to define TECIT_DLLIMPORT before including this header file!
///
/// \code
/// #define TECIT_DLLIMPORT
/// #include "TFormer6.h"
/// #undef  TECIT_DLLIMPORT
/// \endcode
/// \n\n
/// \author   TEC-IT Datenverarbeitung GmbH
/// \author   Hans-Wagner Str. 6, A-4400 Steyr, AUSTRIA/EUROPE
/// \author   http://www.tec-it.com
/// \author   Copyright&copy; by TEC-IT Datenverarbeitung GmbH
/// \date     1996-2009
/// \version  6.0
/// \note For support requests please send an email to support@tec-it.com
///////////////////////////////////////////////////////////////////////////////

/// \mainpage Welcome Page
///
/// \section main_sec_10 Introduction
///
/// Many thanks for evaluating <em>TFORMer</em>!
///
/// The <em>TFORMer</em> product family represents a lean but <b>powerful output and reporting solution</b>. Professional graphical layout features are combined with versatile output capabilities. 
/// The integrated <b>bar-code</b> generator, full-featured <b>UNICODE</b> support, direct <b>PDF</b> generation and the possibility for client as well as server side integration qualifies <em>TFORMer</em> as <b>multi-purpose, multi-platform report generator</b>.
/// <ul>
/// <li><b>Users</b> benefit from the graphical layout editor with instant output capabilities: <em>TFORMer Designer</em> offers outstanding layout and printing features for documents like reports, tables, lists, serial letters, industry forms, vouchers and barcode labels. Especially the ready-to-use industry labels (e.g. VDA-4902, Odette, Galia, AIAG, ...) save time.</li>
/// <li><b>Software developers</b> use <em>TFORMer Runtime</em> as a reporting tool for direct printing (including native ZEBRA<sup>&reg;</sup> support) and PDF-, PostScript<sup>&reg;</sup>-, HTML- and image output. Form layouts are designed graphically with <em>TFORMer Designer</em>, <em>TFORMer Runtime</em> is then used to provide dynamic data for these layouts and to generate the output. It represents the core printing functionality and it is available for all major operating systems.</li>
/// </ul>
/// \section main_sec_11 Output Capabilities
/// 
/// Based on graphically designed FormLayouts <em>TFORMer</em> is able to generate output in the formats listed below. All output formats are supported natively by <em>TFORMer</em>. 
///
/// - <b>Direct printing</b> to all printers supported by Microsoft<sup>&reg;</sup> Windows<sup>&reg;</sup>.
/// - Built-in <b>ZEBRA/ZPL-II</b> support
/// - Built-in <b>PostScript</b> output
/// - Built-in <b>PDF</b> output and PDF document creation
/// - Built-in <b>HTML</b> output
/// - Built-in <b>Image</b> output
/// - <b>ASCII</b> output
///
/// \section main_sec_30 TFORMer Runtime
///
/// <em>TFORMer Runtime</em> is the name of the cross-platform output and reporting engine. It generates arbitrary labels or reports based on FormLayouts designed with <em>TFORMer Designer</em>. 
/// <em>TFORMer Runtime</em> is available with multiple APIs (application programming interface). It can be embedded into your applications or used from within batch-jobs or shell scripts: 
///
/// - <b>DLL</b> \n
/// This Microsoft Windows DLL interface is covered in this document. The DLL-API can be used by virtually all programming languages and development environments. See TFormer6.h.
/// \n\n
/// - <b>Shared Library</b> \n
/// For Linux and UNIX the reporting engine is available as shared library. For tar-balls or RPMs please refer to http://www.tec-it.com.
/// The API (see TFormer6.h) of this shared library is 100% compatible to the Windows-based DLL. 
/// \n\n
/// - <b>.NET Assembly</b> \n
/// The NET-API of <em>TFORMer Runtime</em> is described in a separate document. It provides a clear and easy-to-use programming interface and can be used with all .NET programming languages (.NET 2.0 or higher is required). 
/// \n\n
/// - <b>COM Component</b> \n
/// The COM-API of <em>TFORMer Runtime</em> is described in a separate document. COM objects can be embedded into Microsoft Office documents and may be used by all common development environments (e.g. Microsoft Visual Basic<sup>&reg;</sup>, Microsoft Visual C++<sup>&reg;</sup>, ...). \n
/// <b>IMPORTANT NOTICE:</b> Due to security reasons the COM component of <em>TFORMer Runtime</em> is <b>Site Locked</b> (see the Security section in the COM developer documentation). Without adjusting allowed internet zones or internet domains <em>TFORMer Runtime</em> may not be scripted within web-pages! This applies only to Microsoft Internet Explorer!
/// \n\n
/// - <b>Command Line Application (TFPrint)</b> \n
/// TFPrint offers a stand-alone, command line based printing and reporting application. 
/// All required parameters can be specified by command line parameters (e.g. FormLayout to be printed, output device, output type, DataSource, ...). 
/// TFPrint is available for all platforms (Microsoft Windows, Linux and UNIX). 
/// It can be used in batch-jobs, shell scripts and web-based scripting languages or it can be "spawned" from within your application as separate process. 
/// A detailed description is available in a separate manual. 
///
/// \section main_sec_20 Developer Information
///
/// - \subpage page_concepts 
/// - \subpage page_redist
/// - API Documentation - See section \b Modules in the menu of this help
/// 
/// \section main_sec_40 Additional Information
///
/// - \subpage page_limitations 
/// - \subpage page_support
/// - \subpage page_order 
/// \n\n
/// - \subpage page_contact 
/// \n\n
/// - \subpage page_disclaimer 
/// - \subpage page_credits 

/// \page page_limitations Limitations of the Demo Version
/// The demo version of <em>TFORMer Runtime</em> replaces graphical elements within a FormLayout with a demo-text on a random base.

/// \page page_support Product Support
/// TEC-IT is available for support at any time. This service is free and we would like to hear from you. Please just contact support@tec-it.com.

/// \page page_order How to Order a License
/// In order to enable the full-featured version, you need a valid license key from TEC-IT.
/// A description of the available license-types as well as all necessary information for ordering can be found at http://www.tec-it.com/prices
/// If you don't know the license type according to your application, please ask our sales team (sales@tec-it.com).
/// For placing an online order check out http://www.tec-it.com/order/.

/// \page page_disclaimer Disclaimer
/// \section page_disclaimer_10 Copyright / Urheberrechte
/// We reserve all rights to this document and the information contained therein. 
/// Reproduction, use or disclosure to third parties without express authority is strictly forbidden.
/// \n\n
/// F&uuml;r dieses Dokument und den darin dargestellten Gegenstand behalten wir uns alle Rechte vor. 
/// Vervielf&auml;ltigung, Bekanntgabe an Dritte oder Verwendung au&szlig;erhalb des vereinbarten Zweckes sind nicht gestattet.
/// \section page_disclaimer_20 Exclusion of Liability / Haftungsausschluss;
/// The actual version of this product is available as is. 
/// TEC-IT declines all warranties which go beyond applicable rights. 
/// The licensee bears all risks that might take place during the use of the documentation. 
/// TEC-IT and its contractual partner cannot be penalized for direct and indirect damages or losses 
/// (this includes non-restrictive, damages through loss of revenues, constriction in the exercise of business, 
/// loss of business information or any kind of commercial loss), which is caused by use or inability to use the product, 
/// although the possibility of such damage was pointed out by TEC-IT. 
///
/// \section page_disclaimer_40 Trademarks
/// AIX, AS/400 is a registered trademark of IBM Corporation. \n
/// HTML, DHTML, XML, XHTML are trademarks or registered trademarks of W3C, World Wide Web Consortium, Laboratory for Computer Science NE43-358, Massachusetts Institute of Technology, 545 Technology Square, Cambridge, MA 02139. \n
/// JAVA<sup>&reg;</sup> is a registered trademark of Sun Microsystems, Inc., 901 San Antonio Road, Palo Alto, CA 94303 USA. \n
/// JAVASCRIPT<sup>&reg;</sup> is a registered trademark of Sun Microsystems, Inc., used under license for technology invented and implemented by Netscape. \n
/// LINUX is a registered trademark of Linus Torvalds \n
/// Microsoft<sup>&reg;</sup>, Windows<sup>&reg;</sup>, Microsoft Word<sup>&reg;</sup>, Microsoft Excel<sup>&reg;</sup> are registered trademarks of Microsoft Corporation. \n
/// Navision is a registered trademark of Microsoft Business Solutions ApS in the United States and/or other countries. \n
/// Oracle<sup>&reg;</sup> is a registered trademark of Oracle Corporation. \n
/// PCL<sup>&reg;</sup> is a registered trademark of the Hewlett-Packard Company. \n
/// PostScript is a registered trademark of Adobe Systems Inc. \n
/// SAP, SAP Logo, R/2, R/3, ABAP, SAPscript are trademarks or registered trademarks of SAP AG in Germany (and in several other countries). \n
/// UNIX is a registered trademark of The Open Group \n\n
/// All other products mentioned are trademarks or registered trademarks of their respective companies. If any trademark in this document is not marked as trademark (or registered trademark), we ask you to send us a short message (mailto:office@tec-it.com).
///
/// \page page_contact Contact TEC-IT
/// \section page_contact_10 Contact us!
/// We would be glad to hear from you! Do not hesitate to send us questions, feature requests or general product feedback.
///
///  &copy; 1998-2009
/// \n TEC-IT Datenverarbeitung GmbH   
/// \n Hans-Wagner Stra&szlig;e 6                    
/// \n A-4400 Austria                  
/// \n t +43 (0)7252 72720           
/// \n f +43 (0)7252 72720 77        
/// \n http://www.tec-it.com
/// \n office@tec-it.com

/// \page page_credits Credits
/// \section credits_sec_10  3rd Party Software used in TFORMer
/// <em>TFORMer</em> is based in part on the work of the projects listed below - TEC-IT says "Thank You" to the respective developers!
///
/// <em>TFORMer</em> itself is not free software, but the source code of the mentioned 3rd party libraries is freely available. If you have troubles downloading the source code of these libraries from the web-links specified below please contact us. We are also glad to assist you with any object and/or script files in order to relink <em>TFORMer</em> with possible newer versions of these 3rd party libraries. In each case you agree to be charged with a reasonable amount compensating our handling &amp; distribution efforts.
///
/// \section credits_sec_20  Font Handling Library - FreeType 2 (FreeType Team)
/// For details please refer to http://freetype.sourceforge.net
///
/// License: See file freetype.LICENSE in the Bin directory of the <em>TFORMer</em> installation.
///
/// \section credits_sec_30  Data compression library - zlib
/// For details please refer to http://www.zlib.net
///
/// License: See file zlib.README in the Bin directory of the <em>TFORMer</em> installation.
///
/// \section credits_sec_50  Character-Set Conversion Library (GNU Iconv)
/// For details please refer to http://www.gnu.org/software/libiconv/
///
/// License: LGPL 2. See file iconv.LICENSE in the Bin directory of the <em>TFORMer</em> installation. This file is identical to the file COPYING.LIB which is part of the library source code.
///
/// \section credits_sec_60  XML Library (LibXML2)
/// For details please refer to http://www.opensource.org/licenses/mit-license.php or http://www.xmlsoft.org
///
/// License: MIT License. See file libxml2.LICENSE in the Bin directory of the <em>TFORMer</em> installation.
///
/// \section credits_sec_70  XSLT Library (LibXSLT)
/// For details please refer to http://xmlsoft.org/XSLT/
///
/// License: MIT License. Identical to LibXML2.
/// 
/// \section credits_sec_80  Font Configuration (FontConfig)
/// This 3rd party library is used for TFORMer Runtime on Linux or UNIX only. For details please refer to http://www.fontconfig.org
///
/// License: See file fontconfig.LICENSE in the Bin directory of the <em>TFORMer</em> installation. This file is identical to the file COPYING which is part of the library source code.
/// 
/// \section credits_sec_90  ODBC functionality for LINUX/UNIX (unixODBC)
/// This 3rd party library is used for TFORMer Runtime on Linux or UNIX only. For details please refer to http://www.unixodbc.org/
///
/// License: LGPL 2. See file unixodbc.LICENSE in the Bin directory of the <em>TFORMer</em> installation. This file is identical to the file COPYING which is part of the library source code.
/// 
/// \section credits_sec_100 Source code editing component - Scintilla
/// For details please refer to http://www.scintilla.org/
///
/// License: See file Scintilla.LICENSE in the Bin directory of the <em>TFORMer</em> installation.
/// 
/// \section credits_sec_110 Rendering Library - LibArt_LGPL
/// For details please refer to http://www.levien.com/libart/
///
/// License: See file libart_lgpl.LICENSE in the Bin directory of the <em>TFORMer</em> installation.


/// \page page_concepts Concepts - Printing
///
/// \section concepts_sec_10 Installation
/// Install <em>TFORMer Designer</em> and <em>TFORMer Runtime</em> using the setup application. Besides <em>TFORMer Designer</em> (the graphical layout editor) the setup-application installs <em>TFORMer Runtime</em>. <em>TFORMer Runtime</em> is available as DLL, as COM component, as .NET assembly and as stand-alone application (<c>TFPrint</c>). Both the COM component and the .NET assembly of <em>TFORMer Runtime</em> are registered automatically. The .NET assembly is automatically placed in the GAC. 
/// 
/// \section concepts_sec_20 Design and Test a Form Layout
/// Use <em>TFORMer Designer</em> to create a FormLayout graphically. 
/// \n\n
/// In most cases your FormLayout contains dynamic data which is provided by your application during print-time (e.g. an article number for a product label or a ticket-number for an eTicket). <em>TFORMer</em> uses DataFields as place-holders for such dynamic data. These DataFields are declared as well as used within the FormLayout. 
/// \n\n
/// For testing purposes <em>TFORMer Designer</em> allows you to provide values for such DataFields manually or to import them from files or data-bases. Testing a FormLayout can therefore be done without any line of code. 
/// \n\n
/// Depending on your application choose between stand-alone FormLayouts or Repository-based FormLayouts:
/// 
/// - <b>Stand-Alone</b> Form Layouts \n
///   <em>TFORMer Designer</em> creates stand-alone FormLayouts by default (file extension *.tff). A stand-alone FormLayout can be used on its own. Such a FormLayout contains all the necessary information which is required for printing (layout information, DataField definitions, JobTrayControl, DataSource definitions). 
/// - <b>Repository-based</b> Form Layouts \n
///   If you prefer to organize your FormLayouts and DataFields in a structured way or if you plan to create multiple FormLayouts which use the same data basis (e.g. same DataField definitions), the use of a so-called Repository is recommended. A Repository is a central database for FormLayouts, DataField definitions, JobTrayControl-settings and DataSource definitions.
///   Form layouts, DataField- and DataSource- definitions are stored within a Repository on a per Project base. A Project defines DataFields, DataSources and holds (better: refers) FormLayouts. Each of the DataFields defined in a Project is accessible within every FormLayout contained in that Project. A Repository can contain multiple Projects and one special Global Project. DataFields defined within the Global Project may be used by all FormLayouts in all Projects. A Repository uses the file extension *.tfr.
/// .
/// 
/// \section concepts_sec_30 Embed TFORMer Runtime into your Application
/// Create a new C or C++ project (or choose any programming language which is able to use DLLs or shared libraries). 
/// 
/// - <b>Microsoft Windows</b> based operating systems \n
/// At least you need to include the file <c>TFormer6.h</c> and to define the preprocessor variable <c>TECIT_DLLIMPORT</c>:
/// \code
/// #define TECIT_DLLIMPORT
/// #include "TFormer6.h"
/// #undef  TECIT_DLLIMPORT
/// \endcode
/// \n
/// Link your application with TFormer60.lib. 
/// \n\n
/// - <b>Linux/UNIX</b> based operating systems \n
/// When working on Linux or UNIX some additional lines of code are required: To use <em>TFORMer Runtime</em> include the files TECITStd.h and TFormer6.h into your source code. Make sure to define TECIT_DLLIMPORT, _TEC_UNIX and TEC_UNIX_BUILD before including the files. 
/// \code
/// #define TECIT_DLLIMPORT    
/// #define _TEC_UNIX
/// #define TEC_UNIX_BUILD
/// #include <TECITStd/TECITStd.h>
/// #include <TFormer6.h>
/// #undef   TECIT_DLLIMPORT
/// #undef TECIT_DLLIMPORT
/// #undef _TEC_UNIX
/// #undef TEC_UNIX_BUILD
/// \endcode
/// \n
/// Link your application with the parameter -lTFORMerDLL: 
/// \code
/// $ gcc TFORMerSimpleX.c -o TFORMerSimpleX -ldl -lTFormer6 -L/usr/local/lib -I/usr/local/include
/// \endcode
///
/// \note
/// Complete sample programs are available here:
/// - \subpage page_codewin_complete
/// - \subpage page_codex_complete
///
/// \section concepts_sec_40 License and Initialize TFORMer Runtime
/// Licensing is not required for evaluation purposes. For production use it is a must to apply a license key to <em>TFORMer Runtime</em> before using any other properties or methods of <em>TFORMer</em>. This needs to be done only once, for example when initializing the application. 
/// 
/// \code
/// TFormer_License ("John Doe", LICKIND_WORKSTATION, 1, "LicenseKey"); /* Pass license key */
/// \endcode
/// 
/// Now initialize <em>TFORMer Runtime</em>:
/// \code
///  HTFORM hTForm = TFormer_Init (NULL); /* Allocate memory and retrieve TFORMer handle (pointer) */
///  if (!hTForm)
///    return;
/// \endcode
/// \section concepts_sec_50 Select the Form Layout to be used
/// When you want to use a Repository based FormLayout (*.tfr), you have to specify the Repository, the Project and the name of the FormLayout: 
/// \code
/// eCode = TFormer_SetRepositoryName   (hTForm, "C:/ProgramData/TEC-IT/TFormer6/Samples/Demo Repository/Demos.tfr");
/// eCode = TFormer_SetProjectName      (hTForm, "TFORMer_Runtime_Examples");
/// eCode = TFormer_SetFormName         (hTForm, "BarcodeLabels");
/// \endcode
///
/// When using a stand-alone FormLayout (.tff) only one function call is required:
/// \code
/// eCode = TFormer_SetRepositoryName   (hTForm, "C:/SomePath/BarcodeLabels.tff");
/// \endcode
///
/// \note
/// See \ref concepts_sec_60_4 Additional methods for passing a FormLayout or a Repository to <em>TFORMer Runtime</em>.
/// 
/// \section concepts_sec_60 Provide Dynamic Data
/// As mentioned above a FormLayout usually contains DataFields which are used as place holders for dynamic data. 
/// Before printing a FormLayout you need to provide data for these DataFields. 
/// The required data can be passed to <em>TFORMer</em> using different methods:
/// 
/// \subsection concepts_sec_60_1 Pass Data via Program Code
/// This is the most direct method to pass data to <em>TFORMer Runtime</em>. In this case the developer provides names and values for the DataFields used in a FormLayout directly. The example below sets the values for the DataFields ArticleNo, ArticleName and ArticlePrice:
///
/// \code
/// eCode = TFormer_ResetData           (hTForm);                                           /* Clear the data cache */
/// eCode = TFormer_NewRecord           (hTForm);                                           /* Add one record */
/// eCode = TFormer_SetVarValue         (hTForm, "ArticleNo",  "20021001");                 /* Set the value of the DataField ArticleNo */
/// eCode = TFormer_SetVarValue         (hTForm, "ArticleName",  "Sound Blaster Audigy 2"); /* Set the value of the DataField ArticleName */   
/// eCode = TFormer_SetVarValue         (hTForm, "ArticlePrice",  "175");                   /* Set the value of the DataField ArticlePrice */
/// \endcode
///
/// \subsection concepts_sec_60_2 Load Data from Files or Databases
/// <em>TFORMer Runtime</em> is able to import data from text-/XML-files or from a database (ODBC). The filenames, DSNs or SQL-SELECT statements are specified directly. Take care, that the field-names available in the DataSource (e.g. the column names returned by a SQL-SELECT statement) are equal to the names of the DataFields used in a FormLayout.
/// 
/// \code
/// /* Use the XML file InputData.XML as DataSource */
/// eCode = TFormer_SetXmlDataFile      (hTForm, "/YourPath/InputData.XML"); 
/// \endcode
///
/// \code
/// /* Use the txt file InputData.txt as DataSource, using ';' as column separator, and '"' as text qualifier */
/// eCode = TFormer_SetTxtDataFile      (hTForm, "/YourPath/InputData.txt", ';', '"');
/// \endcode
///
/// \code
/// /* retrieve data from myDSN */
/// eCode = TFormer_SetODBCData         (hTForm, "myDSN", "user", "password", "SELECT * FROM myTable");
/// \endcode
///
/// \note
/// See \ref concepts_sec_60_4 Methods for additional methods for passing a file to <em>TFORMer Runtime</em>.
///
/// \subsection concepts_sec_60_3 Load Data from a User-Defined DataSource
/// DataSources can also be defined by the user with <em>TFORMer Designer</em> as part of a FormLayout. Such a DataSource defines how data is retrieved (e.g. from an ODBC database), the field-mappings and optional DataSource parameters. If you want to use such a DataSource, select it by name as shown below. Use optional DataSource parameters to control certain aspects during runtime (e.g. filenames, SQL statement or an ODBC-DSN). 
///
/// \code
/// /* retrieve data using the predefined datasource myTextDataSource */
/// eCode = TFormer_SetDatasourceName   (hTForm, "myTextDataSource");                    
/// /* set the value of the user-defined DataSource-Parameter parFile (defined in myTextDataSource) */
/// eCode = TFormer_SetDSParameterValue (hTForm, "parFile", "/YourPath/InputData.txt");  
/// \endcode
///
/// \subsection concepts_sec_60_4 Methods for Passing Files
/// If functions are requesting filenames (e.g. #TFormer_SetRepositoryName, #TFormer_SetTxtDataFile, #TFormer_SetXmlDataFile, #TFormer_SetConfigFile), these may be passed in the following ways:
///
/// - Specify the filename directly.
/// - Specify the filename of a ZIP-file (e.g. with WinZip) containing the required file. When zipping a Repository take care to zip all subdirectories as well. When zipping a stand-alone FormLayout take care to include required image files.
/// - BASE64-encode the content of the required file (e.g.: see Convert class of .NET) and use this string instead of a filename. When using this method to pass a stand-alone FormLayout this FormLayout must not use embedded images.
/// - BASE64-encode the content of a ZIP-file (see Convert class of .NET) containing the required file(s). Use this string instead of a filename. When zipping a Repository take care to zip all subdirectories as well. When zipping a stand-alone FormLayout take care to include required image files.
///
/// \note 
/// BASE64 encoded strings <b>must</b> start with the sequence <c>BASE64:</c>. When passing a zipped file <em>TFORMer</em> expects a real WinZip compatible format. A zipped stream is not suitable!
/// For more details please refer to <em>TFORMer Developer Manual</em> and \ref group_ZIPHandler.
/// 
/// \section concepts_sec_70 Print or Export the FormLayout (Printer, PDF, PostScript, HTML, ...)
/// \subsection concepts_sec_71 Print the FormLayout
/// To use the default printer of the current user specify NULL as printer name. 
/// \code
/// eCode = TFormer_SetPrinterName  (hTForm, NULL, NULL);              /* select the default printer of the system */
/// eCode = TFormer_Print           (hTForm);                          /* print ! */
/// \endcode
/// \subsection concepts_sec_72 Export as PDF, PostScript, HTML, ZPL-II or as an Image
/// Instead of sending the form to a printer, you can export the output as a PDF, PostScript, HTML, ASCII or image file. ZPL-II is supported too (the ZEBRA printer language). The procedure is the same as printing the form  but instead of setting a printer name, set the name and type of the output file:
/// \code
/// eCode = TFormer_SetPrinterType  (hTForm, ePrinterType_PDFFile ));  /* set the printer type to pdf */
/// eCode = TFormer_SetOutputName   (hTForm, "TFORMer.pdf" ));         /* write to the file TFORMer.pdf */
/// eCode = TFormer_Print           (hTForm);                          /* print ! */
/// \endcode
/// 
/// \section concepts_sec_80 Clean Up Resources
/// Clean up all resources allocated by <em>TFORMer Runtime</em>:
///
/// \code
/// TFormer_Exit (hTForm) /* Free the TFORMer handle */
/// \endcode

/// \page page_redist Distribution & Deployment 
///
/// <p>
/// <em>TFORMer Runtime</em> requires a number of files (DLLs, etc.) for execution.
/// When distributing your own application these files have to be included in your setup or distribution package.
/// </p>
/// <p> 
/// For a list of required files, please refer to the <em>TFORMer Developer Manual</em>.
/// </p>


/// \page page_codewin_complete Complete C Program (Microsoft Windows)
/// This complete example demonstrates the use of <em>TFORMer Runtime</em> - DLL.
/// It is installed by the setup application and prints a Repository-based FormLayout onto your Windows Default printer.
/// \attention
/// Do not forget to define TECIT_DLLIMPORT before including the header file TFormer6.h!
///
/// \include TFORMerSimple.c


/// \page page_codex_complete Complete C Program (Linux/UNIX)
/// This complete example demonstrates the use of <em>TFORMer Runtime</em> - Shared Library.
/// The sample is named TFORMerSimpleX.c\n 
/// The default installation location is /usr/local/share/TFORMer/SampleCode.
/// It is installed by the tarball/RPM and generates a Repository-based FormLayout as PDF document. 
/// \attention
/// Do not forget to define TECIT_DLLIMPORT, _TEC_UNIX and TEC_UNIX_BUILD before including the header files!
///
/// \include TFORMerSimpleX.c



/// \page page_errorcodes Error Codes
/// This is a partial list of error-codes used by <em>TFORMer Runtime</em>. Operating system specific error-codes are not listed. Error codes are specified in hexadecimal notation (0x).
/// If any of the <em>TFORMer</em> functions returns an error code not equal to zero (ErrOk) than DO NOT call subsequent <em>TFORMer</em>-functions except of #TFormer_Exit, #TFormer_GetLastErrorDescription, #TFormer_GetLastErrorInformation, #TFormer_GetLastErrorSource or any TFormer_Free-functions.
/// An error code indicates an error condition and subsequent calls may fail and produce unexpected results.
///
/// <TABLE>
/// <CAPTION> Partial list of error codes </CAPTION> 
/// <TR> <TD> Error Code </TD><TD> define                    </TD><TD> Description                                                                       </TD> </TR>
/// <TR> <TD> 0x00000000 </TD><TD> ErrOk                     </TD><TD> No Error                                                                          </TD> </TR>
/// <TR> <TD> 0x00000006 </TD><TD> ERROR_INVALID_HANDLE      </TD><TD> No or invalid handle given.                                                       </TD> </TR>    
/// <TR> <TD> 0x00000057 </TD><TD> ERROR_INVALID_PARAMETER   </TD><TD> Given parameter is invalid.                                                       </TD> </TR>
/// <TR> <TD> 0x80020006 </TD><TD> DISP_E_UNKNOWNNAME        </TD><TD> Unknown Name. The element with the given name was not found.                      </TD> </TR>
/// <TR> <TD> 0x8002802B </TD><TD> TYPE_E_ELEMENTNOTFOUND    </TD><TD> Either no element with the given name was found or no elements are in list.       </TD> </TR>
/// <TR> <TD> 0x00000008 </TD><TD> ERROR_NOT_ENOUGH_MEMORY   </TD><TD> Not enough memory to perform requested action.                                    </TD> </TR>    
/// <TR> <TD> 0x8002802D </TD><TD> TYPE_E_NAMECONFLICT       </TD><TD> The given name conflicts with name of other elements (e.g. duplicate DataField). </TD> </TR>
/// </TABLE>



///////////////////////////////////////////////////////////////////////////////
// Doxygen groups (or modules)
//
/// \defgroup group_DLL           DLL
///                               All Functions, Typedefs, Enumerations and Constants used by the DLL interface of <em>TFORMer Runtime</em>.
///
///
///
/// \defgroup group_General       General 
/// \ingroup group_DLL
///                               General functions mandatory for most applications.
///
///
///
/// \defgroup group_Print         Printing a FormLayout
/// \ingroup group_DLL
///                               Adjust output options, print (or generate PDF, PostScript, HTML and ASCII) a Repository-based or stand-alone FormLayout.
///
/// This is the main class of <em>TFORMer Runtime</em> for generating output. The basic steps to generate output are as follows: 
///
/// <TABLE>
/// <CAPTION> General Procedure </CAPTION> 
/// <TR> <TD> Task                                </TD><TD> See Also                                                                                                                      </TD><TD> Note                                                                                                                                                                                                                                    </TD> </TR>
/// <TR> <TD> Create a DataSource                 </TD><TD> DataSource                                                                                                                    </TD><TD> A DataSource is used to pass data to a Job.                                                                                                                                                                                             </TD> </TR>
/// <TR> <TD> Provide data for the DataSource     </TD><TD> DataSourceRecordSet, DataSourceCsv, DataSourceOdbc, DataSourceXml                                                             </TD><TD> Provide data programmatically or connect the DataSource with a file or a database. The DataSource is responsible to provide values for the DataFields required for the FormLayout to be printed.                                        </TD> </TR>
/// <TR> <TD> Create a Job                        </TD><TD> Job                                                                                                                           </TD><TD> The Job class is the main class for printing a FormLayout.                                                                                                                                                                              </TD> </TR>
/// <TR> <TD> Connect the Job with the DataSource </TD><TD> DataSource                                                                                                                    </TD><TD> If no DataSource is connected with the Job the default data for the FormLayout is used. This default data is identical to the data-set which was used for printing or previewing the FormLayout in the last <em>TFORMer Designer</em> session.    </TD> </TR>
/// <TR> <TD> Adjust the properties of the Job    </TD><TD> The most important properties are the output-type and the name of the target device (OutputName, PrinterType or PrinterName). </TD><TD> The order in which the properties are set is irrelevant. The method Print() takes care to apply the settings correctly.                                                                                                                 </TD> </TR>
/// <TR> <TD> Print() the Job                     </TD><TD> Print()                                                                                                                       </TD><TD> Generates the Job based on the FormLayout and DataSource in the required output format.                                                                                                                                                 </TD> </TR>
/// </TABLE> 
///
///
/// \defgroup group_Data          Providing Data
/// \ingroup group_DLL
///	<em>TFORMer Runtime</em> retrieves data used for printing a Job from a DataSource. 
///
/// A DataSource is always iterated Record by Record, the order of the records is not changed. 
///
/// Usually each Record triggers the printing of the detail-band(s) in the FormLayout.
/// Report headers are printed at the top of the first page (above the first page header) after the first Record was read.
/// Report footers are printed at the bottom of the last page after the last Record was read. 
/// Page headers and page footers are printed when required; no Record is needed to trigger them.
/// <em>TFORMer</em> requires at least one Record to produce output. 
/// 
/// Escape sequences in the values of DataFields are translated into their corresponding binary representation if #TFormer_SetHandleEscapeSequences is set to yes. 
///
/// If a DataField which is used in the FormLayout is not present in the DataSource, then the content of this DataField depends on #TFormer_SetReuseValues. By default, <em>TFORMer</em> treats unset DataFields as empty string ("") or 0 (depending on the type of the DataField - see #TFormer_ProjectVarItGetType). 
/// 
/// If no data is specified for printing a FormLayout, <em>TFORMer Runtime</em> uses the active DataSource (specified in the FormLayout) for printing the Job. If you are printing FormLayouts created with <em>TFORMer Designer</em> 5.1 or earlier, this corresponds to the data entered manually in the last <em>TFORMer Designer</em> session. This data is stored in an extra file (filename.xml) located in the same directory as the FormLayout. 
/// 
/// If the DataSource provides more DataFields than defined (either in the stand-alone FormLayout or in the Project of the Repository), these additional DataFields will be ignored. 
/// 
/// In order to identify all DataFields in a Project or in a FormLayout use the functions #TFormer_GetFirstProjectVarIt and #TFormer_GetNextProjectVarIt. If you want to know, whether a returned DataField is actually used in the FormLayout use the function #TFormer_ProjectVarItGetUsage.
///
///
///
/// \defgroup group_Error         Error-Handling 
/// \ingroup group_DLL
///                               Retrieve detailed error information.
///
///
///
/// \defgroup group_Repository    Repository Access 
/// \ingroup group_DLL
///                               Access and modify Repositories or stand-alone FormLayouts.
///
/// <b>Repository based FormLayouts:</b>\n
/// A Repository is a data-structure which references an arbitrary number of FormLayout-objects. 
/// User-defined Project-objects are used to divide a Repository into logical areas. 
/// \n\n
/// Each FormLayout referenced from within a Repository is a member of exactly one Project. 
/// DataFields defined within a Project are only usable by FormLayouts which are member of this Project. 
/// \n\n
/// The so-called GlobalProject is allocated by default and is exclusively used to define DataFields which are available for all FormLayouts (regardless of their actual Project-membership). 
/// It can not store references to a FormLayout. 
/// \n\n
/// The Repository also stores JobTrayControl-objects. 
/// They may be used whenever printing any FormLayout referenced by this Repository (see #TFormer_SetJobTrayControlName). 
/// \n\n
/// When printing a specific FormLayout within a Repository 3 parameters must be specified to identify the FormLayout: 
/// - The Repository name (which is identical to the filename of the Repository, see #TFormer_SetRepositoryName)
/// - The Project name (as defined by the user in <em>TFORMer Designer</em>, see #TFormer_SetProjectName)
/// - And the name of the FormLayout (as defined by the user in <em>TFORMer Designer</em>, see #TFormer_SetFormName)
/// .
/// \remarks
/// A Repository uses the file extension <c>*.tfr</c>. 
/// A sample Repository is usually installed in 
/// - Windows Vista: <c>C:/Program Data/TEC-IT/TFORMer/6.0/Examples/Demo Repository/Demos.tfr</c>
/// - Other versions of Microsoft Windows: <c>C:/Documents and Settings/All Users/Application Data/TEC-IT/TFORMer/6.0/Examples/Demo Repository</c>
/// \n\n
///
/// <b>Stand-alone FormLayouts:</b>\n
/// A stand-alone FormLayout can be used on its own. 
/// It can be imagined like a very simple type of Repository which contains exactly one GlobalProject which stores the DataField-definitions as well as the FormLayout itself. 
/// \n\n
/// A stand-alone FormLayout is not able to share DataField-definitions with other FormLayouts and can not contain user-defined Project-objects. 
/// \n\n
/// The stand-alone FormLayout also stores JobTrayControl-objects. They may be used whenever printing this FormLayout (see #TFormer_SetJobTrayControlName). 
/// \n\n
/// When printing a Job using a stand-alone FormLayout 1 parameter must be specified: 
/// - The Repository name (which is identical to the filename of the stand-alone FormLayout, see #TFormer_SetRepositoryName).
/// .
/// \note
/// Do not use #TFormer_SetProjectName and #TFormer_SetFormName when using a stand-alone FormLayout! 
///
/// \remarks
/// A stand-alone FormLayout uses the file extension <c>*.tff</c>. 
/// Sample FormLayouts are usually installed in 
/// - Windows Vista: <c>C:/Program Data/TEC-IT/TFORMer/6.0/Examples/Command Line</c>
/// - Other versions of Microsoft Windows: <c>C:/Documents and Settings/All Users/Application Data/TEC-IT/TFORMer/6.0/Examples/Command Line</c>
///
///
/// \defgroup group_ZIPHandler    ZIP and BASE64 Handling
/// \ingroup group_DLL
///								  All <em>TFORMer</em> functions accepting filenames may also be called with a name of a ZIP-file or with a BASE64 encoded string (which may contain a zip-file again). 
///									Functions in this group enable the software developer to implement an optimized handling of such zip-files (or BASE64 string).
/// \n\n
///								  Usually all <em>TFORMer</em> functions accepting files are trying to determine if it is a zip-file automatically (a BASE64 encoded file is determined by the <c>BASE64:</c> prefix). 
///								  When <em>TFORMer</em> detects a zip-file it is unzipped automatically to the temp-directory of the current user. This automatic unzipping requires <em>TFORMer</em> to clean up such temporary files after the function call ends.
///								  If you want to avoid the per-function unzipping and deleting, then implement the unzipping functionality yourself by the TecUnzipper-functions. Thereafter pass the resulting files to the <em>TFORMer</em> functions.
/// - Some declarations                                                         
///   \code
///   ERRCODE eCode       = ErrOk;
///   LPCSTR  pszFilename = "repository.zip";
///   \endcode
/// - Allocate an unzipper handle
///	  \code
///	  HTUNZIPPER hZip = TecUnzipper_Alloc( pszFilename );
///	  if( !hZip )
///	    return ERROR_OUTOFMEMORY;
///   \endcode
/// - Make sure that data is always BASE64 decoded. Not required in this case, but should be always done so that you can pass strings starting with "BASE64:" to TecUnzipper_Alloc
///	  \code
///	  TecUnzipper_Base64Decode( hZip );
///   \endcode
/// - Extract data to a temporary directory
///	  \code
///	  eCode = TecUnzipper_Extract( hZip, NULL );
///   \endcode
/// - Check the return code to see if extracting was successful
///	  \code
///	  if( eCode == ErrOk )
///	  {
///   \endcode
/// - Find the first Repository in the zip file
///	  \code
///	    INT nIndex  = TecUnzipper_FindFileByExt( hZip, "TFR", -1, TRUE );
///	    if( nIndex == -1 )
///   \endcode
/// - The zip archive does not contain a <em>TFORMer</em> Repository file.
///	  \code
///	      eCode = ERROR_FILE_NOT_FOUND;
///	    else
///   \endcode
/// - Get the absolute path to the extracted Repository file.
///	  \code
///	      pszFilename = TecUnzipper_GetFile( hZip, nIndex );
///	  }
///	  else
///	  {
///   \endcode
/// - The passed filename was neither a zip file nor a base 64 encoded string. Ignore the error and work with the data as a normal TFR file. 
///	  \code
///	    pszFilename = TecUnzipper_GetFilename( hZip );
///	    eCode       = ErrOk;
///	  }
///   \endcode
/// - pszFilename contains now the path to a normal temporary TFR file
/// - Do something with pszFilename now ...                        
/// - Delete created temporary files
///	  \code
///   TecUnzipper_DeleteAllFiles( hZip );
///   \endcode
/// - Free the unzipper handle                                                 
///	  \code
///	  TecUnzipper_Free( hZip );
///   \endcode
///
/// \defgroup group_Misc          Miscellaneous 
/// \ingroup group_DLL
///                               Special purpose functions for System Integrators.
///
///
///
/// \defgroup group_System        System
/// \ingroup group_DLL
///                               Special purpose functions for operating system depending information.
///
///
///
///////////////////////////////////////////////////////////////////////////////


// ----------------------------------------------------------------------
// avoid multiple includes
#ifndef __TFORMERLIB_H__                                                                                        
#define __TFORMERLIB_H__

// ----------------------------------------------------------------------
// TEC-IT specific stuff
// ----------------------------------------------------------------------
#undef TECIT_DLLSPEC
#ifdef WIN32
#ifdef TECIT_DLLEXPORT
#define TECIT_DLLSPEC _declspec (dllexport)
#endif
#ifdef TECIT_DLLIMPORT
#define TECIT_DLLSPEC _declspec (dllimport)
#endif
#endif

#ifndef TECIT_DLLSPEC
#define TECIT_DLLSPEC
#endif

// for C++ users
#ifdef __cplusplus
  extern "C"
  {
#endif

// ----------------------------------------------------------------------
// Include section                                                                     
// ----------------------------------------------------------------------
#ifdef _WIN32
  #include "windows.h"
#endif

// ----------------------------------------------------------------------
// Defines for return values (error code)
// ----------------------------------------------------------------------
#ifndef ERRCODE
#define ERRCODE LONG
#endif

#ifndef ErrOk
#define ErrOk S_OK
#endif

// ----------------------------------------------------------------------
/// The license types for the <em>TFORMer Runtime</em> (see #TFormer_License).
// ----------------------------------------------------------------------
#define LICKIND_WORKSTATION 1   ///< Workstation license (exactly one installation on a machine identified via the system-id).
#define LICKIND_SITE        2   ///< Site license (up to 250 installations or users (whichever is higher) within one site are allowed).
#define LICKIND_DEVELOPER   3   ///< Developer license (for client-applications only, up to 10000 redistributions per developer license).
#define LICKIND_OFFICE      4   ///< Office license (up to 100 installations or users (whichever is higher) within one site are allowed).
#define LICKIND_WORKGROUP   5   ///< Workgroup license (up to 10 installations or users (whichever is higher) within one site are allowed).
#define LICKIND_SERVER      6   ///< Server license (installation on one (web-)server for up to 10000 users).

// ----------------------------------------------------------------------
/// Defines the type of a DataField. 
/// \remarks
/// The type of a DataField influences its allowed use in expressions within a FormLayout. 
///	The DataFieldType also influences the conversion method when reading a DataField from a DataSource. 
/// Currently three different data-types are supported - it is recommended to use the type Text whenever possible. 
// ----------------------------------------------------------------------
#ifndef _TFR_API_               // TEC-IT specific stuff. NO Repository, so assume external usage without the Repository DLL
  typedef enum EVarType_tag
  {
  eVarType_Text     = 0,      ///< A text (string or character). This is the recommended #EVarType.
    eVarType_Long     = 1,      ///< An integer number. When reading a DataField of this type from a DataSource it is converted automatically to an integer. In case of conversion errors the result is 0 (no error or exceptions will be raised).
    eVarType_Float    = 2,      ///< A floating point number. When reading a DataField of this type from a DataSource it is converted automatically to a double. In case of conversion errors the result is 0.0 (no error or exceptions will be raised).
    eVarType_Invalid            ///< Invalid data-type (DO NOT USE)
  } EVarType;
#endif

// ----------------------------------------------------------------------
/// Enumeration for the type of the output device. 
/// \remarks
/// This enumeration specifies the type of output generated by <em>TFORMer Runtime</em>. 
/// Choose between direct printing (Win32, Repository, Cups, PSPrinter, TxtPrinter, ZPLPrinter) and other supported output formats (PSFile, HTML, PDFFile, TxtFile or the various image formats). 
// ----------------------------------------------------------------------
#ifndef TFORMER_TYPES_DEFINED
  typedef enum EPrinterType_tag
  {
    ePrinterType_Default = 0,       ///< The default output device type. The default output device used depends on the type of the FormLayout to be printed.  Print a Repository-based FormLayout: A printer of type Repository will be used. Without the specification of a PrinterName the default printer within this Repository (named <c>_Default_</c>) is used. Print a stand-alone FormLayout: A printer of type Win32 will be used on Windows, for Linux and UNIX a printer of type Cups will be used.
    ePrinterType_Win32,             ///< Direct printing to a Windows printer via GDI-calls (Microsoft Windows only).
    ePrinterType_Repository,        ///< The printer as defined in the Repository or stand-alone FormLayout. In the later case only the printer <c>_Default_</c> is used (which maps to the default printer of the current user). To use other printers set the PrinterType to Win32.
    ePrinterType_Cups,              ///< A CUPS printer (Linux and UNIX only).
    ePrinterType_PSFile,            ///< PostScript output (as file).
    ePrinterType_HTML,              ///< HTML output (as file). The footer of the generated HTML-pages may be customized. See Footer.html in the <em>TFORMer</em> installation directory for details.
    ePrinterType_PDFFile,           ///< PDF output (as file).
    
    ePrinterType_PSPrinter,         ///< PostScript output (sent to a specific printer).
    ePrinterType_TxtPrinter,        ///< Text output (to a specific windows printer)
    ePrinterType_TxtFile,           ///< Text output (as file)
    ePrinterType_ImageBmp,          ///< BMP image output (as file, one file per page). The specified filename may contain #-characters as placeholders for the page-number.
    ePrinterType_ImageGif,          ///< GIF Image output (as file, one file per page). The specified filename may contain #-characters as placeholders for the page-number.
    ePrinterType_ImageJpg,          ///< JPG image output (as file, one file per page). The specified filename may contain #-characters as placeholders for the page-number.
    ePrinterType_ImagePcx,          ///< PCX image output (as file, one file per page). The specified filename may contain #-characters as placeholders for the page-number.
    ePrinterType_ImagePng,          ///< PNG image output (as file, one file per page). The specified filename may contain #-characters as placeholders for the page-number.
    ePrinterType_ImageTga,          ///< TGA image output (as file, one file per page). The specified filename may contain #-characters as placeholders for the page-number.
    ePrinterType_ImageTifSingle,    ///< TIFF image output (as file, one file per page). The specified filename may contain #-characters as placeholders for the page-number.
    ePrinterType_ImageTifMultiPage, ///< TIFF image output (as file, Multi-Page TIF format).

    ePrinterType_ZPLPrinter, 
    ePrinterType_ZPLFile,   

    ePrinterType_Invalid      ///< Last entry used to verify valid parameters.
  } EPrinterType;
#endif // TFORMER_TYPES_DEFINED

// ----------------------------------------------------------------------
/// Describes the usage of a DataField within a FormLayout. 
// ----------------------------------------------------------------------
typedef enum EVarUsage_tag {
  eVarUsage_NotUsed = 0,        ///< The DataField is defined within the Project but is not used in the FormLayout. 
  eVarUsage_Normal,             ///< A normal DataField. The DataField is in use by the FormLayout, its value is not modified. The DataField is just used as part of expressions or directly in text-, bar-code or picture elements. Therefore the value for this DataField may be provided by an external application or by the user.
  eVarUsage_Calculated,         ///< The DataField is computed automatically. The DataField is in use by the FormLayout and its value is modified. Usually this is done via pre- or post-expressions. Therefore the value for this DataField may not be provided by an external application or by the user.
  eVarUsage_Parameter,          ///< This is a writable system-DataField used by the FormLayout. The value of this system-DataField may be provided by an external application or by the user (e.g. tfDocumentName).
  eVarUsage_Invalid             ///< Invalid DataField usage. 
} EVarUsage;

// ----------------------------------------------------------------------
// Forward declaration for internal structure used by TEC-IT
// ----------------------------------------------------------------------
typedef void* HTFRepository;

// ----------------------------------------------------------------------
// Are the structs defined?
// ----------------------------------------------------------------------
#ifndef TFORMER_TYPES_DEFINED
typedef struct  TTFORMerInstance_tag            TTFORMerInstance;
typedef struct  TPROJECTVAR_ITERATOR_tag        TPROJECTVAR_ITERATOR;
typedef struct  TFORM_ITERATOR_tag              TFORM_ITERATOR;
typedef struct  TPROJECT_ITERATOR_tag           TPROJECT_ITERATOR;
typedef struct  TDATASOURCE_ITERATOR_tag        TDATASOURCE_ITERATOR;
typedef struct  TDSPARAM_ITERATOR_tag           TDSPARAMETER_ITERATOR;
#endif

// ----------------------------------------------------------------------
/// Declaration of <em>TFORMer</em>-Instance handle
/// \remarks
/// This handle stores the connection to <em>TFORMer Runtime</em>
// ----------------------------------------------------------------------
typedef TTFORMerInstance*       HTFORM;

// ----------------------------------------------------------------------
/// Declaration of the DataField iterator
/// \remarks
/// A Repository is divided into Projects. Projects hold DataField declarations.
// ----------------------------------------------------------------------
typedef TPROJECTVAR_ITERATOR*   HPROJECTVAR_ITERATOR;

// ----------------------------------------------------------------------
/// Declaration of the FormLayout iterator 
/// \remarks
/// A FormLayout is stored in a Project of a Repository
// ----------------------------------------------------------------------
typedef TFORM_ITERATOR*         HFORM_ITERATOR;

// ----------------------------------------------------------------------
/// Declaration of the Project iterator 
/// \remarks
/// A Repository is divided into Projects.
// ----------------------------------------------------------------------
typedef TPROJECT_ITERATOR*      HPROJECT_ITERATOR;

// ----------------------------------------------------------------------
/// Declaration of the DataSource iterator ****
/// \remarks
/// A Repository is divided into Projects. Projects hold DataSource declarations.
// ----------------------------------------------------------------------
typedef TDATASOURCE_ITERATOR*      HDATASOURCE_ITERATOR;

// ----------------------------------------------------------------------
/// Declaration of the DataSource-Parameter iterator ****
/// \remarks
/// A Repository is divided into Projects. Projects hold DataSource declarations.
/// DataSources may have Parameters.
// ----------------------------------------------------------------------
typedef TDSPARAMETER_ITERATOR*      HDSPARAMETER_ITERATOR;

// ----------------------------------------------------------------------
/// Declaration of the Unzipper  
/// \remarks
/// This handle is used for unzipping repositories, stand-alone FormLayouts and 
/// data which are passed as a BASE64 encoded ZIP file.
// ----------------------------------------------------------------------
typedef void* HTUNZIPPER;

// ----------------------------------------------------------------------
/// Defines for UNICODE/ANSI Independence  
/// \remarks
/// All functions using string parameters or string return values are available in wide and in ASCII versions. 
/// The function names marked with a trailing T are mapped to the following variants. this mapping depends on the operating system and the preprocessor variable _UNICODE:
/// Microsoft Windows without _UNICODE defined: ASCII version 
/// Microsoft Windows with _UNICODE defined: UNICODE version 
/// Linux and UNIX without _UNICODE defined: UTF-8 
/// Linux and UNIX with _UNICODE defined: UTF-8 
/// \note
/// Use the *T functions whenever possible. It helps you to write portable code. Do not \#define _UNICODE on Linux or UNIX. It is simply not needed (and not supported) because <em>TFORMer Runtime</em> on these platforms uses UTF-8 in each case.
// ----------------------------------------------------------------------
#ifdef _UNICODE
  #define TFormer_LicenseT                      TFormer_LicenseW
  #define TFormer_SetRepositoryNameT            TFormer_SetRepositoryNameW
  #define TFormer_SetPrinterNameT               TFormer_SetPrinterNameW  
  #define TFormer_SetOutputNameT                TFormer_SetOutputNameW
  #define TFormer_SetJobTrayControlNameT        TFormer_SetJobTrayControlNameW
  #define TFormer_SetProjectNameT               TFormer_SetProjectNameW
  #define TFormer_SetFormNameT                  TFormer_SetFormNameW
  #define TFormer_SetDataSourceNameT            TFormer_SetDatasourceNameW
  #define TFormer_SetDSParameterValueT          TFormer_SetDSParameterValueW
  #define TFormer_GetDataSourceNameT            TFormer_GetDatasourceNameW  
  #define TFormer_SetConfigFileT                TFormer_SetConfigFileW
  #define TFormer_SetDataFileT                  TFormer_SetDataFileW
  #define TFormer_SetVarValueT                  TFormer_SetVarValueW
  #define TFormer_OpenDesignerT                 TFormer_OpenDesignerW
  #define TFormer_SetImportFieldForRecordCopyT  TFormer_SetImportFieldForRecordCopyW
  #define TFormer_SetTxtDataFileT               TFormer_SetTxtDataFileW
  #define TFormer_SetODBCDataT                  TFormer_SetODBCDataW
  #define TFormer_SetXmlDataFileT               TFormer_SetXmlDataFileW 
  #define TFormer_SetXmlDataFileExT             TFormer_SetXmlDataFileExW
  #define TFormer_SaveXmlT                      TFormer_SaveXmlW
  #define TFormer_GetLastErrorDescriptionT      TFormer_GetLastErrorDescriptionW
  #define TFormer_GetLastErrorInformationT      TFormer_GetLastErrorInformationW
  #define TFormer_GetLastErrorSourceT           TFormer_GetLastErrorSourceW
  #define TFormer_AddNewProjectT                TFormer_AddNewProjectW
  #define TFormer_DeleteProjectT                TFormer_DeleteProjectW
  #define TFormer_AddNewFormT                   TFormer_AddNewFormW
  #define TFormer_DeleteFormT                   TFormer_DeleteFormW
  #define TFormer_AddNewVarT                    TFormer_AddNewVarW
  #define TFormer_DeleteVarT                    TFormer_DeleteVarW
  #define TFormer_ProjectItGetNameT             TFormer_ProjectItGetNameW
  #define TFormer_ProjectItGetDescriptionT      TFormer_ProjectItGetDescriptionW
  #define TFormer_GetFirstFormItT               TFormer_GetFirstFormItW
  #define TFormer_FormItGetNameT                TFormer_FormItGetNameW
  #define TFormer_FormItGetDescriptionT         TFormer_FormItGetDescriptionW
  #define TFormer_GetFirstProjectVarItT         TFormer_GetFirstProjectVarItW
  #define TFormer_ProjectVarItGetNameT          TFormer_ProjectVarItGetNameW
  #define TFormer_ProjectVarItGetDescriptionT   TFormer_ProjectVarItGetDescriptionW
  #define TFormer_ProjectVarItGetCaptionT       TFormer_ProjectVarItGetCaptionW
  #define TFormer_ProjectVarItGetDisplayOrderT  TFormer_ProjectVarItGetDisplayOrderW
  #define TFormer_ProjectVarItGetDefValueT      TFormer_ProjectVarItGetDefValueW
  #define TFormer_GetVarUsageT                  TFormer_GetVarUsageW
  #define TFormer_GetOSPrinterT                 TFormer_GetOSPrinterW
  #define TFormer_GetOSDefaultPrinterT          TFormer_GetOSDefaultPrinterW

  #define TFormer_GetFirstDatasourceItT         TFormer_GetFirstDatasourceItW
  #define TFormer_DatasourceItGetNameT          TFormer_DatasourceItGetNameW
  #define TFormer_DatasourceItGetDescriptionT   TFormer_DatasourceItGetDescriptionW

  #define TFormer_GetFirstDSParameterItT        TFormer_GetFirstDSParameterItW
  #define TFormer_DSParameterItGetNameT         TFormer_DSParameterItGetNameW
  #define TFormer_DSParameterItGetDescriptionT  TFormer_DSParameterItGetDescriptionW
  #define TFormer_DSParameterItGetCaptionT      TFormer_DSParameterItGetCaptionW
  #define TFormer_DSParameterItGetDisplayOrderT TFormer_DSParameterItGetDisplayOrderW
  #define TFormer_DSParameterItGetDefValueT     TFormer_DSParameterItGetDefValueW

  #define TecUnzipper_AllocT                    TecUnzipper_AllocW
  #define TecUnzipper_ExtractT                  TecUnzipper_ExtractW
  #define TecUnzipper_GetFileT                  TecUnzipper_GetFileW
  #define TecUnzipper_GetFilenameT              TecUnzipper_GetFilenameW 
  #define TecUnzipper_FindFileByExtT            TecUnzipper_FindFileByExtW
#else
  #define TFormer_LicenseT                      TFormer_License
  #define TFormer_SetRepositoryNameT            TFormer_SetRepositoryName
  #define TFormer_SetPrinterNameT               TFormer_SetPrinterName
  #define TFormer_SetOutputNameT                TFormer_SetOutputName
  #define TFormer_SetJobTrayControlNameT        TFormer_SetJobTrayControlName
  #define TFormer_SetProjectNameT               TFormer_SetProjectName
  #define TFormer_SetFormNameT                  TFormer_SetFormName
  #define TFormer_SetDataSourceNameT            TFormer_SetDatasourceName
  #define TFormer_SetDSParameterValueT          TFormer_SetDSParameterValue
  #define TFormer_GetDataSourceNameT            TFormer_GetDatasourceName
  #define TFormer_SetConfigFileT                TFormer_SetConfigFile
  #define TFormer_SetDataFileT                  TFormer_SetDataFile
  #define TFormer_SetVarValueT                  TFormer_SetVarValue
  #define TFormer_OpenDesignerT                 TFormer_OpenDesigner
  #define TFormer_SetImportFieldForRecordCopyT  TFormer_SetImportFieldForRecordCopy
  #define TFormer_SetTxtDataFileT               TFormer_SetTxtDataFile
  #define TFormer_SetODBCDataT                  TFormer_SetODBCData
  #define TFormer_SetXmlDataFileT               TFormer_SetXmlDataFile
  #define TFormer_SetXmlDataFileExT             TFormer_SetXmlDataFileEx
  #define TFormer_SaveXmlT                      TFormer_SaveXml
  #define TFormer_GetLastErrorDescriptionT      TFormer_GetLastErrorDescription
  #define TFormer_GetLastErrorInformationT      TFormer_GetLastErrorInformation
  #define TFormer_GetLastErrorSourceT           TFormer_GetLastErrorSource
  #define TFormer_AddNewProjectT                TFormer_AddNewProject
  #define TFormer_DeleteProjectT                TFormer_DeleteProject
  #define TFormer_AddNewFormT                   TFormer_AddNewForm
  #define TFormer_DeleteFormT                   TFormer_DeleteForm
  #define TFormer_AddNewVarT                    TFormer_AddNewVar
  #define TFormer_DeleteVarT                    TFormer_DeleteVar
  #define TFormer_ProjectItGetNameT             TFormer_ProjectItGetName
  #define TFormer_ProjectItGetDescriptionT      TFormer_ProjectItGetDescription
  #define TFormer_GetFirstFormItT               TFormer_GetFirstFormIt
  #define TFormer_FormItGetNameT                TFormer_FormItGetName
  #define TFormer_FormItGetDescriptionT         TFormer_FormItGetDescription
  #define TFormer_GetFirstProjectVarItT         TFormer_GetFirstProjectVarIt
  #define TFormer_ProjectVarItGetNameT          TFormer_ProjectVarItGetName
  #define TFormer_ProjectVarItGetDescriptionT   TFormer_ProjectVarItGetDescription
  #define TFormer_ProjectVarItGetCaptionT       TFormer_ProjectVarItGetCaption
  #define TFormer_ProjectVarItGetDisplayOrderT  TFormer_ProjectVarItGetDisplayOrder
  #define TFormer_ProjectVarItGetDefValueT      TFormer_ProjectVarItGetDefValue
  #define TFormer_GetVarUsageT                  TFormer_GetVarUsage
  #define TFormer_GetOSPrinterT                 TFormer_GetOSPrinter
  #define TFormer_GetOSDefaultPrinterT          TFormer_GetOSDefaultPrinter

  #define TFormer_GetFirstDatasourceItT         TFormer_GetFirstDatasourceIt
  #define TFormer_DatasourceItGetNameT          TFormer_DatasourceItGetName
  #define TFormer_DatasourceItGetDescriptionT   TFormer_DatasourceItGetDescription

  #define TFormer_GetFirstDSParameterItT        TFormer_GetFirstDSParameterIt
  #define TFormer_DSParameterItGetNameT         TFormer_DSParameterItGetName
  #define TFormer_DSParameterItGetDescriptionT  TFormer_DSParameterItGetDescription
  #define TFormer_DSParameterItGetCaptionT      TFormer_DSParameterItGetCaption
  #define TFormer_DSParameterItGetDisplayOrderT TFormer_DSParameterItGetDisplayOrder
  #define TFormer_DSParameterItGetDefValueT     TFormer_DSParameterItGetDefValue

  #define TecUnzipper_AllocT                    TecUnzipper_Alloc
  #define TecUnzipper_ExtractT                  TecUnzipper_Extract
  #define TecUnzipper_GetFileT                  TecUnzipper_GetFile
  #define TecUnzipper_GetFilenameT              TecUnzipper_GetFilename
  #define TecUnzipper_FindFileByExtT            TecUnzipper_FindFileByExt
#endif



// ----------------------------------------------------------------------
/// Allocates and initializes internal memory.
///
/// \returns - Non-zero if successful\n - NULL if error occurred
///
/// \param[in] hRepository Provide NULL in all cases.\n
///                        This parameter is used internally by TEC-IT to provide a handle to an already opened Repository.
///
/// \remarks
/// This function must be called after loading the <em>TFORMer</em> library. 
/// All other calls to the library must be performed after #TFormer_Init. 
/// Each thread of your application (if multi-threaded) must call #TFormer_Init 
/// and must use the returned handle for subsequent calls.
///
/// \Note Do not use the same handle from within different threads of your application.
///
/// \Note Do not forget to free allocated memory by calling #TFormer_Exit.
///
/// \see #TFormer_Exit
/// \ingroup group_General
// ----------------------------------------------------------------------
TECIT_DLLSPEC
HTFORM      __stdcall       TFormer_Init
(
  HTFRepository  hRepository                        
);

// ----------------------------------------------------------------------
/// Frees system resources previously allocated with #TFormer_Init.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance	The instance provided by a previous call to #TFormer_Init.
///
/// \remarks
/// Frees system resources allocated by #TFormer_Init. Call this function  
/// before exiting your application or if <em>TFORMer Runtime</em> should be unloaded from memory. 
/// After this call the <em>TFORMer Runtime</em> must be initialized again (with #TFormer_Init) 
/// before using it.
///
/// \Note It is not required to call this function after each print-job.
///
/// \see #TFormer_Init
/// \ingroup group_General
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE     __stdcall       TFormer_Exit
(
 HTFORM                     pInstance               
);

// ----------------------------------------------------------------------
/// Sets the filename of the Repository or the stand-alone FormLayout.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance      The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFileName    File name of the Repository or the stand-alone FormLayout
///
/// \remarks
/// Use this function to provide either the .tfr (<em>TFORMer</em> Repository), 
/// or the .tff (<em>TFORMer</em> stand-alone FormLayout) file which should be used.
/// If you provide a .tfr (Repository) file, then you must provide the Project 
/// and FormLayout name afterwards by using #TFormer_SetProjectName and #TFormer_SetFormName.
/// \n\n
/// You can also specify a ZIP file or a BASE64 encoded string in pszFileName (for details see #TecUnzipper_Alloc).
///
/// Repository based FormLayouts:\n
/// A Repository is a data-structure which references an arbitrary number of FormLayout-objects. 
/// User-defined Project-objects are used to divide a Repository into logical areas. 
/// Each FormLayout referenced from within a Repository is a member of exactly one Project. 
/// DataFields defined within a Project are only usable by FormLayouts which are member of this Project. 
/// The so-called GlobalProject is allocated by default and is exclusively used to define DataFields which are available for all FormLayouts (regardless of their actual Project-membership). 
/// It can not store references to a FormLayout. 
/// The Repository also stores JobTrayControl-objects. They may be used whenever printing any FormLayout referenced by this Repository (see #TFormer_SetJobTrayControlName). 
/// When printing a specific FormLayout within a Repository 3 parameters must be specified to identify the FormLayout: 
/// - The Repository name (which is identical to the filename of the Repository, see #TFormer_SetRepositoryName)
/// - The Project name (as defined by the user in <em>TFORMer Designer</em>, see #TFormer_SetProjectName)
/// - And the Form name (as defined by the user in <em>TFORMer Designer</em>, see #TFormer_SetFormName)
/// .
/// \n\n
/// Stand-alone FormLayouts:\n
/// A stand-alone FormLayout can be used on its own. 
/// It can be imagined like a very simple type of Repository which contains exactly one GlobalProject which stores the DataField-definitions as well as the FormLayout itself. 
/// A stand-alone FormLayout is not able to share DataField-definitions with other FormLayouts and can not contain user-defined Project-objects. 
/// The stand-alone FormLayout also stores JobTrayControl-objects. They may be used whenever printing this FormLayout (see #TFormer_SetJobTrayControlName). 
/// When printing a Job using a stand-alone FormLayout 1 parameter must be specified: 
/// - The Repository name (which is identical to the filename of the stand-alone FormLayout, see #TFormer_SetRepositoryName).
/// Do not use #TFormer_SetProjectName and #TFormer_SetFormName when using a stand-alone FormLayout! 
///
/// \note
/// A Repository or a Stand-alone form may be passed in the following ways:
///
/// - Specify the filename directly.
/// - Specify the filename of a ZIP-file (e.g. with WinZip) containing the required file.
/// - BASE64-encode the content of the required file (see Convert class of .NET) and use this string instead of a filename. When using this method to pass a stand-alone form this form must not use embedded images.
/// - BASE64-encode the content of a ZIP-file (see Convert class of .NET) containing the required file(s). Use this string instead of a filename. When zipping a repository take care to zip all subdirectories as well. When zipping a stand-alone form take care to include required image files.
///
/// \note 
/// BASE64 encoded strings <b>must</b> start with the sequence <c>BASE64:</c>. When passing a zipped file <em>TFORMer</em> expects a real WinZip compatible format. A zipped stream is not suitable!
/// For more details please refer to the <em>TFORMer Runtime DLL Programming Reference</em>.
/// 
///
/// \Note All cached data as well as the actual Project and FormLayout are discarded when calling this function.
///
/// \see #TFormer_SetProjectName, #TFormer_SetFormName.
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetRepositoryName 
(
 HTFORM                     pInstance               
,LPCSTR                     pszFileName             
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetRepositoryNameW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszFileName             
);
#endif //#ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Sets the type of the generated output. 
///
/// \remarks
/// Use this function to provide the type of the output device or output file (Direct printing, PDF, Postscript, HTML, ...) .
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance      The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] ePrinterType   Specifies the type of output stream which should be generated, see #EPrinterType_tag.
/// 
/// \note
/// Call this function prior to #TFormer_SetOutputName.
///
/// \see #EPrinterType_tag
/// 
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_SetPrinterType
(
 HTFORM                     pInstance               
,EPrinterType               ePrinterType             
);

// ----------------------------------------------------------------------
/// Sets type and name of the output device. 
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance        The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszPrinterName   Printer name as defined in Windows (see Control Panel). Use NULL or an empty string ("") to use the default printer of the logged-on user.
/// \param[in] pDevMode         Special device-mode settings for Windows printers, usually NULL to use the default settings. 
///                             Can be used for modifying printer driver behavior directly. 
/// \param[in] pszCupsOptions   CUPS options string which is passed directly to CUPS (Linux/UNIX only).
///                             Use NULL for default settings.
///
/// \see #TFormer_SetPrinterType, #TFormer_SetOutputName
/// 
/// \remarks
/// Specifies the output format as well as the target device or target filename. By using a prefix for the device name the output type may be specified (see table below). If a prefix is specified you can omit #TFormer_SetPrinterType.
/// This function must be called before #TFormer_Print. 
/// On Windows platforms the parameter pDevMode may be used to modify default GDI printer settings. All valid fields passed via pDevMode are merged with the default settings of the printer question.
/// For more information check out the Win32 SDK documentation.
///
/// <TABLE>
/// <CAPTION> Output Devices and Formats </CAPTION> 
/// <TR> <TD> Prefix			</TD><TD> Example										</TD><TD> Description                                                                       </TD> </TR>
/// <TR> <TD> No Prefix		</TD><TD> HP4050PS									</TD><TD> Direct printing to the printer specified. This method is only available under Microsoft Windows and uses GDI methods. Supply a printer name as defined by Windows (\\\\server\\printer, printer).</TD> </TR>
/// <TR> <TD> PDFFile:		</TD><TD> PDFFile:C:\\Temp\\out.pdf	</TD><TD> PDF-output is stored in C:\\temp\\out.pdf</TD> </TR>
/// <TR> <TD> PDF:				</TD><TD> PDF:C:\\Temp\\out.pdf			</TD><TD> PDF-output is stored in C:\\temp\\out.pdf</TD> </TR>
/// <TR> <TD> HTML:				</TD><TD> HTML:C:\\Temp\\out.html		</TD><TD> HTML-output stored in C:\\temp\out.html and automatically named subfolders</TD> </TR>
/// <TR> <TD> PS:					</TD><TD> PS:C:\\Temp\\out.ps				</TD><TD> PostScript-output is stored in C:\\temp\\out.ps</TD> </TR>
/// <TR> <TD> PSPRINTER:	</TD><TD> PSPRINTER:HP4050PS				</TD><TD> Direct printing to the printer specified. Because the output format is already PostScript a generic ASCII printer driver must be used.</TD> </TR>
/// <TR> <TD> TXT:				</TD><TD> TXT:C:\\temp\\out.txt			</TD><TD> Plain text output is stored in C:\\temp\\out.txt</TD> </TR>
/// <TR> <TD> TXTPRINTER:	</TD><TD> TXTPRINTER:ZEBRA4M				</TD><TD> Direct printing to the printer specified. Because the output format is plain text which may contain printer control sequences a generic ASCII printer driver must be used.</TD> </TR>
/// <TR> <TD> REPO:				</TD><TD> REPO:PrinterName					</TD><TD> Direct printing to the printer specified in the Repository. This method is only available under Microsoft Windows and uses GDI methods. </TD> </TR>
/// <TR> <TD> IMGBMP:     </TD><TD> IMGBMP:C:\\temp\out.bmp   </TD><TD> The BMP image file named out.bmp is stored in C:\\temp. If the output consists of multiple pages, multiple output files are generated. In this case the page number is appended to the filename (e.g. out_001.bmp). As an alternative you can use the # character as placeholder for the pagenumer: e.g. out###.bmp</TD> </TR>
/// <TR> <TD> IMGGIF:     </TD><TD> IMGGIF:C:\\temp\out.gif   </TD><TD> The GIF image file named out.gif is stored in C:\\temp. If the output consists of multiple pages, multiple output files are generated. In this case the page number is appended to the filename (e.g. out_001.gif). As an alternative you can use the # character as placeholder for the pagenumer: e.g. out###.gif</TD> </TR>
/// <TR> <TD> IMGJPG:     </TD><TD> IMGJPG:C:\\temp\out.jpg   </TD><TD> The JPG image file named out.jpg is stored in C:\\temp. If the output consists of multiple pages, multiple output files are generated. In this case the page number is appended to the filename (e.g. out_001.jpg). As an alternative you can use the # character as placeholder for the pagenumer: e.g. out###.jpg</TD> </TR>
/// <TR> <TD> IMGPCX:     </TD><TD> IMGPCX:C:\\temp\out.pcx   </TD><TD> The PCX image file named out.pcx is stored in C:\\temp. If the output consists of multiple pages, multiple output files are generated. In this case the page number is appended to the filename (e.g. out_001.pcx). As an alternative you can use the # character as placeholder for the pagenumer: e.g. out###.pcx</TD> </TR>
/// <TR> <TD> IMGPNG:     </TD><TD> IMGPNG:C:\\temp\out.png   </TD><TD> The PNG image file named out.png is stored in C:\\temp. If the output consists of multiple pages, multiple output files are generated. In this case the page number is appended to the filename (e.g. out_001.png). As an alternative you can use the # character as placeholder for the pagenumer: e.g. out###.png</TD> </TR>
/// <TR> <TD> IMGTGA:     </TD><TD> IMGTGA:C:\\temp\out.tga   </TD><TD> The TGA image file named out.tga is stored in C:\\temp. If the output consists of multiple pages, multiple output files are generated. In this case the page number is appended to the filename (e.g. out_001.tga). As an alternative you can use the # character as placeholder for the pagenumer: e.g. out###.tga</TD> </TR>
/// <TR> <TD> IMGTIF:     </TD><TD> IMGTIF:C:\\temp\out.tif   </TD><TD> The TIF image file named out.tif is stored in C:\\temp. If the output consists of multiple pages, multiple output files are generated. In this case the page number is appended to the filename (e.g. out_001.tif). As an alternative you can use the # character as placeholder for the pagenumer: e.g. out###.tif</TD> </TR>
/// <TR> <TD> IMGMULTITIF:</TD><TD> IMGMULTITIF:C:\\temp\out.tif</TD><TD>The TIF image file named out.tif is stored in C:\\temp. All pages are stored in a single output file.</TD> </TR>
/// <TR> <TD> ZPL:        </TD><TD> ZPL:C:\\temp\out.zpl      </TD><TD> ZPL-II output (suitable for ZEBRA printers) is stored in C:\\temp\\out.zpl.</TD> </TR>
/// <TR> <TD> ZPLPRINTER: </TD><TD> ZPLPRINTER:ZEBRA4M        </TD><TD> Direct printing in ZPL-II format to the operating system printer named ZEBRA4M. Because the output format is already ZPL-II a generic ASCII printer driver must be used.</TD> </TR>
/// </TABLE>
///
/// \note 
/// <em>TFORMer</em> uses a configuration file named TFORMer.xml for adjusting output options and paper format names. Since paper format names are also used in JobTrayControl please check out #TFormer_SetConfigFile too.
///
/// \note
/// Usually this function is used instead of #TFormer_SetPrinterType and #TFormer_SetOutputName. If you print to a Windows printer you can use #TFormer_SetOutputName to specify the name of the spool job. 
///
/// \ingroup group_Print
// ----------------------------------------------------------------------
/*
#### TODO add Help for CupsOptions samples ####
sides=none
sides=two-sided-long-edge      vertical duplex
sides=two-sided-short-edge     horizontal duplex
*/

TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_SetPrinterName
(
 HTFORM                     pInstance               
 ,LPCSTR                    pszPrinterName          
#ifdef _WIN32                                      
 ,DEVMODE*                  pDevMode                
#elif defined _TEC_UNIX
 ,LPCSTR                    pszCupsOptions          
#endif
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_SetPrinterNameW
(
 HTFORM                     pInstance               
 ,LPCWSTR                   pszPrinterName          
#ifdef _WIN32                                      
 ,DEVMODE*                  pDevMode                
#elif defined _TEC_UNIX
 ,LPCWSTR                   pszCupsOptions          
#endif
);
#endif // #ifndef _TEC_UNIX




/*
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_SetBCDrawMode
(
 HTFORM                     pInstance               
 ,EPrinter_BCDrawMode       eDrawMode
);
*/


// ----------------------------------------------------------------------
/// Specifies the name of the JobTrayControl to be used for this Job.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance          The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszTrayControlName Name of the JobTrayControl as defined with <em>TFORMer Designer</em> - use NULL or "" to use the <c>_Default_</c> JobTrayControl
///
/// \remarks
/// Specify the JobTrayControl for the current Job. 
/// If this property is not set then the Job is generated with the JobTrayControl named <c>_Default_</c>. 
///
/// This function must be called before #TFormer_Print. 
///
/// \note 
/// <em>TFORMer</em> uses a configuration file named <c>TFORMer.xml</c> for adjusting output options and paper format names. Since paper format names are also used in JobTrayControl please check out #TFormer_SetConfigFile too.
/// 
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_SetJobTrayControlName
(
 HTFORM                     pInstance               
 ,LPCSTR                    pszTrayControlName      
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_SetJobTrayControlNameW
(
 HTFORM                     pInstance               
 ,LPCWSTR                   pszTrayControlName      
);
#endif // #ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Specifies the Project-name for printing a Repository-based FormLayout. 
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszProjectName  The Project name as defined using <em>TFORMer Designer</em>.
///
/// \remarks
/// When using a Repository-based FormLayout use this function to provide the Project-name. In addition the name of the FormLayout must be set using #TFormer_SetFormName. 
/// \n
/// Do not use this property for a stand-alone FormLayout.
/// \n\n
/// See #TFormer_SetRepositoryName for a detailed explanation of a Repository and a stand-alone FormLayout.
///
/// \Note 
/// Call this function before #TFormer_SetFormName. 
/// Cached data (set with #TFormer_NewRecord and #TFormer_SetVarValue or by importing data-files) as well as the actual FormLayout are discarded.
///
/// \see #TFormer_SetRepositoryName, #TFormer_SetFormName. 
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetProjectName 
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetProjectNameW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszProjectName          
);
#endif // #ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Specifies the name of the Repository-based FormLayout. 
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFormName     The name of the FormLayout as defined using <em>TFORMer Designer</em>.
///
/// \remarks
/// Use this function to specify the name of the FormLayout which is referenced from within a Project of a Repository. 
/// \n
/// Do not use this property for a stand-alone FormLayout.
/// \n\n
/// See #TFormer_SetRepositoryName for a detailed explanation of a Repository and a stand-alone FormLayout.
///
/// \note
/// Must be called before #TFormer_Print. This call is mandatory if used with a Repository. 
/// Cached data (set with #TFormer_NewRecord and #TFormer_SetVarValue or by importing data-files) as well as the already loaded FormLayout are discarded.
///
/// \see #TFormer_SetRepositoryName, #TFormer_SetProjectName. 
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetFormName 
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init           */
,LPCSTR                     pszFormName             /* IN : FormLayout Name - as defined with <em>TFORMer Designer</em>  */
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetFormNameW
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init           */
,LPCWSTR                    pszFormName             /* IN : FormLayout Name - as defined with <em>TFORMer Designer</em>  */
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Specifies the name of the user-defined DataSource. 
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance          The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszDatasourceName  The name of the DataSource as defined with <em>TFORMer Designer</em>.
///
/// \remarks
/// Use this function to specify the name of the DataSource to be used for printing. 
/// \n
/// See #TFormer_SetRepositoryName for a detailed explanation of a Repository and a stand-alone FormLayout.
///
/// \note
/// Must be called before #TFormer_Print, but after #TFormer_SetFormName.
/// Cached data (set with #TFormer_NewRecord and #TFormer_SetVarValue or by importing data-files) are discarded.
///
/// \see #TFormer_SetRepositoryName, #TFormer_SetProjectName, #TFormer_SetFormName, #TFormer_SetDSParameterValue, #TFormer_GetDatasourceName. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetDatasourceName 
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init           */
 ,LPCSTR                     pszDatasourceName      /* IN : DataSource Name - as defined with <em>TFORMer Designer</em>  */
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetDatasourceNameW
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init           */
 ,LPCWSTR                   pszDatasourceName       /* IN : DataSource Name - as defined with <em>TFORMer Designer</em>  */
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Provides the name of the active DataSource of the current FormLayout. 
///
/// \returns - The name of the active DataSource of the current FormLayout. This name is only valid if peCode is ErrOk (0).
///
/// \param[in] pInstance          The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[out] peCode            Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// Use this function to get the name of the active DataSource of the current FormLayout. If the manual datasource is the active DataSource, this functions returns an empty string.
/// \n
/// \note
/// Must be called after #TFormer_SetFormName.
///
/// \see #TFormer_SetRepositoryName, #TFormer_SetProjectName, #TFormer_SetFormName, #TFormer_SetDSParameterValue. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
LPCSTR   __stdcall         TFormer_GetDatasourceName 
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init           */
 ,ERRCODE*                  peCode   
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
LPCWSTR   __stdcall         TFormer_GetDatasourceNameW
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init           */
 ,ERRCODE*                  peCode   
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Specifies the value of a DataSource-Parameter.  
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance          The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszParameterName   The name of the DataSource-Parameter to be set.
/// \param[in] pszValue           The value to be set.
///
/// \remarks
/// Use this function to specify a value for a DataSource-Parameter. A DataSource-Parameter may be defined by the user with <em>TFORMer Designer</em> and can be used to control certain aspects of a DataSource during runtime (e.g. filenames, SQL statement or an ODBC-DSN). 
/// \n
/// See #TFormer_SetRepositoryName for a detailed explanation of a Repository and a stand-alone FormLayout.
///
/// \note
/// Must be called before #TFormer_Print, but after #TFormer_SetDatasourceName or if a parameter of the active Datasource should be set, after #TFormer_SetFormName.
///
/// \see #TFormer_SetRepositoryName, #TFormer_SetProjectName, #TFormer_SetFormName, #TFormer_SetDatasourceName, #TFormer_GetDatasourceName. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetDSParameterValue
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init           */
 ,LPCSTR                    pszParameterName        /* IN : DataSource-Parameter Name - as defined with <em>TFORMer Designer</em>  */
 ,LPCSTR                    pszValue                /* IN : Value - to be set */
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetDSParameterValueW
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init           */
 ,LPCWSTR                   pszParameterName        /* IN : DataSource-Parameter Name - as defined with <em>TFORMer Designer</em>  */
 ,LPCWSTR                   pszValue                /* IN : Value - to be set */
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Sets a XML file in internal <em>TFORMer</em> format as DataSource.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFileName     Data File Name (in internal <em>TFORMer</em> XML format)
///
/// \remarks
/// Usually the methods #TFormer_SetVarValue and #TFormer_NewRecord are used to hand over variable data to <em>TFORMer Runtime</em>. 
/// However - with this function you can provide a data file, which contains all required input data (DataFields).
/// \n
/// The specified data file must be conforming to the internal <em>TFORMer</em> format. Normally this function is not required by users. Instead, check out the functions #TFormer_SetTxtDataFile or #TFormer_SetXmlDataFile.
/// If no data was supplied (either by TFormer_SetXXXDataFile or by #TFormer_SetVarValue) <em>TFORMer Runtime</em> tries to use the default data-file which was generated by the last <em>TFORMer Designer</em> session for this FormLayout. 
///
/// This function enables <em>TFORMer Runtime</em> to use XML-files (in a format which is used internally by <em>TFORMer</em>) as a DataSource for printing a Job. 
/// The specified XML-file must comply to the formats recognized by <em>TFORMer</em>. The following XML formats can be used: 
/// - Variant 1 of the generic XML-format (see #TFormer_SetXmlDataFile).
/// - Variant 2 of the generic XML-format (see #TFormer_SetXmlDataFile).
/// - The XML-format used in current versions of Microsoft Access (see #TFormer_SetXmlDataFile).
/// - The XML-format used by <em>TFORMer</em> internally (this function).
///
/// \note
/// Behavior if no input data was supplied: If no DataSource was specified (either by TFormer_SetXXXDataFile, #TFormer_SetODBCData or via #TFormer_NewRecord and #TFormer_SetVarValue) the default data-file 
/// (located in the same directory as the FormLayout itself and generated by the last <em>TFORMer Designer</em> session) is used. If this default data-file does not exists no output will be produced.
/// The name of the default data-file is <c>path/FormName.xml</c>. 
///
/// \note
/// A data file may be passed in the following ways:
///
/// - Specify the filename directly.
/// - Specify the filename of a ZIP-file (e.g. with WinZip) containing the required file.
/// - BASE64-encode the content of the required file (see Convert class of .NET) and use this string instead of a filename.
/// - BASE64-encode the content of a ZIP-file (see Convert class of .NET) containing the required file(s). Use this string instead of a filename. 
///
/// \note 
/// BASE64 encoded strings <b>must</b> start with the sequence <c>BASE64:</c>. When passing a zipped file <em>TFORMer</em> expects a real WinZip compatible format. A zipped stream is not suitable!
/// For more details please refer to the <em>TFORMer Runtime DLL Programming Reference</em>.
/// 
/// \note
/// Must be called before #TFormer_Print. When using a Repository-based FormLayout call this function after #TFormer_SetFormName, if using a stand-alone FormLayout (*.tff) then call this function after #TFormer_SetRepositoryName.  
/// 
/// \see #TFormer_SetRepositoryName, #TFormer_SetFormName. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetDataFile 
(
 HTFORM                     pInstance               
,LPCSTR                     pszFileName             
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetDataFileW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszFileName             
);
#endif // #ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Resets the data-cache.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
///
/// \remarks
/// All DataFields handed over to <em>TFORMer</em> with #TFormer_SetVarValue, #TFormer_NewRecord or #TFormer_SetDataFile are cached internally. 
/// As long as this cache is not deleted with this function the cached values will be used for a print-job. 
/// Call this method before starting a new print-job.
///
/// \note
/// If you change the Repository, the Project or the FormLayout the internal cache is deleted automatically. 
///
/// \see #TFormer_NewRecord, #TFormer_SetVarValue, #TFormer_SetDataFile. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_ResetData 
(
 HTFORM                     pInstance
);

// ----------------------------------------------------------------------
/// Appends a new Record to the DataSource. 
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
///
/// \remarks 
/// A Record is a collection of name/value pairs. This pair holds the name of a DataField (first member of the pair) and its value (second member of the pair). 
/// \n
/// Call this function before you set DataField values for the new Record. 
///
/// \see #TFormer_SetRecordCopy, #TFormer_SetVarValue, #TFormer_ResetData. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_NewRecord 
(
 HTFORM                     pInstance               
);


// ----------------------------------------------------------------------
/// Sets the number of copies for the actual Record. 
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] nRecordCopy     The number of copies for the current record.
///
/// \remarks
/// The value provided with this function specifies how often this Record is used for printing. 
/// May be called after #TFormer_NewRecord. If #TFormer_SetRecordCopy is not set, then the default value 1 will be used.
///
/// \note
/// Do not mix up this method with #TFormer_SetNumberOfCopies
///
/// \see #TFormer_NewRecord, #TFormer_SetVarValue. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetRecordCopy 
(
 HTFORM                     pInstance
,UINT                       nRecordCopy
);

// ----------------------------------------------------------------------
/// Sets the value of a DataField. 
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] szVarName       The name of the DataField for which the value should be set.
/// \param[in] szVarValue      The value for the given DataField.
///
/// \remarks
/// Must be called for each DataField after a call to #TFormer_NewRecord. 
/// All data is passed to <em>TFORMer</em> as string - <em>TFORMer</em> converts this string automatically into the specified data-type (see #EVarType). 
/// If a validation rule is violated (see <em>TFORMer Designer</em>), an error-code is returned.
/// If this function is not called for a DataField used in a FormLayout, then the content of this DataField depends on #TFormer_SetReuseValues.
/// \n\n
/// If you want to identify the DataFields which are available in a Project or FormLayout, please use the function #TFormer_GetFirstProjectVarIt.
/// 
/// \note
/// This function will test if\n
/// - the given value confirms to the validation rules set in <em>TFORMer Designer</em>
/// - the given value can be converted to the type of the DataField (Float, Long). A conversion to String is always possible.
///\n If one of this conditions is not met, an error will be returned. Detailed information about the error can be obtained by calling #TFormer_GetLastErrorDescription, #TFormer_GetLastErrorInformation, #TFormer_GetLastErrorSource.
///
/// \see #TFormer_SetReuseValues, #TFormer_NewRecord. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetVarValue 
(
 HTFORM                     pInstance               
,LPCSTR                     szVarName               
,LPCSTR                     szVarValue              
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetVarValueW
(
 HTFORM                     pInstance               
,LPCWSTR                    szVarName               
,LPCWSTR                    szVarValue              
);
#endif // #ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Selects if DataField-values should be reused in consecutive Records.
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] bReuse          Should data be reused for each record?
///
/// \remarks
/// - If set to true, then the value of a DataField will be preserved between Records. \n 
///   The last value will be reused in consecutive Records until a new value is set.
///   This allows you to omit #TFormer_SetVarValue for a DataField if the value should be the same as in the previous Record. 
/// - If set to false (which is the default), then all DataFields are set to "" (empty string) or 0 (depending on the DataFieldType) between Records.
/// It is recommended to set all DataFields for each Record.
///
/// \see #TFormer_NewRecord, #TFormer_SetVarValue. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetReuseValues 
(
 HTFORM                     pInstance               
,BOOL                       bReuse                  
);

// ----------------------------------------------------------------------
/// Selects whether escape sequences like \\n or \\x40 should be translated by <em>TFORMer</em>. 
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] bOn             Should escape sequences be translated?
///
/// \remarks
/// The default value is FALSE. That means <em>TFORMer</em> uses DataField-values exactly as provided by the application, the user or the DataSource. 
/// But in some cases it is required to translate JavaScript-style escape sequences into their corresponding binary representation.
/// If set to TRUE, escape sequences in DataField values will be translated.
/// This method influences all subsequent calls to TFormer_SetXXXDataFile or #TFormer_SetVarValue.
///
/// Each escape sequence begins with a leading backslash. The following escape sequences are supported by <em>TFORMer</em>:
/// <TABLE>
/// <CAPTION> Translated Escape Sequences </CAPTION> 
/// <TR> <TD> Sequence      </TD><TD> Character </TD> </TR>
/// <TR> <TD> \\'						</TD><TD>Single quotation mark                  																					</TD> </TR>
/// <TR> <TD> \\"						</TD><TD>Double quotation mark                  																					</TD> </TR>
/// <TR> <TD> \\\\\					</TD><TD>Backslash                              																					</TD> </TR>
/// <TR> <TD> \\b						</TD><TD>Backspace                              																					</TD> </TR>
/// <TR> <TD> \\f						</TD><TD>Form feed                              																					</TD> </TR>
/// <TR> <TD> \\n						</TD><TD>New line                               																					</TD> </TR>
/// <TR> <TD> \\r						</TD><TD>Carriage return                        																					</TD> </TR>
/// <TR> <TD> \\t						</TD><TD>Horizontal tab                         																					</TD> </TR>
/// <TR> <TD> \\ddd					</TD><TD>Octal sequence (3 digits: ddd)         																					</TD> </TR>
/// <TR> <TD> \\xdd					</TD><TD>Hexadecimal sequence (2 digits: dd)																							</TD> </TR>
/// <TR> <TD> \\udddd				</TD><TD>Not implemented in this <em>TFORMer</em> Version. Unicode sequence (4 hex digits: dddd)		</TD> </TR>
/// </TABLE>
/// In case of an invalid escape sequence (e.g. \\q, \\999, \\129, \\xgg) the backslash is ignored. No error will be raised. 
/// <TABLE>
/// <CAPTION> Invalid Escape Sequences </CAPTION> 
/// <TR> <TD> Invalid Escape Sequence in Data </TD><TD>Result				</TD> </TR>
/// <TR> <TD> \\q															</TD><TD>q						</TD> </TR>
/// <TR> <TD> \\999														</TD><TD>999					</TD> </TR>
/// <TR> <TD> \\609														</TD><TD>09						</TD> </TR>
/// <TR> <TD> \\xgg														</TD><TD>xgg					</TD> </TR>
/// <TR> <TD> \\															</TD><TD>Empty string	</TD> </TR>
/// </TABLE>
///
/// \see #TFormer_SetVarValue. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetHandleEscapeSequences
(
 HTFORM                     pInstance               
,BOOL                       bOn                     
);


// ----------------------------------------------------------------------
/// Sets the number of document copies which should be printed. 
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] nCopies         Number of document copies
///
/// \remarks
/// The default value is 1. This function must be called before #TFormer_Print to instruct <em>TFORMer Runtime</em> to produce multiple copies (using the same data and FormLayout) of one print job.
///
/// \see #TFormer_Print. 
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetNumberOfCopies
(
 HTFORM                     pInstance               
,INT                        nCopies
); 

// ----------------------------------------------------------------------
/// Selects the starting cell of the first label to be printed.
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] nStartCol       Only useful for Label-printing. Specifies the row-number of the first label to be printed (topmost row = 0).
/// \param[in] nStartRow       Only useful for Label-printing. Specifies the column-number of the first label to be printed (leftmost column = 0).
///
/// \remarks
/// This functions sets the row- and column number of the first label to be printed. 
/// Usually <em>TFORMer</em> prints the first label of a label-style report at the first available position of a page (this is the top / left label). 
/// Numbering starts with 0 (leftmost column = 0/topmost row = 0). Call this function before #TFormer_Print otherwise the default will be used (the label in the upper left corner of the page (0, 0)).
///
/// \see #TFormer_Print. 
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetStartRowColumn
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init */
,INT                        nStartCol               /* IN : Starting column [0..n]              */
,INT                        nStartRow               /* IN : Starting row [0..n]                 */
);

// ----------------------------------------------------------------------
/// This method is used for adjusting the output filename or the name of the spool-job (on Microsoft Windows printers).
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszOutputName   The output filename or the name of the spool-job (on Microsoft Windows printers).
///
/// \remarks
/// Sets the filename or the name of the spool-job used for printing (as displayed in the spooler). Call before #TFormer_Print is invoked - otherwise the default values will be used. To reset this value to default call #TFormer_SetOutputName (pInstance, NULL).
/// \note
/// Use this function in conjunction with #TFormer_SetPrinterType to specify the type and the target of the generated output.
/// Alternatively you can use #TFormer_SetPrinterName - this function specifies output type and target device within one string. 
///
/// \deprecated
/// This functionality will be modified in future versions.
///
/// \see #TFormer_Print. 
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetOutputName
(
 HTFORM                     pInstance               
,LPCSTR                     pszOutputName           
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetOutputNameW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszOutputName           
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Sets the range of page-numbers to be the printed.
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] nPageFrom       Starting page of print-job (all pages with smaller page numbers are not printed, default is 1)
/// \param[in] nPageTo         Ending page of print-job (all pages with higher page numbers are not printed, default is UINT_MAX, or 0xffffffff)
///
/// \remarks
/// If required this function must be called before #TFormer_Print.
/// To restore this setting (printing of all pages of the job), call the following command: 
/// \code
/// TFormer_SetPageFromTo (pInstance, 0, 0);
/// \endcode
///
/// \see #TFormer_Print. 
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetPageFromTo
(
 HTFORM                     pInstance              
,UINT                       nPageFrom              
,UINT                       nPageTo                
);

// ----------------------------------------------------------------------
/// Licenses this <em>TFORMer Runtime</em> instance.
/// 
/// \returns always returns ErrOk (0) - even if license key is invalid
///
/// \param[in] pszLicensee                Licensee name (provided by TEC-IT)
/// \param[in] dwKind                     License kind (provided by TEC-IT)
/// \param[in] nLicenses                  Number of licenses (provided by TEC-IT)
/// \param[in] pszLicenseKey              License key (provided by TEC-IT)
/// 
/// \remarks
/// Once the <em>TFORMer Runtime</em> is licensed all restrictions of the demo version are removed. 
/// The method needs to be called prior to any call of #TFormer_Print. 
/// It is recommended to call #TFormer_License once immediately after loading <em>TFORMer Runtime</em>. 
/// \n
/// You will receive the license information from TEC-IT Datenverarbeitung GmbH after ordering (http://www.tec-it.com/order). 
/// All license values must be specified exactly as received from TEC-IT. 
/// The license kind, the number of licenses and the licensee string are checked against the license key.
/// \ingroup group_General
// ----------------------------------------------------------------------
TECIT_DLLSPEC
void      __stdcall     TFormer_License
(
 LPCSTR                 pszLicensee,                
 DWORD                  dwKind,                     
 LONG                   nLicenses,                  
 LPCSTR                 pszLicenseKey               
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
void      __stdcall     TFormer_LicenseW
(
 LPCWSTR                pszLicensee,                
 DWORD                  dwKind,                     
 LONG                   nLicenses,                  
 LPCWSTR                pszLicenseKey               
);
#endif // #ifndef _TEC_UNIX

#ifdef _WIN32
// ----------------------------------------------------------------------
/// Determines if the <em>TFORMer Designer</em> is available on the local machine and can be invoked by the method #TFormer_OpenDesigner.
/// 
/// \returns - TRUE in case of an existing <em>TFORMer Designer</em> installation\n - FALSE if <em>TFORMer Designer</em> is not available
///
/// \remarks
/// The function tests if <em>TFORMer Designer</em> <c>TFormer.exe</c> is available on the system. If the <em>TFORMer Designer</em> is installed, FormLayouts can be edited with <em>TFORMer Designer</em>.
///
/// \see #TFormer_OpenDesigner. 
/// \ingroup group_Misc
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
BOOL   __stdcall        TFormer_IsDesignerAvailable ();

// ----------------------------------------------------------------------
/// Opens the currently selected FormLayout with <em>TFORMer Designer</em>.  
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance            The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszDesignExe         if NULL the function tries to locate <em>TFORMer Designer</em> by itself
/// \param[in] bShowSplash          Show the splash screen of <em>TFORMer Designer</em>?
/// \param[in] bMaximize            Open <em>TFORMer Designer</em> maximized?
/// \param[in] bUseExistingInstance Try to use an existing <em>TFORMer Designer</em> instance? \n
///                                 This parameter avoids multiple <em>TFORMer Designer</em> windows. A new instance of <em>TFORMer Designer</em> is opened if no existing instance is found or no existing instance can be used.
/// \param[in] pszExtraParameters   Parameters which should be sent to <em>TFORMer Designer</em> (Default: NULL)\n
///                                 These parameters are used for branding of <em>TFORMer Designer</em> via the command-line (see below).
///
/// \remarks
/// <em>TFORMer Runtime</em> on its own cannot be used for editing a FormLayout. Use this method to invoke <em>TFORMer Designer</em>. 
/// \remarks
/// - If there is no given EXE file (parameter pszDesignExe), then this function uses a value in the System Registry ([HKCU|HKLM]\\TEC-IT\\TEC-IT Datenverarbeitung GmbH\\6.0\\TFormerApp) to determine the location of TFORMer.exe. 
/// - If this key is not found or the file doesn't exist, then TFORMer.exe will be searched in the directory where <em>TFORMer Runtime</em> is installed. 
/// - If TFORMer.exe is not found here too, then it will search TFORMer.exe in the current directory. 
/// - Finally - if TFORMer.exe was still not found it checks the complete path (environment variable PATH). 
/// 
/// \note
/// Make sure that your purchased license model fits your requirements. If you are creating applications which are distributed to 3rd party you may need to redistribute <em>TFORMer Designer</em> as well. 
///
/// \note
/// #TFormer_SetFormName has to be called prior to calling this function.
///
/// \note 
/// For branding <em>TFORMer Designer</em> there are two different possibilities:\n
/// - Provide Splash-Screen and About Box bitmaps in the same directory as <em>TFORMer Designer</em> is installed. The required filenames are TFSS.mbp (Splash-Screen) and TFAb.mbp (About Box)
/// - Via command-line parameters (these parameters overrule the file-system):\n
///   /Splash [SplashPicture.bmp] \n
///   /AboutBox [AboutboxPicture.bmp]
///
/// \see #TFormer_IsDesignerAvailable. 
/// \ingroup group_Misc
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_OpenDesigner 
(
 HTFORM                     pInstance               
,LPCSTR                     pszDesignExe            
,BOOL                       bShowSplash             
,BOOL                       bMaximize               
,BOOL                       bUseExistingInstance    
,LPCSTR                     pszExtraParameters      
);

TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_OpenDesignerW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszDesignExe            
,BOOL                       bShowSplash             
,BOOL                       bMaximize               
,BOOL                       bUseExistingInstance    
,LPCWSTR                    pszExtraParameters      
);
#endif // _WIN32

// ----------------------------------------------------------------------
/// Specifies which DataField in the input controls the number of Record copies.
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance            The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFieldName         The DataField name which holds the record-copy counter. NULL if none should be used (default: NULL).
/// \param[in] bIsAttribute         Specifies if the given DataField name is the name of an attribute field. Used only for XML-Imports.
///
/// \remarks
/// <em>TFORMer</em> allows you to specify how often one Record of a DataSource should be used for printing (see also #TFormer_SetRecordCopy). 
/// This functions specifies which DataField holds the number of copies for each Record of the DataSource. 
/// \note
/// Must be called before #TFormer_SetTxtDataFile or #TFormer_SetXmlDataFile.
/// \see #TFormer_SetTxtDataFile, TFormer_SetXmlDataFile. 
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetImportFieldForRecordCopy
(
 HTFORM                     pInstance               
,LPCSTR                     pszFieldName            
,BOOL                       bIsAttribute            
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetImportFieldForRecordCopyW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszFieldName            
,BOOL                       bIsAttribute            
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Specifies a text-file as DataSource.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance            The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFileName          Data-File Name.
/// \param[in] cSeparator           DataField separator (e.g. , or ;).
/// \param[in] cTextQualifier       Text qualifier (e.g. ' or " - default is none).
///
/// \remarks 
/// This DataSource enables <em>TFORMer Runtime</em> to use CSV-files (or more generic: any text file in a suitable format) as a DataSource for printing a Job. 
/// \n
/// The data in such a file provides the values of the DataFields. 
/// Each line (except the first one - see below) is treated as a Record. 
/// <em>TFORMer Runtime</em> must match the values in the file with the corresponding DataField-names. 
/// Therefore the first row of the file must contain the DataField-names (see #TFormer_SetVarValue), separated by a Separator-character. 
/// All other rows contain the DataField-values, again separated by a Separator. 
/// \n
/// Each value or name of a DataField may be enclosed in optional Qualifier-characters. This is useful if the Separator-character may be a part of a value. 
///
/// \note
/// Behavior if no input data was supplied: If no DataSource was specified (either by TFormer_SetXXXDataFile, #TFormer_SetODBCData or via #TFormer_NewRecord and #TFormer_SetVarValue) the default data-file 
/// (located in the same directory as the FormLayout itself and generated by the last <em>TFORMer Designer</em> session) is used. If this default data-file does not exists no output will be produced.
/// The name of the default data-file is <c>path/FormName.xml</c>. 
///
/// \note
/// A data file may be passed in the following ways:
///
/// - Specify the filename directly.
/// - Specify the filename of a ZIP-file (e.g. with WinZip) containing the required file.
/// - BASE64-encode the content of the required file (see Convert class of .NET) and use this string instead of a filename.
/// - BASE64-encode the content of a ZIP-file (see Convert class of .NET) containing the required file(s). Use this string instead of a filename. 
///
/// \note 
/// BASE64 encoded strings <b>must</b> start with the sequence <c>BASE64:</c>. When passing a zipped file <em>TFORMer</em> expects a real WinZip compatible format. A zipped stream is not suitable!
/// For more details please refer to the <em>TFORMer Runtime DLL Programming Reference</em>.
/// 
/// \note
/// Must be called after #TFormer_SetFormName and before #TFormer_Print.
///
/// The following example file contains 2 records. The first line lists the DataField-names. The comma (,) is used as Separator-character and the double quote (") is used as Qualifier-character: 
/// <TABLE>
/// <CAPTION> Sample CSV-File </CAPTION> 
/// <TR><TD>Name/Value					</TD><TD>Separator</TD><TD>Name/Value					</TD><TD>Separator</TD><TD>Name/Value								</TD></TR>
/// <TR><TD>"Email"							</TD><TD>,        </TD><TD>"JobTitle"					</TD><TD>,        </TD><TD>"Name"										</TD></TR>
///	<TR><TD>"support@tec-it.com"</TD><TD>,        </TD><TD>"Support Engineer"	</TD><TD>,        </TD><TD>"Mr. Harald Backoffice"	</TD></TR>
///	<TR><TD>"sales@tec-it.com"	</TD><TD>,        </TD><TD>"Sales Assistant"	</TD><TD>,        </TD><TD>"Mrs. Susan Frontdesk"		</TD></TR>
/// </TABLE>
///
/// \see #TFormer_SetXmlDataFile, #TFormer_SetImportFieldForRecordCopy.
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetTxtDataFile 
(
 HTFORM                     pInstance               
,LPCSTR                     pszFileName             
,CHAR                       cSeparator              
,CHAR                       cTextQualifier          
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetTxtDataFileW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszFileName             
,WCHAR                      cSeparator              
,WCHAR                      cTextQualifier          
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Specifies an ODBC DataSource.
/// 
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance    The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszDSN       DSN for ODBC-Connection
/// \param[in] pszUser      User-Name for ODBC-Connection
/// \param[in] pszPwd       Password for ODBC-Connection
/// \param[in] pszSQLSelect Select statement for fetching the data. Column names are treated as DataField names.
/// 
/// \remarks
/// This DataSource enables <em>TFORMer Runtime</em> to use arbitrary database-tables or queries as a DataSource for printing a Job. 
/// \n
/// The data in such a table provides the values of the DataFields. The table is read Record by Record using the specified SQL statement SqlSelect. 
/// <em>TFORMer Runtime</em> must be able to match the column-names of the table with the corresponding DataField-names as used in the FormLayout. 
/// If required use SQL aliases to provide identical column names. 
/// \n
/// The order of the columns in the table or query does not matter. 
///
/// \note
/// Behavior if no input data was supplied: If no DataSource was specified (either by TFormer_SetXXXDataFile, #TFormer_SetODBCData or via #TFormer_NewRecord and #TFormer_SetVarValue) the default data-file 
/// (located in the same directory as the FormLayout itself and generated by the last <em>TFORMer Designer</em> session) is used. If this default data-file does not exists no output will be produced.
/// The name of the default data-file is <c>path/FormName.xml</c>. 
///
/// This example demonstrates how to use the pre-installed ODBC DataSource (named TFORMer_Sample) for generating a report as PDF. 
/// The table tbl_Example looks like follows: 
/// <TABLE>
/// <CAPTION>Sample Table</CAPTION> 
/// <TR><TD>ChapterNumber	</TD><TD>Caption					</TD><TD>Pagenumber	</TD></TR>
/// <TR><TD>1							</TD><TD>Content					</TD><TD>2					</TD></TR>
/// <TR><TD>1.1						</TD><TD>Table of Figures	</TD><TD>6					</TD></TR>
/// <TR><TD>1.2						</TD><TD>List of Tables		</TD><TD>6					</TD></TR>
/// <TR><TD>2							</TD><TD>Disclaimer				</TD><TD>7					</TD></TR>
/// <TR><TD>...						</TD><TD>...							</TD><TD>...				</TD></TR>
/// </TABLE>
///
/// \see #TFormer_SetTxtDataFile, #TFormer_SetXmlDataFile, #TFormer_SetImportFieldForRecordCopy.
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetODBCData 
(
 HTFORM                     pInstance               
,LPCSTR                     pszDSN                  
,LPCSTR                     pszUser                 
,LPCSTR                     pszPwd                  
,LPCSTR                     pszSQLSelect            
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetODBCDataW
(
 HTFORM                     pInstance               
,LPCWSTR                     pszDSN                  
,LPCWSTR                     pszUser                 
,LPCWSTR                     pszPwd                  
,LPCWSTR                     pszSQLSelect            
);
#endif //#ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Specifies a XML-File as DataSource.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance    The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFileName  Data-File name
/// 
/// \remarks
/// Usually the methods #TFormer_SetVarValue and #TFormer_NewRecord are used to hand over variable data to <em>TFORMer Runtime</em>. 
/// This DataSource enables <em>TFORMer Runtime</em> to use XML-files as a DataSource for printing a Job. 
/// The specified XML-file must comply to the formats recognized by <em>TFORMer</em>. The following XML formats can be used: 
/// - Variant 1 of the generic XML-format (this function).
/// - Variant 2 of the generic XML-format (this function).
/// - The XML-format used in current versions of Microsoft Access (this function).
/// - The XML-format used by <em>TFORMer</em> internally (see #TFormer_SetDataFile).
/// .
/// \note
/// Behavior if no input data was supplied: If no DataSource was specified (either by TFormer_SetXXXDataFile, #TFormer_SetODBCData or via #TFormer_NewRecord and #TFormer_SetVarValue) the default data-file 
/// (located in the same directory as the FormLayout itself and generated by the last <em>TFORMer Designer</em> session) is used. If this default data-file does not exists no output will be produced.
/// The name of the default data-file is <c>path/FormName.xml</c>. 
///
/// \note
/// A data file may be passed in the following ways:
///
/// - Specify the filename directly.
/// - Specify the filename of a ZIP-file (e.g. with WinZip) containing the required file.
/// - BASE64-encode the content of the required file (see Convert class of .NET) and use this string instead of a filename.
/// - BASE64-encode the content of a ZIP-file (see Convert class of .NET) containing the required file(s). Use this string instead of a filename. 
///
/// \note 
/// BASE64 encoded strings <b>must</b> start with the sequence <c>BASE64:</c>. When passing a zipped file <em>TFORMer</em> expects a real WinZip compatible format. A zipped stream is not suitable!
/// For more details please refer to the <em>TFORMer Runtime DLL Programming Reference</em>.
/// 
/// \note
/// \par This is an example for the generic XML-format (Variant 1). 
/// \code
/// <Root>
///   <Records>
///     <Record>
///       <V Name="DataFieldName1">Value</V>
///       <V Name="DataFieldName2">Value</V>
///       <V Name="DataFieldName3">Value</V>
///     </Record>
///     <Record>
///       <V Name="DataFieldName1">Value</V>
///       <V Name="DataFieldName2">Value</V>
///       <V Name="DataFieldName3">Value</V>
///     </Record>
///     :
///     Further Data Records
///     :
///   </Records>
/// </Root>
/// \endcode
/// <TABLE>
/// <CAPTION> Description for XML-Elements in Variant 1</CAPTION>
/// <TR><TD>Name</TD><TD>Description</TD></TR>
/// <TR><TD>Root</TD><TD>This is the root element. It encloses all other elements of the XML-file. </TD></TR>
/// <TR><TD>Records</TD><TD>This element encloses all records.</TD></TR>
/// <TR><TD>Record</TD><TD>This element represents one Record. Within a Record the single DataField-values are defined.</TD></TR>
/// <TR><TD>V</TD><TD>Data-Element. This element defines the name of the DataField (see Name) and the value of the DataField.</TD></TR>
/// <TR><TD>DataFieldName</TD><TD>Name of the DataField (must be unique within a Record).</TD></TR>
/// <TR><TD>Value</TD><TD>The actual value of the DataField.</TD></TR>
/// </TABLE>
/// The elements can be named arbitrarily. This means the terms Root, Records and Record can be selected at will. Only the order of the element-nodes is decisive: <em>TFORMer</em> interprets the lowest level of the XML-file as name/value pairs for DataFields, superordinate nodes always correspond to records. 
/// \par This is an example for the generic XML-format (Variant 2). 
/// Variant 2 is comparable to variant 1, but instead of using the syntax \<V Name="DataFieldName"\>Value\</V\> you can specify DataField-values using the following syntax: 
/// \code
/// <Root>
///   <Records>
///     <Record>
///       <DataFieldName1>Value</DataFieldName1>
///       <DataFieldName2>Value</DataFieldName2>
///       <DataFieldName3>Value</DataFieldName3>
///     </Record>
///     <Record>
///       <DataFieldName1>Value</DataFieldName1>
///       <DataFieldName2>Value</DataFieldName2>
///       <DataFieldName3>Value</DataFieldName3>
///     </Record>
///     :
///     Further Data Records
///     :
///   </Records>
/// </Root>
/// \endcode
/// \par The XML files which are exported by Microsoft Access look like this: 
/// \code
/// <?xml version="1.0" encoding="UTF-8"?>
/// <dataroot xmlns:od="urn:schemas-microsoft-com:officedata">
///   <tblTableName>
///     <DataFieldName1>Value</DataFieldName1>
///     <DataFieldName2>Value</DataFieldName2>
///     <DataFieldName3>Value</DataFieldName3>
///   </tblTableName>
///   <tblTableName>
///     <DataFieldName1>Value</DataFieldName1>
///     <DataFieldName2>Value</DataFieldName2>
///     <DataFieldName3>Value</DataFieldName3>
///   </tblTableName>
///   :
///   Further Data Records
///   :
/// </dataroot>
/// \endcode
/// \par This is an example for the internal XML-format. <em>TFORMer Designer</em> uses this format to store DataField name/value pairs. 
/// \code
/// <PSData>
///   <Form>
///     <Records>
///       <Record Copies="1">
///         <V Name="DataFieldName1">Value</V>
///         <V Name="DataFieldName2">Value</V>
///         <V Name="DataFieldName3">Value</V>
///       </Record>
///       <Record Copies="1">
///         <V Name="DataFieldName1">Value</V>
///         <V Name="DataFieldName2">Value</V>
///         <V Name="DataFieldName3">Value</V>
///       </Record>
///     </Records>
///   <ImportedVariables/>
///   </Form>
/// </PSData>
/// \endcode
/// <TABLE>
/// <CAPTION> Description for XML-Elements in the internal <em>TFORMer</em> format</CAPTION>
/// <TR><TD>PSData</TD><TD>This is the root element. It encloses all other elements within the XML-file.</TD></TR>
/// <TR><TD>Form</TD><TD>This element encloses all records for one FormLayout. It must be specified only once.</TD></TR>
/// <TR><TD>Records</TD><TD>This element encloses all records to be printed. It must be specified only once.</TD></TR>
/// <TR><TD>Record</TD><TD>This element encloses all DataFields for one Record. For each Record there is one (optional) attribute named Copies. If this attribute is missing it defaults to 1  meaning that this Record is used once. If you want to use the same data for multiple records you can specify NumberOfCopies using this attribute.</TD></TR>
/// <TR><TD>V</TD><TD>Data-Element. This element consists of the DataField-name (attribute Name) and the value.</TD></TR>
/// <TR><TD>DataFieldName</TD><TD>Name of the DataField (must be unique within a Record).</TD></TR>
/// <TR><TD>Value</TD><TD>The value of the DataField.</TD></TR>
/// <TR><TD>ImportedVariables</TD><TD>Used internally by <em>TFORMer</em>.</TD></TR>
/// </TABLE>
///
/// \note
/// Must be called after #TFormer_SetFormName and before #TFormer_Print.
///
/// \see #TFormer_SetTxtDataFile, #TFormer_SetImportFieldForRecordCopy.
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetXmlDataFile 
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init     */
,LPCSTR                     pszFileName             /* IN : Data-File Name                          */
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetXmlDataFileW
(
 HTFORM                     pInstance               /* IN : TFORMer-Instance - see TFormer_Init     */
,LPCWSTR                    pszFileName             /* IN : Data-File Name                          */
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Specifies a XML-File as DataSource and transforms it with the given XSLT.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance			 The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFileName	   Data-file name
/// \param[in] pszXsltFileName XSLT Data-file name, if NULL then no transformation is done on given data file
/// \param[in] pszXsltTFC      XSLT TFC-file name. Internal Use ONLY (generates TFC File from given xml file)
/// 
/// \remarks
/// This DataSource enables <em>TFORMer Runtime</em> to use XML-files as a DataSource for printing a Job. 
/// The specified XML-file must comply to the formats recognized by <em>TFORMer</em>. The following XML formats can be used: 
/// - Variant 1 of the generic XML-format.
/// - Variant 2 of the generic XML-format.
/// - The XML-format used in current versions of Microsoft Access.
/// - The XML-format used by <em>TFORMer</em> internally (see #TFormer_SetDataFile).
///
/// \warning
/// Internal use only!
///
/// \see #TFormer_SetTxtDataFile, #TFormer_SetImportFieldForRecordCopy, #TFormer_SetXmlDataFile.
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetXmlDataFileEx 
(
 HTFORM                     pInstance               
,LPCSTR                     pszFileName             
,LPCSTR                     pszXsltFileName         
,LPCSTR                     pszXsltTFC              
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SetXmlDataFileExW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszFileName             
,LPCWSTR                    pszXsltFileName         
,LPCWSTR                    pszXsltTFC              
);
#endif // #ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Saves the actual data as XML File in the internal <em>TFORMer</em> format.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance    The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFileName  Data-File name to save the data to
/// 
/// \remarks
/// All DataFields set by TFormer_SetXXXDataFile, #TFormer_SetODBCData or by #TFormer_SetVarValue are cached internally. 
/// This function writes the cache to the specified file. 
/// The generated file can be reused later with #TFormer_SetDataFile.
///
/// \note
/// Can be called after #TFormer_SetXmlDataFile, #TFormer_SetTxtDataFile, #TFormer_SetODBCData or after #TFormer_NewRecord.
/// \see #TFormer_SetTxtDataFile, #TFormer_SetXmlDataFile, #TFormer_SetODBCData.
///
/// \ingroup group_Data
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SaveXml 
(
 HTFORM                     pInstance               
,LPCSTR                     pszFileName             
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_SaveXmlW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszFileName             
);
#endif //#ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Sets the configuration file for <em>TFORMer Runtime</em>.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance    The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszFilename  Path and filename of a <em>TFORMer</em> configuration file
/// \remarks
/// <em>TFORMer</em> uses a system-wide configuration file named <c>TFORMer.xml</c> which defines general output options. 
/// This file is installed automatically by the setup application (usually into <c>C:\\Program Files\\TEC-IT\\TFORMer 6\\Bin</c> (Windows) or <c>/usr/local/share/TFORMer</c> (Linux)). 
/// #TFormer_SetConfigFile enables you to use different or multiple configuration-files (e.g. on a per-user base). 
/// Please refer to the <c>TFORMer Developer Manual</c> and the file <c>TFORMer.xml</c> to get a detailed explanation regarding the following settings: 
/// <TABLE>
/// <CAPTION> Option Overview </CAPTION> 
/// <TR> <TD> Output Type      </TD><TD> Description                                                                       </TD> </TR>
/// <TR> <TD> All Output Types </TD><TD> 
///                                      - Error handling for bar-codes, images, expressions and UNICODE fonts
///                                      - Paper format names (see #TFormer_SetJobTrayControlName)                         </TD> </TR>
/// <TR> <TD> PDF              </TD><TD> 
///                                      - Page setup (size, orientation, compression, margins, font-embedding)
///                                      - Image compression method and JPEG-quality
///                                      - Maximum resolution for images (so-called down-sampling)                         </TD> </TR>    
/// <TR> <TD> PostScript       </TD><TD> 
///                                      - Page setup (size, orientation, color-mode)
///                                      - Header and footer sequences, character encoding                                 </TD> </TR>
/// <TR> <TD> HTML             </TD><TD> 
///                                      - Page setup (size, orientation, resolution)
///                                      - Bar-code resolution                                                             </TD> </TR>
/// <TR> <TD> Images           </TD><TD> 
///                                      - Page setup (size, orientation, resolution)
///                                      - Image compression method                                                        
///                                      - Color mode, anti-aliasing                                                       </TD> </TR>
/// <TR> <TD> ZPL-II (ZEBRA)   </TD><TD> 
///                                      - Page setup (size, orientation, resolution)
///                                      - Scaling, dithering, compression method                                          </TD> </TR>
/// </TABLE>
///
/// If no XML file is specified using this function, #TFormer_Print will use the default configuration file.
///
/// \note
/// The configuration file <c>TFORMer.xml</c> may be passed in the following ways:
///
/// - Specify the filename directly.
/// - Specify the filename of a ZIP-file (e.g. with WinZip) containing the required file.
/// - BASE64-encode the content of the required file (see Convert class of .NET) and use this string instead of a filename. 
/// - BASE64-encode the content of a ZIP-file (see Convert class of .NET) containing the required file(s). Use this string instead of a filename. 
///
/// \note 
/// BASE64 encoded strings <b>must</b> start with the sequence <c>BASE64:</c>. When passing a zipped file <em>TFORMer</em> expects a real WinZip compatible format. A zipped stream is not suitable!
/// For more details please refer to the <em>TFORMer Runtime DLL Programming Reference</em>.
/// 
///
/// \see #TFormer_Print
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_SetConfigFile
(
 HTFORM                     pInstance                /* IN : TFORMer-Instance - see TFormer_Init        */
,LPCSTR                     pszFilename              /* IN : Path to a TFORMer configuration file       */
); 

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_SetConfigFileW
(
 HTFORM                     pInstance                /* IN : TFORMer-Instance - see TFormer_Init        */
,LPCWSTR                    pszFilename              /* IN : Path to a TFORMer configuration file       */
); 
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Prints the current FormLayout using the adjusted DataSource.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance    The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// 
/// \remarks
/// - For Repository based FormLayouts: \n
///   Starts the output based on the selected Repository, Project, FormLayout name and data (see TFormer_SetXXXDataFile or #TFormer_NewRecord, #TFormer_SetVarValue).
/// - For stand-alone FormLayouts: \n
///   Starts the output based on the selected stand-alone FormLayout and the input data (see TFormer_SetXXXDataFile or #TFormer_NewRecord, #TFormer_SetVarValue).
///
/// \note
/// Behavior if no input data was supplied: If no DataSource was specified (either by TFormer_SetXXXDataFile, #TFormer_SetODBCData or via #TFormer_NewRecord and #TFormer_SetVarValue) the default data-file 
/// (located in the same directory as the FormLayout itself and generated by the last <em>TFORMer Designer</em> session) is used. If this default data-file does not exists no output will be produced.
/// The name of the default data-file is <c>path/FormName.xml</c>. 
///
/// \note 
/// <em>TFORMer</em> uses a configuration file named <c>TFORMer.xml</c> for adjusting PostScript, PDF and HTML output options like form sizes, color mode and so on.
/// Check out #TFormer_SetConfigFile or refer to the <c>TFORMer Developer Manual</c> for more information.
/// 
/// \see #TFormer_SetRepositoryName, #TFormer_SetProjectName, #TFormer_SetFormName, 
///      #TFormer_SetPrinterName, #TFormer_SetTxtDataFile, #TFormer_SetXmlDataFile, #TFormer_SetConfigFile.
/// \ingroup group_Print
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
ERRCODE   __stdcall         TFormer_Print 
(
 HTFORM                     pInstance               
); 

// ----------------------------------------------------------------------
/// Returns the last error description as string.
///
/// \returns Error description or empty string if no description is available.
///
/// \param[in] pInstance    The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// 
/// \remarks
/// This function is used to retrieve detailed information about an error. Call this function after any method returned an error code not equal to #ErrOk (0).
/// 
/// \warning
/// Subject to change!
/// 
/// \see #TFormer_GetLastErrorInformation, #TFormer_GetLastErrorSource
/// \ingroup group_Error
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
LPCSTR   __stdcall          TFormer_GetLastErrorDescription
(
 HTFORM                     pInstance               
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
LPCWSTR   __stdcall         TFormer_GetLastErrorDescriptionW
(
 HTFORM                     pInstance               
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the last error information as string.
///
/// \returns Error information or empty string if no information is available.
///
/// \param[in] pInstance    The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// 
/// \remarks
/// This function is used to retrieve detailed information about an error. Call this function after any method returned an error code not equal to #ErrOk (0).
/// The error information holds extra information regarding the error - e.g. the text of a validation rule.
/// 
/// \warning
/// Subject to change!
/// 
/// \see #TFormer_GetLastErrorDescription, #TFormer_GetLastErrorSource
/// \ingroup group_Error
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
LPCSTR   __stdcall          TFormer_GetLastErrorInformation
(
 HTFORM                     pInstance               
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC 
LPCWSTR   __stdcall         TFormer_GetLastErrorInformationW
(
 HTFORM                     pInstance               
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the last error source as string.
///
/// \returns Error source or empty string if no source information is available.
///
/// \param[in] pInstance    The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// 
/// \remarks
///  This function is used to retrieve detailed information about an error. . Call this function after any method returned an error code not equal to #ErrOk (0).
/// 
/// \warning
/// Subject to change!
/// 
/// \see #TFormer_GetLastErrorDescription, #TFormer_GetLastErrorInformation
/// \ingroup group_Error
// ----------------------------------------------------------------------
TECIT_DLLSPEC 
LPCSTR   __stdcall			    TFormer_GetLastErrorSource
(
 HTFORM                     pInstance               
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC   
LPCWSTR   __stdcall		  	  TFormer_GetLastErrorSourceW
(
 HTFORM                     pInstance               
);
#endif // #ifndef _TEC_UNIX


// **********************************************************************
// System-Helper functions
// **********************************************************************

// ----------------------------------------------------------------------
/// Refreshes the internal list of (operating system) printers available for the current user.
///
/// \remarks
/// Fills the internal structures for providing information about installed operating system printers.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in]  pInstance      The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
///
/// \see #TFormer_GetOSPrintersCount, #TFormer_GetOSPrinter, #TFormer_GetOSDefaultPrinter
/// \ingroup group_System
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LONG __stdcall              TFormer_RefreshOSPrinters
(
 HTFORM                     pInstance             
);

// ----------------------------------------------------------------------
/// Returns the number of (operating system) printers available for the current user.
///
/// \remarks
/// If the internal structures are not initialized, this function will call #TFormer_RefreshOSPrinters automatically.
///
/// \returns - Number of (operating system) printers [0..n].
///
/// \param[in]  pInstance      The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
///
/// \see #TFormer_RefreshOSPrinters, #TFormer_GetOSPrinter, #TFormer_GetOSDefaultPrinter
/// \ingroup group_System
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LONG __stdcall              TFormer_GetOSPrintersCount
(
 HTFORM                     pInstance             
);

// ----------------------------------------------------------------------
/// Returns the name of the (operating system) printer with the specified index.
///
/// \remarks
/// The index must be in the range [0...#TFormer_GetOSPrintersCount].
/// If the internal structures are not initialized, this function will call #TFormer_RefreshOSPrinters automatically.
///
/// \returns - The name of the (operating system) printer.
///
/// \param[in]  pInstance      The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in]  nIndex         The index of the printer [0..#TFormer_GetOSPrintersCount - 1].
///
/// \see #TFormer_RefreshOSPrinters, #TFormer_GetOSPrintersCount, #TFormer_GetOSDefaultPrinter
/// \ingroup group_System
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR __stdcall            TFormer_GetOSPrinter
(
 HTFORM                     pInstance             
,LONG                       nIndex             
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR __stdcall           TFormer_GetOSPrinterW
(
 HTFORM                     pInstance             
,LONG                       nIndex                
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the name of the default (operating system) printer of the current user.
///
/// \remarks
/// If the internal structures are not initialized, this function will call #TFormer_RefreshOSPrinters automatically.
///
/// \returns - The name of the default (operating system) printer for the current user.
///
/// \param[in]  pInstance      The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
///
/// \see #TFormer_RefreshOSPrinters, #TFormer_GetOSPrintersCount, #TFormer_GetOSPrinter
/// \ingroup group_System
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR __stdcall            TFormer_GetOSDefaultPrinter
(
 HTFORM                     pInstance             
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR __stdcall           TFormer_GetOSDefaultPrinterW
(
 HTFORM                     pInstance             
);
#endif // #ifndef _TEC_UNIX


// **********************************************************************
// Repository-Access functions
// **********************************************************************


// ----------------------------------------------------------------------
/// Creates and adds a new Project to the Repository.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance        The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszProjectName   The name of the Project to be created.
/// \param[in] pszProjectDesc   The description of the Project. May be empty or NULL.
///
/// \remarks
/// The Project name must follow the \ref Naming_Rules "Naming Rules".
///
/// \note
/// For a stand-alone FormLayout this function is not available.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_AddNewProject
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
,LPCSTR                     pszProjectDesc          
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_AddNewProjectW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszProjectName          
,LPCWSTR                    pszProjectDesc          
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Deletes a Project from the Repository.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance        The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszProjectName   The name of the Project to be deleted.
///
/// \remarks
/// Deletes the Project as well as all DataFields and FormLayouts belonging to this Project from a Repository.
/// 
/// \note
/// Please note that this function does not delete files for a FormLayout (*.xmd), data-files (*.xml) or any subdirectories. 
/// This function only modifies the Repository file itself.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_DeleteProject
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_DeleteProjectW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszProjectName          
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Adds a new Form to the given Project.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance        The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszProjectName   The name of the Project in the Repository.
/// \param[in] pszFormName      The name of the FormLayout to be created.
/// \param[in] pszFormDesc      The (optional) description of the FormLayout. May be empty or NULL.
/// \param[in] pszFormDesign    Design-tool for editing this FormLayout (not used anymore).
/// \param[in] pszFormFile      (optional) Template file for new FormLayout.
///
/// \remarks
/// The name of the FormLayout must follow the \ref Naming_Rules "Naming Rules".
///
/// \remarks
/// Adds a new FormLayout to the Project specified by pszProjectName and creates the corresponding Project file.
/// If pszFormFile is empty, this function creates an empty FormLayout.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_AddNewForm
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
,LPCSTR                     pszFormName             
,LPCSTR                     pszFormDesc             
,LPCSTR                     pszFormDesign           
,LPCSTR                     pszFormFile             
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_AddNewFormW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszProjectName          
,LPCWSTR                    pszFormName             
,LPCWSTR                    pszFormDesc             
,LPCWSTR                    pszFormDesign           
,LPCWSTR                    pszFormFile             
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Deletes a FormLayout from the specified Project.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance        The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszProjectName   The name of the Project.
/// \param[in] pszFormName      The name of the FormLayout to be deleted.
///
/// \remarks
/// Deletes a FormLayout from the specified Project.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_DeleteForm
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
,LPCSTR                     pszFormName             
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_DeleteFormW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszProjectName          
,LPCWSTR                    pszFormName             
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Adds a new DataField to the specified Project.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance        The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszProjectName   The name of the Project (may be NULL for global DataFields)
/// \param[in] pszVarName       The name of the DataField to be added.
/// \param[in] eVType           The data-type of the DataField.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_AddNewVar
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
,LPCSTR                     pszVarName              
,EVarType                   eVType                  
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_AddNewVarW
(
 HTFORM                     pInstance               
,LPCWSTR                     pszProjectName          
,LPCWSTR                     pszVarName              
,EVarType                   eVType                  
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Deletes a DataField from the specified Project.
///
/// \returns - ErrOk (0) in case of success\n - Non-zero if error occurred
///
/// \param[in] pInstance        The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in] pszProjectName   The name of the Project (may be NULL for global DataFields)
/// \param[in] pszVarName       The name of the DataField to be deleted.
///
/// \remarks
/// Deletes a DataField from the specified Project. 
/// If no Project is specified, the DataField will be removed from the global Project.
///
/// \note
/// Deleting a DataField which is still in use by FormLayouts will make these FormLayouts unusable!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_DeleteVar
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
,LPCSTR                     pszVarName              
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_DeleteVarW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszProjectName          
,LPCWSTR                    pszVarName              
);
#endif // #ifndef _TEC_UNIX

// **********************************************************************
// Repository-Access Iteration functions
// **********************************************************************

// ----------------------------------------------------------------------
/// Creates a new iterator and sets its position to the first Project within the Repository.
///
/// \returns - Iterator in case of success\n - NULL in case of an error (check peCode in this case)
///
/// \param[in]  pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// This function is used for enumerating Projects within the actual Repository. 
/// It returns an iterator and sets its position to the first Project. 
/// Use #TFormer_ProjectItGetName and #TFormer_ProjectItGetDescription to retrieve the name and description of the actual Project.
///
/// \note
/// Release the iterator after use with #TFormer_FreeProjectIt ! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
HPROJECT_ITERATOR   __stdcall     TFormer_GetFirstProjectIt
(
 HTFORM                     pInstance               
,ERRCODE*                   peCode                  
);

// ----------------------------------------------------------------------
/// Moves the iterator to the next Project within the Repository
///
/// \returns - ErrOk (0) in iterator was moved to next Project\n - Non-zero if error occurred or there are no more Projects
///
/// \param[in]  hProjectIt      Project iterator (see #TFormer_GetFirstProjectIt)
///
/// \remarks
/// This function is used for enumerating Projects within the actual Repository. 
/// Use #TFormer_ProjectItGetName and #TFormer_ProjectItGetDescription to retrieve the name and description of the actual Project.
///
/// \note
/// Release the iterator after use with #TFormer_FreeProjectIt! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_GetNextProjectIt
(
 HPROJECT_ITERATOR          hProjectIt          
);

// ----------------------------------------------------------------------
/// Returns the name of the Project the iterator is pointing to.
///
/// \returns - The name of the Project the iterator is pointing to. This name is only valid if peCode is ErrOk (0).
///
/// \param[in]  hProjectIt      Project iterator (see #TFormer_GetFirstProjectIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_ProjectItGetName
(
 HPROJECT_ITERATOR          hProjectIt         
,ERRCODE*                   peCode             
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_ProjectItGetNameW
(
 HPROJECT_ITERATOR          hProjectIt         
,ERRCODE*                   peCode             
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the description of the Project the iterator is pointing to.
///
/// \returns - The description of the Project the iterator is pointing to. This description is only valid if peCode is ErrOk (0).
///
/// \param[in]  hProjectIt      Project iterator (see #TFormer_GetFirstProjectIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \note
/// The returned pointer is for temporary use only! The Project description may be empty.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_ProjectItGetDescription
( 
 HPROJECT_ITERATOR          hProjectIt          
,ERRCODE*                   peCode              
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_ProjectItGetDescriptionW
( 
 HPROJECT_ITERATOR          hProjectIt          
,ERRCODE*                   peCode              
);
#endif // #ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Releases the Project iterator and frees any resources associated with it.
///
/// \returns - No return value.
///
/// \param[in]  hProjectIt      Project iterator (see #TFormer_GetFirstProjectIt)
///
/// \note
/// Release the iterator after use! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
void      __stdcall         TFormer_FreeProjectIt
(
 HPROJECT_ITERATOR          hProjectIt              
);

// ----------------------------------------------------------------------
/// Returns an iterator pointing to the first FormLayout in the specified Project.
///
/// \returns - Iterator in case of success\n - NULL in case of an error (check peCode in this case)
///
/// \param[in]  pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in]  pszProjectName  The name of the Project within the Repository.
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// Creates a new FormLayout iterator and sets its position to the first FormLayout in the Project specified by pszProjectName.
/// To get the name of the FormLayout call #TFormer_FormItGetName.  
///
/// \note
/// Release the iterator after use with #TFormer_FreeFormIt ! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
HFORM_ITERATOR   __stdcall  TFormer_GetFirstFormIt
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
,ERRCODE*                   peCode                  
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
HFORM_ITERATOR   __stdcall  TFormer_GetFirstFormItW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszProjectName          
,ERRCODE*                   peCode                  
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Move the iterator to the next FormLayout within the Project
///
/// \returns - ErrOk (0) if iterator was moved to the next FormLayout\n - Non-zero if error occurred or there are no more FormLayouts in the Project
///
/// \param[in]  hFormIt     FormLayout iterator (see #TFormer_GetFirstFormIt)
///
/// \remarks
/// This function is used for enumerating FormLayouts within a Project. 
/// Use #TFormer_FormItGetName and #TFormer_FormItGetDescription to retrieve the name and description of the actual FormLayout.
///
/// \note
/// Release the iterator after use with #TFormer_FreeFormIt ! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_GetNextFormIt
(
 HFORM_ITERATOR             hFormIt                 
);

// ----------------------------------------------------------------------
/// Returns the name of the FormLayout the iterator is pointing to.
///
/// \returns - The name of the FormLayout the iterator is pointing to. This name is only valid if peCode is ErrOk (0).
///
/// \param[in]  hFormIt         FormLayout iterator (see #TFormer_GetFirstFormIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_FormItGetName
(
 HFORM_ITERATOR             hFormIt                 
,ERRCODE*                   peCode                  
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_FormItGetNameW
(
 HFORM_ITERATOR             hFormIt                 
,ERRCODE*                   peCode                  
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the description of the FormLayout the iterator is pointing to.
///
/// \returns - The description of the FormLayout the iterator is pointing to. This description is only valid if peCode is ErrOk (0).
///
/// \param[in]  hFormIt         FormLayout iterator (see #TFormer_GetFirstFormIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \note
/// The returned pointer is for temporary use only! The FormLayout description may be empty.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_FormItGetDescription
(
 HFORM_ITERATOR             hFormIt                 
,ERRCODE*                   peCode                  
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_FormItGetDescriptionW
(
 HFORM_ITERATOR             hFormIt                 
,ERRCODE*                   peCode                  
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Releases the FormLayout iterator and frees any resources associated with it.
///
/// \returns - No return value.
///
/// \param[in]  hFormIt         FormLayout iterator (see #TFormer_GetFirstFormIt)
///
/// \note
/// Release the iterator after use! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
void      __stdcall         TFormer_FreeFormIt
(
 HFORM_ITERATOR             hFormIt                 
);

// ----------------------------------------------------------------------
/// Returns an iterator pointing to the first DataField in the specified Project.
///
/// \returns - Iterator in case of success\n - NULL in case of an error (check peCode in this case)
///
/// \param[in]  pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in]  pszProjectName  The name of the Project within the Repository.
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// Creates a new DataField iterator and sets its position to the first DataField in the Project specified by pszProjectName.
/// If pszProjectName is NULL or empty the DataField of the global Project are iterated.
/// To get the name of the DataField call #TFormer_ProjectVarItGetName.  
///
/// \note
/// Release the iterator after use with #TFormer_FreeProjectVarIt! Memory leaks may occur otherwise.
///
/// \note
/// \par This example iterates all DataFields within the project.
/// \code
/// // get iterator
/// it = TFormer_GetFirstProjectVarIt (hTForm, lpszProjectName, &eCode);
/// 
/// // iterate all DataFields
/// while ((it != NULL) && (eCode == ErrOk))
/// {
///   // check, whether the DataField is editable
///   eVarUsage = TFormer_ProjectVarItGetUsage (hTForm, it, &eCode);      
///   if ((eVarUsage == eVarUsage_Normal) && (eCode == ErrOk))
///   {
///     // do something
///   }   
///   // get next DataField
///   eCode = TFormer_GetNextProjectVarIt (it);
/// }
/// // release iterator
/// TFormer_FreeProjectVarIt (it);
/// \endcode
///
/// \ingroup group_Repository
/// 
/// \see #TFormer_GetNextProjectVarIt, #TFormer_ProjectVarItGetUsage
// ----------------------------------------------------------------------
TECIT_DLLSPEC
HPROJECTVAR_ITERATOR   __stdcall     TFormer_GetFirstProjectVarIt
(
 HTFORM                     pInstance               
,LPCSTR                     pszProjectName          
,ERRCODE*                   peCode                  
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
HPROJECTVAR_ITERATOR   __stdcall     TFormer_GetFirstProjectVarItW
(
 HTFORM                     pInstance               
,LPCWSTR                    pszProjectName          
,ERRCODE*                   peCode                  
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Move the iterator to the next DataField within the Project
///
/// \returns - ErrOk (0) if iterator was moved to the next DataField\n - Non-zero if error occurred or there are no more DataFields in the Project
///
/// \param[in]  hProjectVarIt  DataField iterator (see #TFormer_GetFirstProjectVarIt)
///
/// \remarks
/// This function is used for enumerating DataFields within a Project. 
/// Use #TFormer_ProjectVarItGetName and #TFormer_ProjectVarItGetDescription to retrieve the name and description of the DataField.
///
/// \note
/// Release the iterator after use with #TFormer_FreeProjectVarIt! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
/// 
/// \see #TFormer_GetFirstProjectVarIt, #TFormer_ProjectVarItGetUsage
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_GetNextProjectVarIt
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
);

// ----------------------------------------------------------------------
/// Returns the name of the DataField the iterator is pointing to.
///
/// \returns - The name of the DataField the iterator is pointing to. This name is only valid if peCode is ErrOk (0).
///
/// \param[in]  hProjectVarIt   DataField iterator (see #TFormer_GetFirstProjectVarIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// The caption is only used for informational purposes and for providing the end-user additional information describing the purpose of the DataField.
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_ProjectVarItGetName
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_ProjectVarItGetNameW
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the description of the DataField the iterator is pointing to.
///
/// \returns - The description of the DataField the iterator is pointing to. This description is only valid if peCode is ErrOk (0).
///
/// \param[in]  hProjectVarIt   DataField iterator (see #TFormer_GetFirstProjectVarIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// The description is only used for informational purposes and for providing the end-user additional information describing the purpose of the DataField. 
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_ProjectVarItGetDescription
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_ProjectVarItGetDescriptionW
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the caption of the DataField the iterator is pointing to.
///
/// \returns - The caption of the DataField the iterator is pointing to. This caption is only valid if peCode is ErrOk (0).
///
/// \param[in]  hProjectVarIt   DataField iterator (see #TFormer_GetFirstProjectVarIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// The caption is only used for informational purposes and for providing the end-user additional information describing the purpose of the DataField. 
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_ProjectVarItGetCaption
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_ProjectVarItGetCaptionW
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the display-order of the DataField the iterator is pointing to.
///
/// \returns - The display-order of the DataField the iterator is pointing to. The display-order is only valid if peCode is ErrOk (0).
///
/// \param[in]  hProjectVarIt   DataField iterator (see #TFormer_GetFirstProjectVarIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// This value is used for assigning a display-order to a DataField. A good example is an application which presents a sorted list of DataFields to the user. 
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_ProjectVarItGetDisplayOrder
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_ProjectVarItGetDisplayOrderW
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Releases the DataField iterator and frees any resources associated with it.
///
/// \returns - No return value.
///
/// \param[in]  hProjectVarIt   DataField iterator (see #TFormer_GetFirstProjectVarIt)
///
/// \note
/// Release the iterator after use! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
void      __stdcall         TFormer_FreeProjectVarIt
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
);

// ----------------------------------------------------------------------
/// Returns the default-value of the DataField the iterator is pointing to
///
/// \returns - The default value of the DataField the iterator is pointing to. This value is only valid if peCode is ErrOk (0).
///
/// \param[in]  hProjectVarIt   DataField iterator (see #TFormer_GetFirstProjectVarIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// The default value is the standard-value of a DataField when creating data manually or automatically within <em>TFORMer Designer</em>. 
/// The default value is only used by <em>TFORMer Designer</em>. When a DataSource is used to pass data to <em>TFORMer Runtime</em> and a DataField is not set by this DataSource the value of the DataField will be an empty string ("") or 0. See also #TFormer_SetReuseValues.
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_ProjectVarItGetDefValue
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_ProjectVarItGetDefValueW
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the data-type of the DataField the iterator is pointing to
///
/// \returns - The data-type of the DataField the iterator is pointing to. This value is only valid if peCode is ErrOk (0).
///
/// \param[in]  hProjectVarIt   DataField iterator (see #TFormer_GetFirstProjectVarIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
EVarType   __stdcall        TFormer_ProjectVarItGetType
(
 HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);

// ----------------------------------------------------------------------
/// Returns the usage of the DataField specified with the iterator.
///
/// \remarks
/// A FormLayout utilizes user- or system DataField as place-holders for dynamic data. Values for such DataFields are usually provided by the user (entered manually in <em>TFORMer Designer</em>) or imported from a DataSource. 
/// Each DataField can be used on its own, as part of expressions (e.g. in a text- or bar-code element) or it may be computed within the FormLayout itself (e.g. pre- and post-expressions). 
/// This method queries the FormLayout and returns an enumeration (see #EVarUsage) regarding the use of a specific DataField in the FormLayout.
///
/// \returns - The usage of the DataField in the current FormLayout. This value is only valid if peCode is ErrOk (0).
///
/// \param[in]  pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in]  hProjectVarIt   DataField iterator (see #TFormer_GetFirstProjectVarIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \ingroup group_Repository
/// 
/// \see #TFormer_GetFirstProjectVarIt, #TFormer_GetNextProjectVarIt
// ----------------------------------------------------------------------
TECIT_DLLSPEC
EVarUsage __stdcall         TFormer_ProjectVarItGetUsage
(
 HTFORM                     pInstance             
,HPROJECTVAR_ITERATOR       hProjectVarIt         
,ERRCODE*                   peCode                
);

// ----------------------------------------------------------------------
/// Retrieves the usage of the DataField specified by its name.
///
/// \remarks
/// A FormLayout utilizes user- or system DataFields as place-holders for dynamic data. Values for such DataFields are usually provided by the user (entered manually in <em>TFORMer Designer</em>) or imported from a DataSource. 
/// Each DataField can be used on its own, as part of expressions (e.g. in a text- or bar-code element) or it may be computed within the FormLayout itself (e.g. pre- and post-expressions). 
/// This method queries the FormLayout and returns an enumeration (see #EVarUsage) regarding the use of a specific DataField in the FormLayout.
///
/// \returns - The usage of the DataField in the current FormLayout. This value is only valid if peCode is ErrOk (0).
///
/// \param[in]  pInstance      The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in]  pszVarName     the name of the DataField
/// \param[out] peCode         Error Code (ErrOk (0) in case of success)
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
EVarUsage __stdcall         TFormer_GetVarUsage
(
 HTFORM                     pInstance             
,LPCSTR                     pszVarName            
,ERRCODE*                   peCode                
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
EVarUsage __stdcall         TFormer_GetVarUsageW
(
 HTFORM                     pInstance             
,LPCWSTR                    pszVarName            
,ERRCODE*                   peCode                
);
#endif // #ifndef _TEC_UNIX




// ---------------------------------------------------------------------
// ----------------------------------------------------------------------
/// Returns an iterator pointing to the first DataSource in the specified Project.
///
/// \returns - Iterator in case of success\n - NULL in case of an error (check peCode in this case)
///
/// \param[in]  pInstance       The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in]  pszProjectName  The name of the Project within the Repository.
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// Creates a new DataSource iterator and sets its position to the first DataSource in the Project specified by pszProjectName.
/// To get the name of the DataSource call #TFormer_DatasourceItGetName.  
///
/// \note
/// Release the iterator after use with #TFormer_FreeDatasourceIt ! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
HDATASOURCE_ITERATOR   __stdcall  TFormer_GetFirstDatasourceIt
(
 HTFORM                     pInstance               
 ,LPCSTR                     pszProjectName          
 ,ERRCODE*                   peCode                  
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
HDATASOURCE_ITERATOR   __stdcall  TFormer_GetFirstDatasourceItW
(
 HTFORM                     pInstance               
 ,LPCWSTR                    pszProjectName          
 ,ERRCODE*                   peCode                  
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Move the iterator to the next DataSource within the Project
///
/// \returns - ErrOk (0) if iterator was moved to the next DataSource\n - Non-zero if error occurred or there are no more DataSources in the Project
///
/// \param[in]  hDatasourceIt     DataSource iterator (see #TFormer_GetFirstDatasourceIt)
///
/// \remarks
/// This function is used for enumerating DataSources within a Project. 
/// Use #TFormer_DatasourceItGetName and #TFormer_DatasourceItGetDescription to retrieve the name and description of the actual DataSource.
///
/// \note
/// Release the iterator after use with #TFormer_FreeDatasourceIt ! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_GetNextDatasourceIt
(
 HDATASOURCE_ITERATOR             hDatasourceIt                 
 );

// ----------------------------------------------------------------------
/// Returns the name of the DataSource the iterator is pointing to.
///
/// \returns - The name of the DataSource the iterator is pointing to. This name is only valid if peCode is ErrOk (0).
///
/// \param[in]  hDatasourceIt   DataSource iterator (see #TFormer_GetFirstDatasourceIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_DatasourceItGetName
(
 HDATASOURCE_ITERATOR        hDatasourceIt                 
 ,ERRCODE*                   peCode                  
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_DatasourceItGetNameW
(
 HDATASOURCE_ITERATOR        hDatasourceIt                 
 ,ERRCODE*                   peCode                  
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the description of the DataSource the iterator is pointing to.
///
/// \returns - The description of the DataSource the iterator is pointing to. This description is only valid if peCode is ErrOk (0).
///
/// \param[in]  hDatasourceIt   DataSource iterator (see #TFormer_GetFirstDatasourceIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \note
/// The returned pointer is for temporary use only! The DataSource description may be empty.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_DatasourceItGetDescription
(
 HDATASOURCE_ITERATOR        hDatasourceIt                 
 ,ERRCODE*                   peCode                  
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_DatasourceItGetDescriptionW
(
 HDATASOURCE_ITERATOR       hDatasourceIt                 
 ,ERRCODE*                  peCode                  
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Releases the DataSource iterator and frees any resources associated with it.
///
/// \returns - No return value.
///
/// \param[in]  hDatasourceIt   Datasourceiterator (see #TFormer_GetFirstDatasourceIt)
///
/// \note
/// Release the iterator after use! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
void      __stdcall         TFormer_FreeDatasourceIt
(
 HDATASOURCE_ITERATOR       hDatasourceIt                 
);



// ----------------------------------------------------------------------
/// Returns an iterator pointing to the first DataSource-Parameter of the specified DataSource in the specified Project.
///
/// \returns - Iterator in case of success\n - NULL in case of an error (check peCode in this case)
///
/// \param[in]  pInstance           The <em>TFORMer</em>-Instance provided by a previous call to #TFormer_Init.
/// \param[in]  pszProjectName      The name of the Project within the Repository.
/// \param[in]  pszDatasourceName   The name of the DataSource within the Project.
/// \param[out] peCode              Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// Creates a new DataSource-Parameter iterator and sets its position to the first DataSource-Parameter in the DataSource specified by pszDatasourceName.
/// If pszProjectName is NULL or empty the DataSource will get searched in the global Project.
/// To get the name of the DataSource-Parameter call #TFormer_DSParameterItGetName.  
///
/// \note
/// Release the iterator after use with #TFormer_FreeDSParameterIt! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
/// 
/// \see #TFormer_GetNextDSParameterIt
// ----------------------------------------------------------------------
TECIT_DLLSPEC
HDSPARAMETER_ITERATOR   __stdcall     TFormer_GetFirstDSParameterIt
(
 HTFORM                      pInstance
 ,LPCSTR                     pszProjectName
 ,LPCSTR                     pszDatasourceName
 ,ERRCODE*                   peCode
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
HDSPARAMETER_ITERATOR   __stdcall     TFormer_GetFirstDSParameterItW
(
 HTFORM                     pInstance               
 ,LPCWSTR                   pszProjectName          
 ,LPCWSTR                   pszDatasourceName
 ,ERRCODE*                  peCode        
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Move the iterator to the next DataSource-Parameter within the DataSource.
///
/// \returns - ErrOk (0) if iterator was moved to the next DataSource-Parameter\n - Non-zero if error occurred or there are no more DataSource-Parameters in the DataSource
///
/// \param[in]  hIt   DataSource-Parameter iterator (see #TFormer_GetFirstDSParameterIt)
///
/// \remarks
/// This function is used for enumerating DataSource-Parameters within a DataSource. 
/// Use #TFormer_DSParameterItGetName and #TFormer_DSParameterItGetDescription to retrieve the name and description of the DataSource-Parameter.
///
/// \note
/// Release the iterator after use with #TFormer_FreeDSParameterIt! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
/// 
/// \see #TFormer_GetFirstDSParameterIt
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE   __stdcall         TFormer_GetNextDSParameterIt
(
 HDSPARAMETER_ITERATOR       hIt         
 );

// ----------------------------------------------------------------------
/// Returns the name of the DataSource-Parameter the iterator is pointing to.
///
/// \returns - The name of the DataSource-Parameter the iterator is pointing to. This name is only valid if peCode is ErrOk (0).
///
/// \param[in]  hIt             DataSource-Parameter iterator (see #TFormer_GetFirstDSParameterIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_DSParameterItGetName
(
 HDSPARAMETER_ITERATOR       hIt         
 ,ERRCODE*                   peCode                
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_DSParameterItGetNameW
(
 HDSPARAMETER_ITERATOR        hIt         
 ,ERRCODE*                   peCode                
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the description of the DataSource-Parameter the iterator is pointing to.
///
/// \returns - The description of the DataSource-Parameter the iterator is pointing to. This description is only valid if peCode is ErrOk (0).
///
/// \param[in]  hIt             DataSource-Parameter iterator (see #TFormer_GetFirstDSParameterIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// The description is only used for informational purposes and for providing the end-user additional information describing the purpose of the DataSource-Parameter. 
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_DSParameterItGetDescription
(
 HDSPARAMETER_ITERATOR       hIt         
 ,ERRCODE*                   peCode                
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_DSParameterItGetDescriptionW
(
 HDSPARAMETER_ITERATOR       hIt         
 ,ERRCODE*                   peCode                
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the caption of the DataSource-Parameter the iterator is pointing to.
///
/// \returns - The caption of the DataSource-Parameter the iterator is pointing to. This caption is only valid if peCode is ErrOk (0).
///
/// \param[in]  hIt             DataSource-Parameter iterator (see #TFormer_GetFirstDSParameterIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// The caption is only used for informational purposes and for providing the end-user additional information describing the purpose of the DataSource-Parameter. 
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_DSParameterItGetCaption
(
 HDSPARAMETER_ITERATOR       hIt         
 ,ERRCODE*                   peCode                
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_DSParameterItGetCaptionW
(
 HDSPARAMETER_ITERATOR       hIt         
 ,ERRCODE*                   peCode                
 );
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the display-order of the DataSource-Parameter the iterator is pointing to.
///
/// \returns - The display-order of the DataSource-Parameter the iterator is pointing to. The display-order is only valid if peCode is ErrOk (0).
///
/// \param[in]  hIt             DataSource-Parameter iterator (see #TFormer_GetFirstDSParameterIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \remarks
/// This value is used for assigning a display-order to a DataSource-Parameter. A good example is an application which presents a sorted list of DataSource-Parameters to the user. 
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_DSParameterItGetDisplayOrder
(
 HDSPARAMETER_ITERATOR       hIt         
 ,ERRCODE*                   peCode                
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_DSParameterItGetDisplayOrderW
(
 HDSPARAMETER_ITERATOR       hIt         
 ,ERRCODE*                   peCode                
 );
#endif // #ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Returns the default-value of the DataSource-Parameter the iterator is pointing to
///
/// \returns - The default value of the DataSource-Parameter the iterator is pointing to. This value is only valid if peCode is ErrOk (0).
///
/// \param[in]  hIt             DataSource-Parameter iterator (see #TFormer_GetFirstDSParameterIt)
/// \param[out] peCode          Error Code (ErrOk (0) in case of success)
///
/// \note
/// The returned pointer is for temporary use only!
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR   __stdcall          TFormer_DSParameterItGetDefValue
(
 HDSPARAMETER_ITERATOR      hIt         
 ,ERRCODE*                  peCode                
 );

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR   __stdcall         TFormer_DSParameterItGetDefValueW
(
 HDSPARAMETER_ITERATOR       hIt         
 ,ERRCODE*                   peCode                
 );
#endif // #ifndef _TEC_UNIX


// ----------------------------------------------------------------------
/// Releases the DataSource-Parameter iterator and frees any resources associated with it.
///
/// \returns - No return value.
///
/// \param[in]  hIt   DataSource-Parameter iterator (see #TFormer_GetFirstDSParameterIt)
///
/// \note
/// Release the iterator after use! Memory leaks may occur otherwise.
///
/// \ingroup group_Repository
// ----------------------------------------------------------------------
TECIT_DLLSPEC
void      __stdcall         TFormer_FreeDSParameterIt
(
 HDSPARAMETER_ITERATOR       hIt         
 );

// ----------------------------------------------------------------------
// Extra documentation
// ----------------------------------------------------------------------
// ----------------------------------------------------------------------
/// \page Naming_Rules Naming Rules
/// This page describes the rules for valid names for Projects, FormLayouts and DataFields.
/// - Valid names start either with an underscore (_) or a letter. 
/// - Within the name letters, digits and underscores are allowed. 
/// - Do not use special characters (e.g. $&<>()) or spaces.
///
/// As a general rule creates names according to valid JavaScript identifiers.
/// 
/// For example following names are VALID:
/// - MyDataField
/// - _0MyDataField
/// - MyData_Field
/// .
/// These names are NOT VALID:
/// - 3MyDataField
/// - MyData Field
/// .
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
/// \page Additional_Documents Additional Documents
/// The following additional documents and manuals are available:
/// \n\n
/// - <em>TFORMer Designer</em> Tutorial\n
///   This is an introduction into the layout editor <em>TFORMer Designer</em>\n
/// \n
/// - <em>TFORMer Designer</em> User Manual\n
///   This manual provides a detailed description of the layout editor <em>TFORMer Designer</em>\n
///   It demonstrates how to design labels and reports, how to print documents and how to provide user-data to <em>TFORMer</em>\n
/// \n
/// - <em>TFORMer Runtime</em> Developer Manual\n
///   This document provides an overview for installing and using <em>TFORMer Runtime</em> on various platforms\n
/// \n
/// - <em>TFORMer Runtime</em> API References\n
///   These documents provide descriptions for all available APIs of <em>TFORMer Runtime</em>\n
/// \n
/// - TEC-IT Barcode Reference\n
///   <em>TFORMer</em> includes built-in barcode support - if you are new to bar-coding or you need a source for specific information about barcode symbologies we suggest reading of TEC-IT's Barcode Reference (http://www.tec-it.com/documentation)\n
// ----------------------------------------------------------------------

// **********************************************************************
// ZIP-file and BASE64 helper functions
// **********************************************************************


// ----------------------------------------------------------------------
/// Allocated memory for a new unzipper handle.
///
/// \remarks
/// <em>TFORMer</em> functions accepting filenames like TFormer_SetRepositoryName, TFormer_SetDataFileXML, ... are accepting the following parameters:
/// * Plain file names (*.tfr, *.tff, *.xml)
/// * Zipped files (zipped *.tfr, zipped *.tff, ...)
/// * BASE64 encoded strings containing plain or zip files (in such a case the string must start with "BASE64:" - without apostrophes)
/// Usually all <em>TFORMer</em> functions accepting files are trying to determine if it is a zip-file automatically (a BASE64 encoded file is determined by the BASE64: prefix). 
/// When <em>TFORMer</em> detects a zip-file it is unzipped automatically to the temp-directory of the current user. This automatic unzipping requires <em>TFORMer</em> to clean up such temporary files after the function call ends.
/// If you want to avoid the per-function unzipping and deleting, then implement the unzipping functionality yourself by the TecUnzipper-functions. 
///
/// \remarks
/// Free allocated memory with #TecUnzipper_Free after use.
///
/// \remarks
/// This function may be changed in the future.
///
/// \returns - A valid handle or NULL if the handle could not be allocated.
///
/// \param[in]  pszFilename    The filename of a valid zip file or a BASE64 encoded string (e.g. BASE64:h/5tzWP%$378jdfh...).
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
HTUNZIPPER __stdcall		TecUnzipper_Alloc
( 
 LPCSTR					        pszFilename			/* IN : Filename or BASE64 string   */			
);

#ifndef _TEC_UNIX
// ----------------------------------------------------------------------
/// Allocated memory for a new unzipper handle.
///
/// \remarks
/// <em>TFORMer</em> functions accepting filenames like TFormer_SetRepositoryName, TFormer_SetDataFileXML, ... are accepting the following parameters:
/// * Plain file names (*.tfr, *.tff, *.xml)
/// * Zipped files (zipped *.tfr, zipped *.tff, ...)
/// * BASE64 encoded strings containing plain or zip files (in such a case the string must start with "BASE64:" - without apostrophes)
/// Usually all <em>TFORMer</em> functions accepting files are trying to determine if it is a zip-file automatically (a BASE64 encoded file is determined by the BASE64: prefix). 
/// When <em>TFORMer</em> detects a zip-file it is unzipped automatically to the temp-directory of the current user. This automatic unzipping requires <em>TFORMer</em> to clean up such temporary files after the function call ends.
/// If you want to avoid the per-function unzipping and deleting, then implement the unzipping functionality yourself by the TecUnzipper-functions. 
///
/// \remarks
/// Free allocated memory with #TecUnzipper_Free after use.
///
/// \remarks
/// This function may be changed in the future.
///
/// \returns - A valid handle or NULL if the handle could not be allocated.
///
/// \param[in]  pszFilename    The filename of a valid zip file or a BASE64 encoded string (e.g. BASE64:h/5tzWP%$378jdfh...).
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
HTUNZIPPER __stdcall		TecUnzipper_AllocW
( 
 LPCWSTR					      pszFilename			/* IN : Filename or BASE64 string as wide character string  */			
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Deallocates memory allocated by #TecUnzipper_Alloc.
///
/// \remarks
/// Free allocated memory with #TecUnzipper_Free after use. Files unzipped with function of this group are not deleted automatically, 
/// Call #TecUnzipper_DeleteAllFiles to do this.
///
/// \remarks
/// This function may be changed in the future.
///
/// \param[in]  htUnzipper    <em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
VOID		__stdcall		TecUnzipper_Free
( 
 HTUNZIPPER					htUnzipper			/* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
);

// ----------------------------------------------------------------------
/// Deletes all files and directories on disk that have been created by TecUnzipper_Extract.
///
/// \remarks
/// This function may be changed in the future.
///
/// \returns - ErrOk(0) on success
///
/// \see #TecUnzipper_Extract
///
/// \param[in]  htUnzipper    <em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE		__stdcall	TecUnzipper_DeleteAllFiles
( 
 HTUNZIPPER			  	htUnzipper				/* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
);

// ----------------------------------------------------------------------
/// Tests if the passed to #TecUnzipper_Alloc is a BASE64 encoded string and decodes it to a temporary file.
///
/// \remarks
/// The string must start with BASE64:. If this is not the case an error is returned. 
///
/// \see #TecUnzipper_Alloc
///
/// \returns - ErrOk(0) on success.
///
/// \param[in]  htUnzipper					<em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE			__stdcall	TecUnzipper_Base64Decode
(
 HTUNZIPPER					  htUnzipper			/* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
);

// ----------------------------------------------------------------------
/// Extracts a ZIP file into a temporary directory.
///
/// \remarks
/// This function may be changed in the future.
///
/// \returns - ErrOk(0) on success or an error if the data passed to #TecUnzipper_Alloc was not a valid zip-file.
///
/// \param[in]  htUnzipper    <em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
/// \param[in]  pszDirectory  Path for the target folder, NULL if <em>TFORMer</em> should create a temporary directory.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE		__stdcall		TecUnzipper_Extract
( 
 HTUNZIPPER					  htUnzipper			/* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
,LPCSTR						    pszDirectory		/* IN: Path for the target folder, NULL if TFORMer should create a temporary directory */
);

#ifndef _TEC_UNIX
// ----------------------------------------------------------------------
/// Extracts a ZIP file into a temporary directory.
///
/// \remarks
/// This function may be changed in the future.
///
/// \returns - ErrOk(0) on success or an error if the data passed to #TecUnzipper_Alloc was not a valid zip-file.
///
/// \param[in]  htUnzipper    <em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
/// \param[in]	 pszDirectory  Path for the target folder, NULL if <em>TFORMer</em> should create a temporary directory.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
ERRCODE		__stdcall		TecUnzipper_ExtractW
( 
 HTUNZIPPER					  htUnzipper			/* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
,LPCWSTR						  pszDirectory		/* IN: Path for the target folder, NULL if TFORMer should create a temporary directory */
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Returns the number of files that have been extracted.
///
/// \remarks
/// Returns the number of files that have been extracted with #TecUnzipper_Extract.  If no zip-file was specified (e.g. simply passing a BASE64 encoded tff) with #TecUnzipper_Alloc then the return value is one.
/// Use #TecUnzipper_GetFile for iterating the single files contained in the zip or BASE64 string.
///
/// \remarks
/// This function may be changed in the future.
///
/// \see #TecUnzipper_GetFile
///
/// \returns - the number of files that have been extracted.
///
/// \param[in]  htUnzipper    <em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC	
INT			__stdcall		  TecUnzipper_GetFileCount
( 
 HTUNZIPPER					  htUnzipper			/* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
);

// ----------------------------------------------------------------------
/// Returns the filename with the specified index (0..#TecUnzipper_GetFileCount - 1).
///
/// \remarks
/// Returns the absolute filename for the file with the specified index. The index starts at 0 and must be smaller than the return value of #TecUnzipper_GetFileCount.
///
/// \remarks
/// This function may be changed in the future.
///
/// \see #TecUnzipper_GetFileCount
/// \see #TecUnzipper_FindFileByExt
///
/// \returns - The absolute filename or NULL if the index is out of range. The returned string is only temporary available
///            and should be copied immediately! 
///
/// \param[in]  htUnzipper		<em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
/// \param[in]  nIndex			  0-based file index 
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR			__stdcall	TecUnzipper_GetFile
( 
 HTUNZIPPER					  htUnzipper			    /* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
,INT						      nIndex				      /* IN: File index */
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR			__stdcall	TecUnzipper_GetFileW
( 
 HTUNZIPPER					  htUnzipper			    /* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
,INT						      nIndex				      /* IN: File index */
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Finds the first file in the set of extracted files that matches the specified file extension.
///
/// \remarks
/// Finds the first file matching the specified file extension contained in an (optional BASE64 encoded) zip file. Usually this function is used to find contained *.tfr or *.tff files. 
/// Only call this function if #TecUnzipper_Extract returned ErrOk(0)
///
/// \remarks
/// This function may be changed in the future.
///
/// \see #TecUnzipper_GetFile
/// \see #TecUnzipper_Extract
///
/// \returns - The index of the first found file or -1 if no file was found (see #TecUnzipper_GetFile).
///
/// \param[in]  htUnzipper					      <em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
/// \param[in]  pszExtension				      Search file with this file extension only (the file extension must be specified without a dot at the beginning - e.g. tff)
/// \param[in]  nStartIndex				        Start index to start the search with (-1 for start at the beginning)
/// \param[in]  bCaseInsensitiveSearch		TRUE if the case should be case insensitive
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
INT			__stdcall	TecUnzipper_FindFileByExt
( 
 HTUNZIPPER				htUnzipper				      /* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
,LPCSTR		  	    pszExtension			      /* IN: Search file with this file extension only (the file extension must be specified without a dot at the beginning - e.g. tff) */	
,INT					    nStartIndex				      /* IN: Start index to start the search with (-1 for start at the beginning) */
,BOOL					    bCaseInsensitiveSearch	/* IN: TRUE if the case should be case insensitive */
);

#ifndef _TEC_UNIX
// ----------------------------------------------------------------------
/// Finds the first file in the set of extracted files that matches the specified file extension.
///
/// \remarks
/// Finds the first file matching the specified file extension contained in an (optional BASE64 encoded) zip file. Usually this function is used to find contained *.tfr or *.tff files. 
/// Only call this function if #TecUnzipper_Extract returned ErrOk(0)
///
/// \remarks
/// This function may be changed in the future.
///
/// \see #TecUnzipper_GetFile
/// \see #TecUnzipper_Extract
///
/// \returns - The index of the first found file or -1 if no file was found (see #TecUnzipper_GetFile).
///
/// \param[in]  htUnzipper					      <em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
/// \param[in]  pszExtension				      Search file with this file extension only (the file extension must be specified without a dot at the beginning - e.g. tff)
/// \param[in]  nStartIndex				        Start index to start the search with (-1 for start at the beginning)
/// \param[in]  bCaseInsensitiveSearch		TRUE if the case should be case insensitive
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
INT			__stdcall	TecUnzipper_FindFileByExtW
( 
 HTUNZIPPER				htUnzipper				      /* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
,LPCWSTR				  pszExtension			      /* IN: Search file with this file extension only (the file extension must be specified without a dot at the beginning - e.g. tff) */	
,INT					    nStartIndex				      /* IN: Start index to start the search with (-1 for start at the beginning) */
,BOOL					    bCaseInsensitiveSearch	/* IN: TRUE if the case should be case insensitive */
);
#endif // #ifndef _TEC_UNIX

// ----------------------------------------------------------------------
/// Creates a BASE64 encoded string.
///
/// \remarks
/// This is a helper function for non .NET based environments. When using .NET use the Convert class. 
///
/// \remarks
/// This function may be changed in the future.
///
/// \see #TecUnzipper_Base64_Free
///
/// \returns - A zero-terminated buffer containing a BASE64 encoded string. Free this buffer yourself with #TecUnzipper_Base64_Free.
///
/// \param[in]  szData					A pointer to a binary buffer containing the data to be encoded.
/// \param[in]  lLen					  The number of bytes contained in the buffer.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPSTR		__stdcall	TecUnzipper_Base64_Encode
( 
 LPCSTR					  szData					        /* IN: A pointer to a binary buffer containing the data to be encoded. */
,LONG					    lLen					          /* IN: The number of bytes contained in the buffer. */
);

// ----------------------------------------------------------------------
/// Free a BASE64 encoded string created with #TecUnzipper_Base64_Encode.
///
/// \remarks
/// This function may be changed in the future.
///
/// \see #TecUnzipper_Base64_Encode
///
/// \param[in]  szStr					A pointer to a BASE64 buffer returned by #TecUnzipper_Base64_Encode.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
VOID			__stdcall	TecUnzipper_Base64_Free
(
 LPSTR						  szStr		              /* IN: A pointer to a BASE64 buffer returned by #TecUnzipper_Base64_Encode. */
);


// ----------------------------------------------------------------------
/// Returns the filename of the zip file being used or of the temp file created by #TecUnzipper_Base64Decode.
///
/// \remarks
/// This function may be changed in the future.
///
/// \see #TecUnzipper_Base64_Encode
///
/// \returns - A filename or NULL in case of an error. The returned string is only temporary available and should be copied immediately! 
///
/// \param[in]  htUnzipper					<em>TECUnzipper</em>-handle - see #TecUnzipper_Alloc.
///
/// \ingroup group_ZIPHandler
// ----------------------------------------------------------------------
TECIT_DLLSPEC
LPCSTR			__stdcall	TecUnzipper_GetFilename
( 
 HTUNZIPPER					  htUnzipper			    /* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
);

#ifndef _TEC_UNIX
TECIT_DLLSPEC
LPCWSTR			__stdcall	TecUnzipper_GetFilenameW
( 
 HTUNZIPPER					  htUnzipper			    /* IN: TECUnzipper-handle - see #TecUnzipper_Alloc */
);
#endif // #ifndef _TEC_UNIX

#ifdef __cplusplus
  }
#endif

  
  
#endif
