/*===========================================================================*/
/*                        Copyright (C) 2009 by                              */
/*                                                                           */
/*                    -- TEC-IT Datenverarbeitung GmbH --                    */
/*   -- team for engineering and consulting in information technologies --   */
/*                                                                           */
/*                           All rights reserved.                            */
/*                                                                           */
/*              This is a part of the TEC-IT Standard Software.              */
/*                                                                           */
/*    This source code is only intended as a supplement to the References    */
/*      and related electronic documentation provided with the product.      */
/*                                                                           */
/*     See these sources for detailed information regarding this product     */
/*===========================================================================*/
package com.tecit.TFORMer.examples;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.tecit.TFORMer.TFORMer;
import com.tecit.TFORMer.TFormerException;
import com.tecit.TFORMer.Enumerations.EPrinterType;
import com.tecit.TFORMer.Printing.Job;
import com.tecit.TFORMer.Printing.JobDataRecordSet;

/**
 * A simple example of the JFORMer using Servlet
 * @author DavideZ
 */
public class JTFORMerServlet extends HttpServlet
{
	final static private String PARAM_COPIES    = "copies";
	final static private String PARAM_START_ROW = "startrow";
	final static private String PARAM_START_COL = "startcol";
	final static private String PARAM_TEMPLATE_FILE = "templfile";
	
	/**
	 * Process the user request (POST and GET)
	 */
	protected void service(HttpServletRequest request, HttpServletResponse response)
	throws IOException
	{
		PrintInfo info = new PrintInfo();
		boolean noMissingValues = true;
		
		try {
			noMissingValues &= info.setCopies(request.getParameter(PARAM_COPIES));
			noMissingValues &= info.setStartColumn(request.getParameter(PARAM_START_COL));
			noMissingValues &= info.setStartRow(request.getParameter(PARAM_START_ROW));
			noMissingValues &= info.setTemplateFile(request.getParameter(PARAM_TEMPLATE_FILE));
			if( noMissingValues ){
				// print the file as requested by the user... 
				print(info);
				// ... and send the file printed
				sendFile(response, info.getOutputFile());
			} else {
				// notify to the user the missing values 
				writePage(response, info, "Insert all parameters", null);
			}
		}
		catch(Throwable ex){
			// notify to the user the exception 
			writePage(response, info, null, ex);
		}
	}
	
	/**
	 * Send the HTML page with the fields with which the user can custom 
	 * the document
	 */
	private void writePage(HttpServletResponse response, PrintInfo info
			                  , String msg, Throwable ex)
	throws IOException
	{
		PrintWriter out = response.getWriter();

		response.setContentType("text/html");
		out.println("<html>");
		out.println("<head>");
		out.println("<title>Servlet using Java TFORMer Runtime Example</title>");
		out.println("</head>");
		out.println("<body>");
		out.println("<h1>Servlet using Java TFORMer Runtime Example</h1>");
		out.println("<form name=\"form1\" method=\"POST\">");
		
		if( msg!=null )
			out.println("<b>"+msg+"</b><br/>");

		if( ex!=null ){
			out.println("<i>"+ex.getMessage()+"</i><br/><pre>");
			ex.printStackTrace(out);
			out.println("</pre><br/>");
		}
		
		out.println("<table border=0>");
		out.println("<tr><td>Template file:</td>");
		out.println("<td><input type=\"text\" size=100 name=\""+PARAM_TEMPLATE_FILE+"\" value=\""+info.templateFile+"\" /></td></tr>");

		out.println("<tr><td>Number of copies:</td>");
		out.println("<td><input type=\"text\" size=100 name=\""+PARAM_COPIES+"\" value=\""+info.copies+"\" /></td></tr>");

		out.println("<tr><td>Start column:</td>");
		out.println("<td><input type=\"text\" size=100 name=\""+PARAM_START_COL+"\" value=\""+info.startColumn+"\" /></td></tr>");
		
		out.println("<tr><td>Start row:</td>");
		out.println("<td><input type=\"text\" size=100 name=\""+PARAM_START_ROW+"\" value=\""+info.startRow+"\" /></td></tr>");

		out.println("<tr><td align=\"center\" size=100 colspan=2><input type=\"submit\" value=\"Print as PDF !\" /></td></tr>");
		
		out.println("</table>");
		out.println("<p><a href=\"http://www.tec-it.com\"><img alt=\"Open TEC-IT Homepage\" src=\"../TEC-IT_Banner_120x42.gif\" /></a>");
		out.println("</form>");
		out.println("</body>");
		out.println("<html>");
	}
	
	/**
	 * Send the document printed by TFORMer Runtime 
	 */
	private void sendFile(HttpServletResponse response, File file)
	throws IOException
	{
		response.setContentType("application/pdf");
		response.setHeader("Content-Length", String.valueOf(file.length()));
    response.setHeader("Content-Disposition", "attachment; filename=\"J2EE_TFORMer_file.pdf\"");
    
    OutputStream out = response.getOutputStream();
    FileInputStream fis = new FileInputStream(file);
    byte[] arr = new byte[4096];
    int size;
    try {
    	while( (size=fis.read(arr))>=0 ){
    		out.write(arr, 0, size);
    	}
    }
    catch(IOException ex){
    	throw ex;
    }
    finally {
    	// delete the TFORMer file!
    	file.delete();
    	fis.close();
    }
	}
	
	/**
	 * Execute the generation of the form
	 */
	private void print(PrintInfo pi)
	throws TFormerException
	{
		Job job = null;
		String msg = null;
		
		try {
			msg = "load library using path "+System.getProperty("java.library.path");
			job = new Job();
		
			msg = "set repository "+ pi.getTemplateFile();
			job.setRepositoryName(pi.getTemplateFile());
			
			msg = "set project name "+pi.getProjectName();
			job.setProjectName(pi.getProjectName());
			
			msg = "set form name "+pi.getFormName();
			job.setFormName(pi.getFormName());
			
			msg = "set records";
			JobDataRecordSet datasource = new JobDataRecordSet();
			Iterator records = pi.getRecords();
			for(int j=1; records.hasNext(); j++){
				msg = "add record #"+j;
				datasource.add((JobDataRecordSet.Record)records.next());
			}
			
			msg = "create output type "+pi.getOutputType();
			job.setPrinterType(pi.getOutputType());

			EPrinterType outputType = pi.getOutputType();
			if( outputType==EPrinterType.Default 
					|| outputType==EPrinterType.Cups 
					|| outputType==EPrinterType.Win32 ){
				// use printer
				msg = "set printer "+pi.getPrinterName();
				job.setPrinterName(pi.getPrinterName());
			} else {
				// print to file
				msg = "create output file "+pi.getOutputFile();
				job.setOutputName(pi.getOutputFile().getAbsolutePath());
			}

			msg = "insert data to job";
			job.setJobData(datasource);

			msg = "set the number of copies";
			job.setNumberOfCopies(pi.getCopies());

			msg = "set the start column";
			job.setStartColumn(pi.getStartColumn());

			msg = "set the number of copies";
			job.setStartRow(pi.getStartRow());
			
			msg = "printing";
			job.print();
		}
		catch(TFormerException ex){
			throw new TFormerException("Failed to "+msg, ex);
		}
		catch(Throwable ex){
			throw new TFormerException("Failed to "+msg, ex);
		}
		finally {
			// free the resources otherwise some memory remains allocated!
			// NOTE: the method "dispose" must be invoked also when an
			//       exception occurs (good practice is the use of finally)
			// free the resources
			try {
				if( job!=null )
					job.dispose();
			} 
			catch (TFormerException ex) {
				throw new TFormerException("Failed to dispose", ex);
			} 
		}
	}
	
	/**
	 * Simple bean class to maintain the user variables 
	 */
	protected class PrintInfo
	{
		private String projectName;
		private String formName;
		private String templateFile;
		private EPrinterType outputType;
		private File outputFile;
		private String printerName;
		private String copies;
		private String startColumn;
		private String startRow;
		private ArrayList records;

		public PrintInfo()
		{
			this.projectName = "TFORMer_Runtime_Examples";
			this.formName = "BarcodeLabels";

			File file = TFORMer.getExampleFile("Demo Repository", "Demos.tfr");
			this.templateFile = file.getAbsolutePath();

			this.outputType = EPrinterType.PDFFile;

			try {
				this.outputFile = File.createTempFile("TFORMer", "pdf");
				this.outputFile.deleteOnExit();
			}
			catch(IOException ex){
				this.outputFile = new File(System.getProperty("temp.dir")+File.separator
						                      +"TFORMer"+System.currentTimeMillis());
			}
			this.printerName = null;
			this.copies = "1";
			this.startColumn = "0";
			this.startRow = "0";

			this.records = new ArrayList();
			JobDataRecordSet.Record record = new JobDataRecordSet.Record();
			record.setData("ArticleNo",  "20021001");                   
			record.setData("ArticleName",  "Sound Blaster Audigy 2");  
			record.setData("ArticlePrice", "175");
			this.records.add(record);
			
			record = new JobDataRecordSet.Record();
			record.setData("ArticleNo",  "20021002");                   
			record.setData("ArticleName",  "Creative Labs X-Fi ");
			record.setData("ArticlePrice", "130");
			this.records.add(record);
			
			record = new JobDataRecordSet.Record();
			record.setData("ArticleNo",  "20021003");
			record.setData("ArticleName",  "Creative Labs Live!");
			record.setData("ArticlePrice", "29");
			this.records.add(record);
			 
			record = new JobDataRecordSet.Record();
			record.setData("ArticleNo",  "20021004");
			record.setData("ArticleName",  "Griffin Technology iMic ");
			record.setData("ArticlePrice", "30");
			this.records.add(record);
			
			record = new JobDataRecordSet.Record();
			record.setData("ArticleNo",  "20021005");
			record.setData("ArticleName",  "MadDog MultiMedia Predator");
			record.setData("ArticlePrice", "30");
			this.records.add(record);
			
			record = new JobDataRecordSet.Record();
			record.setData("ArticleNo",  "20021006");
			record.setData("ArticleName",  "Creative Labs PCMCIA ");
			record.setData("ArticlePrice", "90");
			this.records.add(record);
		}
		
		public String getProjectName()
		{
			return projectName;
		}
		public String getFormName()
		{
			return formName;
		}
		public boolean setTemplateFile(String value)
		{
			if( value==null || value.length()==0 ) return false;
			this.templateFile = value;
			return true;
		}
		public String getTemplateFile()
		{
			return templateFile;
		}
		public EPrinterType getOutputType()
		{
			return outputType;
		}
		public File getOutputFile()
		{
			return outputFile;
		}
		public String getPrinterName()
		{
			return printerName;
		}
		public Iterator getRecords()
		{
			return this.records.iterator();
		}
		public boolean setCopies(String value)
		{
			if( value==null || value.length()==0 ) return false;
			this.copies = value;
			return true;
		}
		public int getCopies()
		{
			return Integer.parseInt(copies);
		}
		public boolean setStartColumn(String value)
		{
			if( value==null || value.length()==0 ) return false;
			this.startColumn = value;
			return true;
		}
		public int getStartColumn()
		{
			return Integer.parseInt(startColumn);
		}
		public boolean setStartRow(String value)
		{
			if( value==null || value.length()==0 ) return false;
			this.startRow = value;
			return true;
		}
		public int getStartRow()
		{
			return Integer.parseInt(startRow);
		}
	}
}
