/*===========================================================================*/
/*                        Copyright (C) 2009 by                              */
/*                                                                           */
/*                    -- TEC-IT Datenverarbeitung GmbH --                    */
/*   -- team for engineering and consulting in information technologies --   */
/*                                                                           */
/*                           All rights reserved.                            */
/*                                                                           */
/*              This is a part of the TEC-IT Standard Software.              */
/*                                                                           */
/*    This source code is only intended as a supplement to the References    */
/*      and related electronic documentation provided with the product.      */
/*                                                                           */
/*     See these sources for detailed information regarding this product     */
/*===========================================================================*/
package com.tecit.TFORMer.examples;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.event.ListDataListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import com.tecit.TFORMer.TFORMer;
import com.tecit.TFORMer.TFormerException;
import com.tecit.TFORMer.Enumerations.EPrinterType;
import com.tecit.TFORMer.Printing.Job;
import com.tecit.TFORMer.Printing.JobDataRecordSet;
import com.tecit.TFORMer.Repository.FormLayout;
import com.tecit.TFORMer.Repository.Project;
import com.tecit.TFORMer.Repository.Repository;

/**
 * A simple example of the JFORMer using Swing Framework
 * @author DavideZ
 */
public class JTFORMerSwing extends JFrame implements ActionListener 
{
	private static final long serialVersionUID = -6523558030819178014L;
	
	/** program name */
	private static String TITLE = "TEC-IT JFORMer Example";
	/** data format */
	private static String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
	/** date formatter */
	private static SimpleDateFormat SDF = new SimpleDateFormat(DATE_FORMAT);

	/** button to generate/execute JFORMer */
	private JButton btnGenerate;
	/** button to extract information from the template */
	private JButton btnShowTemplateInfo;
	/** name of the form */
	private JTextField fldFormName;
	/** name of the project */
	private JTextField fldProject; 
	/** name of the template file */
	private JTextField fldTemplate; 
	/** button to search the template file */
	private JButton btnTemplateChoose;
	/** name of the output file */
	private JTextField fldOuputFile;
	/** name of the printer */
	private JTextField fldPrinterName;
	/** button to search the output file */
	private JButton btnOuputFileChoose;
	/** text area for the user messages */
	private JTextArea txtMessages;
	/** record model to use into JTable */
	private JobRecordsTableModel recordsTableModel;
	/** output types model to use into JComboBox */
	private OutputTypesModel outputTypesModel;
	/** flag for platform type */
	boolean isUnix; 
	
	/**
	 * Entry point
	 */
	public static void main(String[] args)
	{
		JFrame frame = new JTFORMerSwing();
    frame.setTitle(TITLE);
    frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    frame.pack();
    frame.setVisible(true);		
	}

	/**
	 * Constructor
	 * This method create the GUI of the program
	 */
	public JTFORMerSwing()
	{
		this.isUnix = (System.getProperty("file.separator").equals("/"));

		this.btnGenerate = new JButton("Generate");
		this.btnGenerate.addActionListener(this);
		this.btnShowTemplateInfo = new JButton("Show template info");
		this.btnShowTemplateInfo.addActionListener(this);
		this.fldFormName = new JTextField();
		this.fldProject = new JTextField();
		this.fldTemplate = new JTextField();
		this.btnTemplateChoose = new JButton("...");
		this.btnTemplateChoose.addActionListener(this);
		this.fldPrinterName = new JTextField();
		this.fldOuputFile = new JTextField();
		this.btnOuputFileChoose = new JButton("...");
		this.btnOuputFileChoose.addActionListener(this);
		this.txtMessages = new JTextArea();
		this.outputTypesModel = new OutputTypesModel();
		
		//
		// create the GUI 
		//
		JTable table = new JTable();
		JPanel formPanel = new JPanel();
    
		// create the form panel
		FormPanelBuilder fpb = new FormPanelBuilder(formPanel);
    fpb.add("Template file", this.fldTemplate, this.btnTemplateChoose);
    fpb.add("Project name" , this.fldProject);
    fpb.add("Form name"    , this.fldFormName);
    fpb.add("Output type"  , new JComboBox(this.outputTypesModel));
    fpb.add("Output file"  , this.fldOuputFile,this.btnOuputFileChoose);
    fpb.add("Printer name" , this.fldPrinterName);
    fpb.add("Records data" , table);

    // botton pannel
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(1,2));
		buttonPanel.add(this.btnGenerate);
		buttonPanel.add(this.btnShowTemplateInfo);
		
		// create the panel with form and button "generate"
		JPanel centralPanel = new JPanel();
		centralPanel.setLayout(new BorderLayout());
		centralPanel.add(formPanel, BorderLayout.CENTER);
		centralPanel.add(buttonPanel, BorderLayout.SOUTH);

		JScrollPane messagePanel = new JScrollPane(this.txtMessages);
		messagePanel.setPreferredSize(new Dimension(250, 145));
		
		// create the main panel with two tabs 
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Main"   , centralPanel);
		tabbedPane.addTab("Console", messagePanel); 
		add(tabbedPane);

		//
		// setting the default values
		//
		JobDataRecordSet.Record record;
		File file = TFORMer.getExampleFile("Demo Repository", "Demos.tfr");
		this.fldTemplate.setText(file.getAbsolutePath());

		this.fldFormName.setText("BarcodeLabels");
		this.fldFormName.setEditable(false);
		this.fldOuputFile.setText(System.getProperty("user.home")+File.separator
				+"JTFORMerExample.pdf");

		this.fldProject.setText("TFORMer_Runtime_Examples");
		this.fldProject.setEditable(false);
		
		this.recordsTableModel = new JobRecordsTableModel(
				new String[]{"ArticleNo", "ArticleName", "ArticlePrice"});
		record = new JobDataRecordSet.Record();
		record.setData("ArticleNo",  "20021001");                   
    record.setData("ArticleName",  "Sound Blaster Audigy 2");  
    record.setData("ArticlePrice", "175");
    this.recordsTableModel.add(record);

		record = new JobDataRecordSet.Record();
    record.setData("ArticleNo",  "20021002");                   
    record.setData("ArticleName",  "Creative Labs X-Fi ");
    record.setData("ArticlePrice", "130");
    this.recordsTableModel.add(record);

		record = new JobDataRecordSet.Record();
    record.setData("ArticleNo",  "20021003");
    record.setData("ArticleName",  "Creative Labs Live!");
    record.setData("ArticlePrice", "29");
    this.recordsTableModel.add(record);
    
		record = new JobDataRecordSet.Record();
    record.setData("ArticleNo",  "20021004");
    record.setData("ArticleName",  "Griffin Technology iMic ");
    record.setData("ArticlePrice", "30");
    this.recordsTableModel.add(record);
 
		record = new JobDataRecordSet.Record();
    record.setData("ArticleNo",  "20021005");
    record.setData("ArticleName",  "MadDog MultiMedia Predator");
    record.setData("ArticlePrice", "30");
    this.recordsTableModel.add(record);

		record = new JobDataRecordSet.Record();
    record.setData("ArticleNo",  "20021006");
    record.setData("ArticleName",  "Creative Labs PCMCIA ");
    record.setData("ArticlePrice", "90");
    this.recordsTableModel.add(record);
    table.setModel(this.recordsTableModel);
    
		info("Program ready");
	}

	/**
	 * Perform the events of the GUI
	 */
	public void actionPerformed(ActionEvent event)
	{
		String errorMessage = null;
		
		if( event.getSource()==this.btnGenerate ){
			//
			// button "Generate" event
			//
			// execute the generation of the form
			boolean res = print(this.fldProject.getText(), this.fldFormName.getText()
					, this.fldTemplate.getText(), this.outputTypesModel.getSelectedType()
					, this.fldOuputFile.getText(), this.fldPrinterName.getText()
					, this.recordsTableModel.records());
			if( res ){
				// print successfull
				JOptionPane.showMessageDialog(this, "Print successfull\n"
						+"See messages into console tab."
						, TITLE, JOptionPane.OK_OPTION | JOptionPane.INFORMATION_MESSAGE);
			}
			else errorMessage = "Print failed: see details into \"Console\"";
		}
		else if( event.getSource()==this.btnShowTemplateInfo ){
			//
			// button "Show template info" event
			//
			String templateFileName = this.fldTemplate.getText();
			if( templateFileName==null || templateFileName.equals("") ){
				errorMessage = "Choose first the template file";
			} 
			else if( extractTemplateInfo(templateFileName) ){
				// extraction successfull
				JOptionPane.showMessageDialog(this, "See information into \"Console\""
						, TITLE, JOptionPane.OK_OPTION | JOptionPane.INFORMATION_MESSAGE);
			}
			else errorMessage = "Extraction failed: see details into \"Console\"";
		}
		else if( event.getSource()==this.btnTemplateChoose ){
			//
			// button to choose template file
			//
			// show the dialog with which the use can choose the template file
			JFileChooser fc = new JFileChooser(this.fldTemplate.getText());
			fc.setFileSelectionMode(JFileChooser.FILES_ONLY );
			if( fc.showOpenDialog(this)==JFileChooser.APPROVE_OPTION){
				this.fldTemplate.setText(fc.getSelectedFile().getAbsolutePath());
			}
		}
		else if( event.getSource()==this.btnOuputFileChoose ){
			//
			// button to choose output file
			//
			// show the dialog with which the use can choose the output file
			JFileChooser fc = new JFileChooser(this.fldOuputFile.getText());
			if( fc.showOpenDialog(this)==JFileChooser.APPROVE_OPTION){
				this.fldOuputFile.setText(fc.getSelectedFile().getAbsolutePath());
			}
		}

		if( errorMessage!=null )
			JOptionPane.showMessageDialog(this, errorMessage, TITLE
				, JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
	}
	
	/**
	 * Add a user message as information
	 * @param text content of the message
	 */
	private void info(String text)
	{
		this.txtMessages.append(SDF.format(new Date())+" ");
		this.txtMessages.append("INFO> "+text+"\n");
	}

	/**
	 * Add a user message as error
	 * @param text content of the message
	 */
	private void error(String text, Throwable ex)
	{
		this.txtMessages.append(SDF.format(new Date())+" ");
		this.txtMessages.append("ERROR> "+text);
		if( ex!=null ){
			// add information if the exception is generated by TFORMer
			if( ex instanceof TFormerException ){
				TFormerException fex = (TFormerException) ex;
				this.txtMessages.append("\n         Message: "+fex.getMessage());
				this.txtMessages.append("\n     Information: "+fex.getInformation());
				this.txtMessages.append("\nSource reference: "+fex.getSourceReference());
				this.txtMessages.append("\nError code (int): "+fex.getErrorCode());
			}
			
			// add the stack trace 
			this.txtMessages.append("\n"+ex.getClass().getName()+": "+ex.getMessage());
			StackTraceElement[] traces = ex.getStackTrace();
			for(int j=0; traces!=null && j<traces.length; j++){
				this.txtMessages.append("\n   at "+traces[j].getClassName());
				this.txtMessages.append("."+traces[j].getMethodName());
				if( traces[j].getFileName()==null ){
					this.txtMessages.append("(Unknown Source)");					
				} 
				else if( traces[j].getLineNumber()<0 ){
					this.txtMessages.append("("+traces[j].getFileName()+":");
					this.txtMessages.append("native)");
				} else {
					this.txtMessages.append("("+traces[j].getFileName()+":");
					this.txtMessages.append(traces[j].getLineNumber()+")");
				}
			}
		}
		this.txtMessages.append("\n");
	}

	/**
	 * Extract informations from the template
	 * @param templateFile name of the template file
	 */
	private boolean extractTemplateInfo(String templateFile)
	{
		String msg = null;
		boolean res = true;
		Repository repos = null;
		Project project, oldProject;
		FormLayout formLayout, oldFormLayout;
		String value;
		
		try {
			msg = "load template file "+templateFile;
			repos = new Repository(templateFile, false, false);
			info(msg);

			msg = "getDescription()";
			value = repos.getDescription();
			info("description="+value);

			msg = "getComment()";
			value = repos.getComment();
			info("comment="+value);
			
			msg = "isModified()";
			value = String.valueOf(repos.isModified());
			info("isModified="+value);

			msg = "getFirstProject()";
			project = repos.getFirstProject();
			
			while( project!=null ){
				msg = "project.getName()";
				value = project.getName();
				info("*********** Project "+value+" ***********");
				
				msg = "project.getDescription()";
				value = project.getDescription();
				info("description="+value);
				
				msg = "project.getComment()";
				value = project.getComment();
				info("comment="+value);
	
				msg = "project.getNumberOfFormLayouts()";
				value = String.valueOf(project.getNumberOfFormLayouts());
				info("total forms="+value);
				
				// show details of the forms
				msg = "project.getFirstFormLayout()";
				formLayout = project.getFirstFormLayout();
				
				while( formLayout!=null ){
					msg = "formLayout.getName()";
					value = formLayout.getName();
					info("*********** Form "+value+" ***********");
					
					msg = "formLayout.getDescription()";
					value = formLayout.getDescription();
					info("description="+value);
					
					msg = "formLayout.getComment()";
					value = formLayout.getComment();
					info("comment="+value);

					// pass to next form
					msg = "formLayout.getNext()";
					formLayout = (oldFormLayout=formLayout).getNext();
					
					// free the resources
					msg = "formLayout.dispose()";
					oldFormLayout.dispose();
				}				
				
				// pass to next project
				msg = "project.getNext()";
				project = (oldProject=project).getNext();
				
				// free the resources
				msg = "project.dispose()";
				oldProject.dispose();
			}
		}
		catch(TFormerException ex){
			error("Failed to "+msg, ex);
			res = false;
		}
		catch(Throwable ex){
			error("Failed to "+msg, ex);
			res = false;
		}
		finally {
			// free the resources
			try {
				if( repos!=null )
					repos.dispose();
			} 
			catch (TFormerException e) {
				e.printStackTrace();
			} 
		}
		return res;
	}
		
	/**
	 * Execute the generation of the form
	 * @param projectName name of the project
	 * @param formName name of the form
	 * @param templateFile name of the template file
	 * @param outputType type of output
	 * @param outputFile name of the output file
	 * @param printerName name of the printer
	 * @param records iterator with the records to insert into the document 
	 */
	private boolean print(String projectName, String formName, String templateFile
			, EPrinterType outputType, String outputFile, String printerName
			, Iterator<JobDataRecordSet.Record> records)
	{
		Job job = null;
		String msg = null;
		boolean res = true;
		
		try {
			msg = "load library using path "+System.getProperty("java.library.path");
			job = new Job();
			info(msg);
		
			msg = "set repository "+ templateFile;
			job.setRepositoryName(templateFile);
			info(msg);
			
			msg = "set project name "+projectName;
			job.setProjectName(projectName);
			info(msg);
			
			msg = "set form name "+formName;
			job.setFormName(formName);
			info(msg);
			
			JobDataRecordSet datasource = new JobDataRecordSet();
			for(int j=1; records.hasNext(); j++){
				msg = "add record #"+j;
				datasource.add(records.next());
				info(msg);
			}
			
			msg = "create output type "+outputType;
			job.setPrinterType(outputType);
			info(msg);

			if( outputType==EPrinterType.Default 
					|| outputType==EPrinterType.Cups 
					|| outputType==EPrinterType.Win32 ){
				// use printer
				msg = "set printer "+printerName;
				job.setPrinterName(printerName);
				info(msg);
			} else {
				// print to file
				msg = "create output file "+outputFile;
				job.setOutputName(outputFile);
				info(msg);
			}

			msg = "insert data to job";
			job.setJobData(datasource);
			info(msg);
			
			msg = "printing";
			job.print();
			info(msg);
			
			info("file printed");
		}
		catch(TFormerException ex){
			error("Failed to "+msg, ex);
			res = false;
		}
		catch(Throwable ex){
			error("Failed to "+msg, ex);
			res = false;
		}
		finally {
			// free the resources otherwise some memory remains allocated!
			// NOTE: the method "dispose" must be invoked also when an
			//       exception occurs (good practice is the use of finally)
			// free the resources
			try {
				if( job!=null )
					job.dispose();
			} 
			catch (TFormerException ex) {
				error("Failed to dispose", ex);
			} 
		}
		return res;
	}
	
	/**
	 * TFORMer output types applied to combobox
	 */
	protected class OutputTypesModel implements ComboBoxModel
	{
		private HashMap<String, EPrinterType> values;
		private Object[] keys;
		private Object selected;
		
		public OutputTypesModel()
		{
			super();
			
			// create the index map
			this.values = new HashMap<String, EPrinterType>();
			this.values.put("Default printer", EPrinterType.Default); 
			this.values.put("Defined printer in template", EPrinterType.Repository);
			this.values.put("User printer", (isUnix ? EPrinterType.Cups 
					                                    : EPrinterType.Win32));
			this.values.put("File PostScript", EPrinterType.PSFile);                    
			this.values.put("File PDF", EPrinterType.PDFFile);
			this.values.put("File HTML", EPrinterType.HTML);
			this.values.put("File Text", EPrinterType.TxtFile);
			this.values.put("File Bitmap", EPrinterType.ImageBmp);
			this.values.put("File GIF", EPrinterType.ImageGif);
			this.values.put("File Jpeg", EPrinterType.ImageJpg);
			this.values.put("File PCX", EPrinterType.ImagePcx);
			this.values.put("File PNG", EPrinterType.ImagePng);
			this.values.put("File TGA", EPrinterType.ImageTga);
			this.values.put("File TIFF", EPrinterType.ImageTifSingle);
			this.values.put("File TIFF Multipage", EPrinterType.ImageTifMultiPage);
			this.values.put("File ZPL-II output (ZEBRA printers)", EPrinterType.ZPLFile);
			
			// extract the keys and order them
			this.keys = this.values.keySet().toArray();
			Arrays.sort(this.keys);
			
			// setting the default value
			this.selected = this.keys[0];
		}

		public EPrinterType getSelectedType()
		{
			return this.values.get(this.selected);
		}
		
		public Object getSelectedItem()
		{
			return this.selected;
		}

		public void setSelectedItem(Object selected)
		{
			this.selected = selected; 
		}

		public void addListDataListener(ListDataListener listener)
		{}

		public Object getElementAt(int index)
		{
			return this.keys[index];
		}

		public int getSize()
		{
			return this.keys.length;
		}

		public void removeListDataListener(ListDataListener listener)
		{}
	}
	
	/**
	 * Record Data model
	 * This class shows how use JTFORMer record in Java objects  
	 */
	protected class JobRecordsTableModel implements TableModel
	{
		private String[] columns;
		private ArrayList<JobDataRecordSet.Record> records;
		
		public JobRecordsTableModel(String[] columns)
		{
			this.columns = columns;
			this.records = new ArrayList<JobDataRecordSet.Record>();
		}

		public void add(JobDataRecordSet.Record record)
		{
			this.records.add(record);
		}

		public Class<String> getColumnClass(int columnIndex)
		{
			return String.class;
		}
		
		public int getRowCount()
		{
			return this.records.size();
		}

	  public String getColumnName(int columnIndex)
	  {
	  	return this.columns[columnIndex];
	  }
		
	  public int getColumnCount()
	  {
	  	return this.columns.length;
	  }
	 
	  public boolean isCellEditable(int rowIndex, int columnIndex)
	  {
	  	return true;
	  }

	  public void setValueAt(Object value, int rowIndex, int columnIndex)
	  {
	  	JobDataRecordSet.Record record;
	  	
	  	if( rowIndex==this.records.size() ){
	  		// new row
	  		record = new JobDataRecordSet.Record();
	  	}
	  	else record = this.records.get(rowIndex);
	  		
	  	record.setData(this.columns[columnIndex]
	  	                            , (value==null ? null : value.toString()));
	  }
	  
	  public Object getValueAt(int rowIndex, int columnIndex)
	  {
	  	if( rowIndex<0 || rowIndex>=this.records.size() )
	  		return null;
	  	
	  	JobDataRecordSet.Record record = this.records.get(rowIndex);
	  	return record.getData(this.columns[columnIndex]);
	  }
	  
	  public Iterator<JobDataRecordSet.Record> records()
	  {
	  	return this.records.iterator();
	  }
	  
	  public void addTableModelListener(TableModelListener l)
	  {}

	  public void removeTableModelListener(TableModelListener l)
	  {}
	}

	/**
	 * Class to help the creation of a form panel
	 */
	protected class FormPanelBuilder
	{
		private JPanel panel;
		private GridBagLayout gbl = new GridBagLayout();
		public GridBagConstraints gbc = new GridBagConstraints();
		private int rows;
		
		public FormPanelBuilder(JPanel panel)
		{
			this.panel = panel;
			this.panel.setLayout(gbl);
		}

		public void add(String label, JComponent input)
		{
			add(label, input, null);
		}
		
		public void add(String label, JComponent input, JComponent optional)
		{
	    this.gbc.gridwidth = 1;
			add(new JLabel(label+":"), rows, 0, false);
			if( optional==null ){
		    gbc.gridwidth = 2;
				add(input, rows, 1, true);
			} else {
				add(input   , rows, 1, false);
				add(optional, rows, 2, true);
			}
			rows++;
		}
		
		public void add(JComponent component, int row, int column, boolean isLast)
		{
			gbc.gridx = (column==0 ? 0 : GridBagConstraints.RELATIVE);
	    gbc.gridy = row;
	    if( isLast )
	    	gbc.gridwidth = GridBagConstraints.REMAINDER; //end row
	    if( column==1 )
	    	gbc.fill = GridBagConstraints.HORIZONTAL;
	    gbl.setConstraints(component, gbc);
	    panel.add(component);
		}
	}
}
