/*===========================================================================*/
/*                        Copyright (C) 2007 - 2009 by                       */
/*                                                                           */
/*                    -- TEC-IT Datenverarbeitung GmbH --                    */
/*   -- team for engineering and consulting in information technologies --   */
/*                                                                           */
/*                           All rights reserved.                            */
/*                                                                           */
/*              This is a part of the TEC-IT Standard Software.              */
/*                                                                           */
/*    This source code is only intended as a supplement to the References    */
/*      and related electronic documentation provided with the product.      */
/*                                                                           */
/*     See these sources for detailed information regarding this product     */
/*===========================================================================*/

/*=============================================================================
   This sample application generates PDF output (based on the form named 
   BusinessCard_Multi in the project TFORMer_Runtime_Examples of the installed 
   Demo Repository.
   ----------------------------------------------------------------------------
   History:
   (2007-04-20) : created (gk)
   (2007-10-27) : adapted to V5.1
=============================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using TECIT.TFORMer;

namespace ConsoleApplication1
{
    class Program
    {
        static string oName;

        static TECIT.TFORMer.PrinterType OutputType()
        {
        
        ConsoleKeyInfo cki;
        TECIT.TFORMer.PrinterType [] ot = {PrinterType.Win32, PrinterType.PostscriptFile, PrinterType.PdfFile, PrinterType.HtmlFile, PrinterType.TextFile, PrinterType.ePrinterType_ImageBmp, PrinterType.ePrinterType_ImageGif, PrinterType.ePrinterType_ImageJpg, PrinterType.ePrinterType_ImagePcx, PrinterType.ePrinterType_ImagePng, PrinterType.ePrinterType_ImageTga, PrinterType.ePrinterType_ImageTifSingle, PrinterType.ePrinterType_ImageTifMultiPage, PrinterType.ePrinterType_ZPLFile};
        // note: the # in the filename is a placeholder for the current page number
        String [] fName = {"", "Output.PS", "Output.PDF", "Output.HTML", "Output.TXT", "Output##.BMP", "Output##.GIF", "Output##.JPG", "Output##.PCX", "Output##.PNG", "Output##.TGA", "Output##.TIF", "Output.TIF", "Output.ZPL"};
        int i;

        do
        {
            System.Console.Clear();
            System.Console.WriteLine("1...Print directly to default printer");
            System.Console.WriteLine("2...PostScript output file");
            System.Console.WriteLine("3...PDF output file");
            System.Console.WriteLine("4...HTML output file");
            System.Console.WriteLine("5...Text output file");
            System.Console.WriteLine("6...BMP output image");
            System.Console.WriteLine("7...GIF output image");
            System.Console.WriteLine("8...JPG output image");
            System.Console.WriteLine("9...PCX output image");
            System.Console.WriteLine("A...PNG output image");
            System.Console.WriteLine("B...TGA output image");
            System.Console.WriteLine("C...TIF output image");
            System.Console.WriteLine("D...Multipage TIF output image");
            System.Console.WriteLine("E...ZPL-II output (for ZEBRA printers)");
            System.Console.WriteLine();
            System.Console.Write("Please select:");
            cki = System.Console.ReadKey(true);
            System.Console.WriteLine();
            i = "123456789ABCDE".IndexOf(cki.KeyChar.ToString().ToUpper());
        } while (i < 0);

        oName = fName[i];
        return ot[i];
        }


        static void Main(string[] args)
        {
          TECIT.TFORMer.PrinterType ot;

          try
          {
            // create a Job instance
            Job job = new Job();

            // select the repository-based form-layout
#if WIN32
            // on Windows use this path
            System.IO.DirectoryInfo directory = new System.IO.DirectoryInfo(Environment.CurrentDirectory);
            job.RepositoryName = directory.FullName + "\\..\\..\\..\\..\\Demo Repository\\Demos.tfr";
#else
            // This path is used on Linux
            job.RepositoryName ="/usr/local/share/TFORMer/Demo Repository/Demos.tfr";
#endif
            job.ProjectName = "TFORMer_Runtime_Examples";
            job.FormName = "BusinessCard_Multi";

            // create an in-memory data-source
            JobDataRecordSet jobdata = new JobDataRecordSet();

            // provide the first record setting the data-field values for Email, JobTitle and Name
            // the first record triggers output of the first business-card
            job.JobData = jobdata;
            Record record = new Record();
            record.Data.Add("Email", "support@tec-it.com");
            record.Data.Add("JobTitle", "Support Engineer");
            record.Data.Add("Name", "Mr. Harald Backoffice");
            jobdata.Records.Add(record);

            // provide the second record setting the data-field values for Email, JobTitle and Name
            // this record triggers output of the second business-card
            record = new Record();
            record.Data.Add("Email", "sales@tec-it.com");
            record.Data.Add("JobTitle", "Sales Assistant");
            record.Data.Add("Name", "Mrs. Susan Frontdesk");
            jobdata.Records.Add(record);

            // Ask user for output type. File extension will be appended accordingly
            ot = OutputType();
            job.PrinterType = ot;
#if WIN32
            // on Windows use this path
            job.OutputName = "C:/temp/" + oName;
#else
            // This path is used on Linux
            job.OutputName = oName;
#endif //WIN32

            // generate output
            System.Console.Write("Printing...");
            job.Print();

            System.Console.WriteLine("Job (" + job.OutputName + ") printed successfully!");
          } catch (TECIT.TFORMer.TFORMerException tformerEx) {
            System.Console.WriteLine(tformerEx.ToString());
          } catch (Exception ex) {
            System.Console.WriteLine(ex.ToString());
          } finally {
            System.Console.WriteLine("Press any key to close this window ...");
            System.Console.ReadKey(true);
          }
        }
    }
}
