/*===========================================================================*/
/*                        Copyright (C) 2009 by                              */
/*                                                                           */
/*                    -- TEC-IT Datenverarbeitung GmbH --                    */
/*   -- team for engineering and consulting in information technologies --   */
/*                                                                           */
/*                           All rights reserved.                            */
/*                                                                           */
/*              This is a part of the TEC-IT Standard Software.              */
/*                                                                           */
/*    This source code is only intended as a supplement to the References    */
/*      and related electronic documentation provided with the product.      */
/*                                                                           */
/*     See these sources for detailed information regarding this product     */
/*===========================================================================*/

/*=============================================================================
   This sample application generates PDF output (based on the form named 
   BarcodeLabels in the project TFORMer_Runtime_Examples of the installed 
   Demo Repository)
   The data for the output is provided by a text-datasource, that will be 
   created via the NET Runtime.
   (see form in Demo Repository) 
   ----------------------------------------------------------------------------
   History:
   (2008-01-2) : created
=============================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using TECIT.TFORMer;

namespace ConsoleApplication1
{
  class Program
  {
    static void Main(string[] args)
    {
      try
      {
        System.IO.DirectoryInfo directory = new System.IO.DirectoryInfo(Environment.CurrentDirectory);

        Repository rep = new Repository(directory.FullName + "\\..\\..\\..\\..\\Demo Repository\\Demos.tfr", false, false);

        Project project = rep.GetProject("TFORMer_Runtime_Examples");

        if (project.GetDataSource("ArticleList") == null)
        {
          //create a csv-datasource to use our data
          DataSourceCsv datasource = (DataSourceCsv)project.AddDataSource(DataSourceType.CSV, "ArticleList");

          //modify properties of the datasource
          datasource.Filename = directory.FullName + "\\..\\..\\DataSource\\ArticleList.csv";
          datasource.Separator = ";";
          datasource.Qualifier = "\"";

          //create a fieldmapping for the datasource
          datasource.AddFieldMapping("ArticleNo").DataField = "ArticleNo";
          datasource.AddFieldMapping("ArticleName").DataField = "ArticleName";
          datasource.AddFieldMapping("ArticlePrice").DataField = "ArticlePrice";

          //save the repository for the printjob
          rep.Save();
        }
        //create a printjob
        Job job = new Job();

        //create a jobdata
        JobDataDataSource jobdata = new JobDataDataSource();

        //assign jobdata to the job
        job.JobData = jobdata;
        
        //select the demo repository
        job.RepositoryName = directory.FullName + "\\..\\..\\..\\..\\Demo Repository\\Demos.tfr";
        job.ProjectName = "TFORMer_Runtime_Examples";
        job.FormName = "BarcodeLabels";

        //select the created datasource
        jobdata.Name = "ArticleList";

        // A PDF-document named C:/temp/ArticleLabels.pdf should be generated
        job.NumberOfCopies = 1;
        job.PrinterType = PrinterType.PdfFile;
        job.OutputName = "C:\\TEMP\\ArticleLabels.pdf";

        job.Print();
      }
      catch (TECIT.TFORMer.TFORMerException tformerEx)
      {
        System.Console.WriteLine(tformerEx.ToString());
      }
      catch (Exception ex)
      {
        System.Console.WriteLine(ex.ToString());
      }
      finally
      {
        System.Console.WriteLine("Press any key to close this window ...");
        System.Console.ReadKey(true);
      }
    }
  }
}